{**********************************************************************
                Copyright (c) PilotLogic Software House

 Package pl_FFmpeg5
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****

***********************************************************************}

Unit libavdevice;

{$MODE objfpc}{$H+}

Interface

Uses
  ctypes,
  Classes, dynlibs,
  ffmpeg_types, libavutil, libavcodec, libavformat;

Const
{$IF defined(MSWINDOWS)}
   avdeviceLibName = 'avdevice-59.dll';
{$ELSEIF defined(LINUX)}
   avdeviceLibName = 'libavdevice.so.59';
{$ELSEIF defined(DARWIN)}
   avdeviceLibName = 'libavdevice.59.dylib';
{$ELSE}
   avdeviceLibName = 'libavdevice.so.59';
{$ENDIF}

Type
   pAVDeviceRect = ^AVDeviceRect;

   AVDeviceRect = Record
      x: int;
      y: int;
      width: int;
      height: int;
   End;

   AVAppToDevMessageType = Array[0..3] Of AnsiChar;  
   AVDevToAppMessageType = Array[0..3] Of AnsiChar;

Const

   AV_APP_TO_DEV_NONE: AVAppToDevMessageType = ('N', 'O', 'N', 'E');
   AV_APP_TO_DEV_WINDOW_SIZE: AVAppToDevMessageType = ('G', 'E', 'O', 'M');
   AV_APP_TO_DEV_WINDOW_REPAINT: AVAppToDevMessageType = ('R', 'E', 'P', 'A');
   AV_APP_TO_DEV_PAUSE: AVAppToDevMessageType = ('P', 'A', 'U', ' ');
   AV_APP_TO_DEV_PLAY: AVAppToDevMessageType = ('P', 'L', 'A', 'Y');
   AV_APP_TO_DEV_TOGGLE_PAUSE: AVAppToDevMessageType = ('P', 'A', 'U', 'T');
   AV_APP_TO_DEV_SET_VOLUME: AVAppToDevMessageType = ('S', 'V', 'O', 'L');
   AV_APP_TO_DEV_MUTE: AVAppToDevMessageType = (' ', 'M', 'U', 'T');
   AV_APP_TO_DEV_UNMUTE: AVAppToDevMessageType = ('U', 'M', 'U', 'T');
   AV_APP_TO_DEV_TOGGLE_MUTE: AVAppToDevMessageType = ('T', 'M', 'U', 'T');
   AV_APP_TO_DEV_GET_VOLUME: AVAppToDevMessageType = ('G', 'V', 'O', 'L');
   AV_APP_TO_DEV_GET_MUTE: AVAppToDevMessageType = ('G', 'M', 'U', 'T');

Const

   AV_DEV_TO_APP_NONE: AVDevToAppMessageType = ('N', 'O', 'N', 'E');
   AV_DEV_TO_APP_CREATE_WINDOW_BUFFER: AVDevToAppMessageType = ('B', 'C', 'R', 'E');
   AV_DEV_TO_APP_PREPARE_WINDOW_BUFFER: AVDevToAppMessageType = ('B', 'P', 'R', 'E');
   AV_DEV_TO_APP_DISPLAY_WINDOW_BUFFER: AVDevToAppMessageType = ('B', 'D', 'I', 'S');
   AV_DEV_TO_APP_DESTROY_WINDOW_BUFFER: AVDevToAppMessageType = ('B', 'D', 'E', 'S');
   AV_DEV_TO_APP_BUFFER_OVERFLOW: AVDevToAppMessageType = ('B', 'O', 'F', 'L');
   AV_DEV_TO_APP_BUFFER_UNDERFLOW: AVDevToAppMessageType = ('B', 'U', 'F', 'L');
   AV_DEV_TO_APP_BUFFER_READABLE: AVDevToAppMessageType = ('B', 'R', 'D', ' ');
   AV_DEV_TO_APP_BUFFER_WRITABLE: AVDevToAppMessageType = ('B', 'W', 'R', ' ');
   AV_DEV_TO_APP_MUTE_STATE_CHANGED: AVDevToAppMessageType = ('C', 'M', 'U', 'T');
   AV_DEV_TO_APP_VOLUME_LEVEL_CHANGED: AVDevToAppMessageType = ('C', 'V', 'O', 'L');

Type

   pAVDeviceCapabilitiesQuery = ^AVDeviceCapabilitiesQuery;
   AVDeviceCapabilitiesQuery = Record
      av_class: pAVClass;
      device_context: pavformatContext;
      codec: AVCodecID;
      sample_format: AVSampleFormat;
      pixel_format: AVPixelFormat;
      sample_rate: int;
      channels: int;
      channel_layout: int64_t;
      window_width: int;
      window_height: int;
      frame_width: int;
      frame_height: int;
      fps: AVRational;
   End;

   pAVDeviceInfo = ^AVDeviceInfo;
   ppAVDeviceInfo = ^pAVDeviceInfo;
   AVDeviceInfo = Record
      device_name: pAnsiChar;
      device_description: pAnsiChar;
   End;

   pAVDeviceInfoList = ^AVDeviceInfoList;
   AVDeviceInfoList = Record
      devices: ppAVDeviceInfo;
      nb_devices: int;
      default_device: int;
   End;
      
//-------------------------------------------------
Type

   Tavdevice_version = Function(): unsigned; cdecl;
   Tavdevice_configuration = Function(): pAnsiChar; cdecl;
   Tavdevice_license = Function(): pAnsiChar; cdecl;
   Tavdevice_register_all = Procedure(); cdecl;  
   Tav_device_ffversion = Function(): unsigned; cdecl;
   Tav_input_audio_device_next = Function(d: pAVInputFormat): pAVInputFormat; cdecl;
   Tav_input_video_device_next = Function(d: pAVInputFormat): pAVInputFormat; cdecl;
   Tav_output_audio_device_next = Function(d: pAVOutputFormat): pAVOutputFormat; cdecl;
   Tav_output_video_device_next = Function(d: pAVOutputFormat): pAVOutputFormat; cdecl;
   Tavdevice_app_to_dev_control_message = Function(s: pavformatContext; _type: AVAppToDevMessageType; data: Pointer; data_size: size_t): int; cdecl;
   Tavdevice_dev_to_app_control_message = Function(s: pavformatContext; _type: AVDevToAppMessageType; data: Pointer; data_size: size_t): int; cdecl;
   Tavdevice_list_devices = Function(s: pavformatContext; Var device_list: pAVDeviceInfoList): int; cdecl;
   Tavdevice_free_list_devices = Procedure(Var device_list: pAVDeviceInfoList); cdecl;
   Tavdevice_list_input_sources = Function(device: pAVInputFormat; Const device_name: pAnsiChar; device_options: pAVDictionary;
                                           Var device_list: pAVDeviceInfoList): int; cdecl;
   Tavdevice_list_output_sinks = Function(device: pAVOutputFormat; Const device_name: pAnsiChar; device_options: pAVDictionary;
                                          Var device_list: pAVDeviceInfoList): int; cdecl;

//-------------------------------------------------
 Var

   avdevice_version : Tavdevice_version = Nil;
   avdevice_configuration : Tavdevice_configuration = Nil;
   avdevice_license : Tavdevice_license = Nil;
   avdevice_register_all : Tavdevice_register_all = Nil; 
   av_device_ffversion : Tav_device_ffversion = Nil;
   av_input_audio_device_next : Tav_input_audio_device_next = Nil;
   av_input_video_device_next : Tav_input_video_device_next = Nil;
   av_output_audio_device_next : Tav_output_audio_device_next = Nil;
   av_output_video_device_next : Tav_output_video_device_next = Nil;
   avdevice_app_to_dev_control_message : Tavdevice_app_to_dev_control_message = Nil;
   avdevice_dev_to_app_control_message : Tavdevice_dev_to_app_control_message = Nil;
   avdevice_list_devices : Tavdevice_list_devices = Nil;
   avdevice_free_list_devices : Tavdevice_free_list_devices = Nil;
   avdevice_list_input_sources : Tavdevice_list_input_sources = Nil;
   avdevice_list_output_sinks : Tavdevice_list_output_sinks = Nil;

//-------------------------------------------------

Function  libavdevice_Load(Const aLibName: String = avdeviceLibName): Boolean;
Procedure libavdevice_Free;
Function  libavdevice_IsLoaded: boolean;

//==============================================================
//==============================================================
implementation

Const
   InvalidLibHandle = 0;

var
  avdevice_Handle: TLibHandle;

function libavdevice_IsLoaded: boolean;
begin
  Result := (avdevice_Handle <> InvalidLibHandle);
end;

Procedure libavdevice_Free;
begin
  if libavdevice_IsLoaded then
  begin
    DynLibs.UnloadLibrary(avdevice_Handle);
    avdevice_Handle := InvalidLibHandle;
  end;
end;

function libavdevice_Load(Const aLibName: String = avdeviceLibName): Boolean;
//.............................
   Function _GetProcAddress(Const aProcName: String): {$IFDEF cpui8086}FarPointer{$ELSE}Pointer{$ENDIF};
   Begin
      result := Nil;
      If aProcName = '' Then
         exit;

      result := dynlibs.GetProcedureAddress(avdevice_Handle, AnsiString(aProcName));
   End;
   //.....................................
var
  thelib: string;

{$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
  ssl:Integer;
  ssp:Integer;
{$ENDIF}
begin
  Result := False;

  if avdevice_Handle <> InvalidLibHandle then Exit(True);

  if aLibName = '' then
     thelib := avdeviceLibName else
     thelib := aLibName;

  avdevice_Handle := dynlibs.LoadLibrary(thelib);

   //.....................................
   // On UNIX If Library NOT load, try to load .so file
  {$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
   If avdevice_Handle = 0 Then
   Begin
      ssl := length(thelib);
      ssp := Pos('.so.', thelib);
      If ssp > 0 Then
         delete(thelib, ssp, ssl - ssp + 1);

      avdevice_Handle := dynlibs.LoadLibrary(thelib + '.so');
   End;
  {$ENDIF}

    if avdevice_Handle = InvalidLibHandle then Exit(False);

    //.........................................

    Pointer(avdevice_version) := _GetProcAddress('avdevice_version');
    Pointer(avdevice_configuration) := _GetProcAddress('avdevice_configuration');
    Pointer(avdevice_license) := _GetProcAddress('avdevice_license');
    Pointer(avdevice_register_all) := _GetProcAddress('avdevice_register_all');     
    Pointer(av_device_ffversion) := _GetProcAddress('av_device_ffversion');
    Pointer(av_input_audio_device_next) := _GetProcAddress('av_input_audio_device_next');
    Pointer(av_input_video_device_next) := _GetProcAddress('av_input_video_device_next');
    Pointer(av_output_audio_device_next) := _GetProcAddress('av_output_audio_device_next');
    Pointer(av_output_video_device_next) := _GetProcAddress('av_output_video_device_next');
    Pointer(avdevice_app_to_dev_control_message) := _GetProcAddress('avdevice_app_to_dev_control_message');
    Pointer(avdevice_dev_to_app_control_message) := _GetProcAddress('avdevice_dev_to_app_control_message');
    Pointer(avdevice_list_devices) := _GetProcAddress('avdevice_list_devices');
    Pointer(avdevice_free_list_devices) := _GetProcAddress('avdevice_free_list_devices');
    Pointer(avdevice_list_input_sources) := _GetProcAddress('avdevice_list_input_sources');
    Pointer(avdevice_list_output_sinks) := _GetProcAddress('avdevice_list_output_sinks');


    //.........................................

    Result := True;

end;

end.
