{**********************************************************************
                Copyright (c) PilotLogic Software House

 Package pl_FFmpeg5
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****

***********************************************************************}

Unit libavutil;

{$MODE objfpc}{$H+}

Interface

Uses
  ctypes,
  Classes, dynlibs,
  ffmpeg_types;

Const
{$IF defined(MSWINDOWS)}
   avutilLibName = 'avutil-57.dll';
{$ELSEIF defined(LINUX)}
   avutilLibName = 'libavutil.so.57';
{$ELSEIF defined(DARWIN)}
   avutilLibName = 'libavutil.57.dylib';
{$ELSE}
   avutilLibName = 'libavutil.so.57';
{$ENDIF}


Const
  AV_HAVE_BIGENDIAN = 0;
  AV_HAVE_FAST_UNALIGNED = 1;

Type
  FF_PAD_STRUCTURE_AVBPrint = Record
     str: PAnsiChar;
     len: Cardinal;
     size: Cardinal;
     size_max: Cardinal;
     reserved_internal_buffer: Array[0..0] Of AnsiChar;
  End;
  pAVBPrint = ^AVBPrint;
  AVBPrint = Record
     str: PAnsiChar;
     len: Cardinal;
     size: Cardinal;
     size_max: Cardinal;
     reserved_internal_buffer: Array[0..0] Of AnsiChar;
     reserved_padding: Array[0..1024 - SizeOf(FF_PAD_STRUCTURE_AVBPrint) - 1] Of AnsiChar;
  End;

Const
  AV_CH_FRONT_LEFT = $00000001;
  AV_CH_FRONT_RIGHT = $00000002;
  AV_CH_FRONT_CENTER = $00000004;
  AV_CH_LOW_FREQUENCY = $00000008;
  AV_CH_BACK_LEFT = $00000010;
  AV_CH_BACK_RIGHT = $00000020;
  AV_CH_FRONT_LEFT_OF_CENTER = $00000040;
  AV_CH_FRONT_RIGHT_OF_CENTER = $00000080;
  AV_CH_BACK_CENTER = $00000100;
  AV_CH_SIDE_LEFT = $00000200;
  AV_CH_SIDE_RIGHT = $00000400;
  AV_CH_TOP_CENTER = $00000800;
  AV_CH_TOP_FRONT_LEFT = $00001000;
  AV_CH_TOP_FRONT_CENTER = $00002000;
  AV_CH_TOP_FRONT_RIGHT = $00004000;
  AV_CH_TOP_BACK_LEFT = $00008000;
  AV_CH_TOP_BACK_CENTER = $00010000;
  AV_CH_TOP_BACK_RIGHT = $00020000;
  AV_CH_STEREO_LEFT = $20000000;
  AV_CH_STEREO_RIGHT = $40000000;
  AV_CH_WIDE_LEFT = $0000000080000000;
  AV_CH_WIDE_RIGHT = $0000000100000000;
  AV_CH_SURROUND_DIRECT_LEFT = $0000000200000000;
  AV_CH_SURROUND_DIRECT_RIGHT = $0000000400000000;
  AV_CH_LOW_FREQUENCY_2 = $0000000800000000;
  AV_CH_LAYOUT_NATIVE = $8000000000000000;
  AV_CH_LAYOUT_MONO = (AV_CH_FRONT_CENTER);
  AV_CH_LAYOUT_STEREO = (AV_CH_FRONT_LEFT Or AV_CH_FRONT_RIGHT);
  AV_CH_LAYOUT_2POINT1 = (AV_CH_LAYOUT_STEREO Or AV_CH_LOW_FREQUENCY);
  AV_CH_LAYOUT_2_1 = (AV_CH_LAYOUT_STEREO Or AV_CH_BACK_CENTER);
  AV_CH_LAYOUT_SURROUND = (AV_CH_LAYOUT_STEREO Or AV_CH_FRONT_CENTER);
  AV_CH_LAYOUT_3POINT1 = (AV_CH_LAYOUT_SURROUND Or AV_CH_LOW_FREQUENCY);
  AV_CH_LAYOUT_4POINT0 = (AV_CH_LAYOUT_SURROUND Or AV_CH_BACK_CENTER);
  AV_CH_LAYOUT_4POINT1 = (AV_CH_LAYOUT_4POINT0 Or AV_CH_LOW_FREQUENCY);
  AV_CH_LAYOUT_2_2 = (AV_CH_LAYOUT_STEREO Or AV_CH_SIDE_LEFT Or AV_CH_SIDE_RIGHT);
  AV_CH_LAYOUT_QUAD = (AV_CH_LAYOUT_STEREO Or AV_CH_BACK_LEFT Or AV_CH_BACK_RIGHT);
  AV_CH_LAYOUT_5POINT0 = (AV_CH_LAYOUT_SURROUND Or AV_CH_SIDE_LEFT Or AV_CH_SIDE_RIGHT);
  AV_CH_LAYOUT_5POINT1 = (AV_CH_LAYOUT_5POINT0 Or AV_CH_LOW_FREQUENCY);
  AV_CH_LAYOUT_5POINT0_BACK = (AV_CH_LAYOUT_SURROUND Or AV_CH_BACK_LEFT Or AV_CH_BACK_RIGHT);
  AV_CH_LAYOUT_5POINT1_BACK = (AV_CH_LAYOUT_5POINT0_BACK Or AV_CH_LOW_FREQUENCY);
  AV_CH_LAYOUT_6POINT0 = (AV_CH_LAYOUT_5POINT0 Or AV_CH_BACK_CENTER);
  AV_CH_LAYOUT_6POINT0_FRONT = (AV_CH_LAYOUT_2_2 Or AV_CH_FRONT_LEFT_OF_CENTER Or AV_CH_FRONT_RIGHT_OF_CENTER);
  AV_CH_LAYOUT_HEXAGONAL = (AV_CH_LAYOUT_5POINT0_BACK Or AV_CH_BACK_CENTER);
  AV_CH_LAYOUT_6POINT1 = (AV_CH_LAYOUT_5POINT1 Or AV_CH_BACK_CENTER);
  AV_CH_LAYOUT_6POINT1_BACK = (AV_CH_LAYOUT_5POINT1_BACK Or AV_CH_BACK_CENTER);
  AV_CH_LAYOUT_6POINT1_FRONT = (AV_CH_LAYOUT_6POINT0_FRONT Or AV_CH_LOW_FREQUENCY);
  AV_CH_LAYOUT_7POINT0 = (AV_CH_LAYOUT_5POINT0 Or AV_CH_BACK_LEFT Or AV_CH_BACK_RIGHT);
  AV_CH_LAYOUT_7POINT0_FRONT = (AV_CH_LAYOUT_5POINT0 Or AV_CH_FRONT_LEFT_OF_CENTER Or AV_CH_FRONT_RIGHT_OF_CENTER);
  AV_CH_LAYOUT_7POINT1 = (AV_CH_LAYOUT_5POINT1 Or AV_CH_BACK_LEFT Or AV_CH_BACK_RIGHT);
  AV_CH_LAYOUT_7POINT1_WIDE = (AV_CH_LAYOUT_5POINT1 Or AV_CH_FRONT_LEFT_OF_CENTER Or AV_CH_FRONT_RIGHT_OF_CENTER);
  AV_CH_LAYOUT_7POINT1_WIDE_BACK = (AV_CH_LAYOUT_5POINT1_BACK Or AV_CH_FRONT_LEFT_OF_CENTER Or AV_CH_FRONT_RIGHT_OF_CENTER);
  AV_CH_LAYOUT_OCTAGONAL = (AV_CH_LAYOUT_5POINT0 Or AV_CH_BACK_LEFT Or AV_CH_BACK_CENTER Or AV_CH_BACK_RIGHT);
  AV_CH_LAYOUT_HEXADECAGONAL = (AV_CH_LAYOUT_OCTAGONAL Or AV_CH_WIDE_LEFT Or AV_CH_WIDE_RIGHT Or AV_CH_TOP_BACK_LEFT Or AV_CH_TOP_BACK_RIGHT Or
                                AV_CH_TOP_BACK_CENTER Or AV_CH_TOP_FRONT_CENTER Or AV_CH_TOP_FRONT_LEFT Or AV_CH_TOP_FRONT_RIGHT);
  AV_CH_LAYOUT_STEREO_DOWNMIX = (AV_CH_STEREO_LEFT Or AV_CH_STEREO_RIGHT);

Type
  AVMatrixEncoding = (
     AV_MATRIX_ENCODING_NONE, AV_MATRIX_ENCODING_DOLBY, AV_MATRIX_ENCODING_DPLII, AV_MATRIX_ENCODING_DPLIIX, AV_MATRIX_ENCODING_DPLIIZ,
     AV_MATRIX_ENCODING_DOLBYEX, AV_MATRIX_ENCODING_DOLBYHEADPHONE, AV_MATRIX_ENCODING_NB);
Const
  AV_DICT_MATCH_CASE = 1;
  AV_DICT_IGNORE_SUFFIX = 2;
  AV_DICT_DONT_STRDUP_KEY = 4;
  AV_DICT_DONT_STRDUP_VAL = 8;
  AV_DICT_DONT_OVERWRITE = 16;
  AV_DICT_APPEND = 32;
  AV_DICT_MULTIKEY = 64;

Type
  AVDictionaryEntry = Record
     key: PAnsiChar;
     value: PAnsiChar;
  End;
  pAVDictionaryEntry = ^AVDictionaryEntry;
  AVDictionary = Record
  End;
  pAVDictionary = ^AVDictionary;
  ppAVDictionary = ^pAVDictionary;
Type
  AVBuffer = Record
  End;
  pAVBuffer = ^AVBuffer;
  AVBufferRef = Record
     buffer: pAVBuffer;
     data: puint8_t;
     size: int;
  End;
  pAVBufferRef = ^AVBufferRef;
  ppAVBufferRef = ^pAVBufferRef;
Const
  AV_BUFFER_FLAG_READONLY = (1 Shl 0);

Type
  TFreeProc = Procedure(opaque: Pointer; data: puint8_t); cdecl;

  AVBufferPool = Record
  End;
  pAVBufferPool = ^AVBufferPool;

  Tbuffer_pool_init_proc = Function(size: int): pAVBufferRef; cdecl;

  Tav_buffer_pool_init2_alloc_proc = Function(opaque: Pointer; size: int): pAVBufferRef; cdecl;
  Tav_buffer_pool_init2_pool_free_proc = Procedure(opaque: Pointer); cdecl;

  AVRational = Record
     num: int;
     den: int;
  End;
  pAVRational = ^AVRational;
  TAVRational = AVRational;

Type
  AVMediaType = (
     AVMEDIA_TYPE_UNKNOWN = -1,
     AVMEDIA_TYPE_VIDEO = 0,
     AVMEDIA_TYPE_AUDIO = 1,
     AVMEDIA_TYPE_DATA = 2,
     AVMEDIA_TYPE_SUBTITLE = 3,
     AVMEDIA_TYPE_ATTACHMENT = 4,
     AVMEDIA_TYPE_NB = 5
     );
  TAVMediaType =  AVMediaType;

Const
  FF_LAMBDA_SHIFT = 7;
  FF_LAMBDA_SCALE = (1 Shl FF_LAMBDA_SHIFT);
  FF_QP2LAMBDA = 118;
  FF_LAMBDA_MAX = (256 * 128 - 1);
  FF_QUALITY_SCALE = FF_LAMBDA_SCALE;
  AV_NOPTS_VALUE = int64_t($8000000000000000);
  AV_TIME_BASE = 1000000;
  AV_TIME_BASE_Q: AVRational = (num: 1; den: AV_TIME_BASE);
Type
  AVPictureType = (
     AV_PICTURE_TYPE_NONE = 0,
     AV_PICTURE_TYPE_I = 1,
     AV_PICTURE_TYPE_P = 2,
     AV_PICTURE_TYPE_B = 3,
     AV_PICTURE_TYPE_S = 4,
     AV_PICTURE_TYPE_SI = 5,
     AV_PICTURE_TYPE_SP = 6,
     AV_PICTURE_TYPE_BI = 7
     );
Type
  pAVPixelFormat = ^AVPixelFormat;
  AVPixelFormat = (
     AV_PIX_FMT_NONE = -1,
     AV_PIX_FMT_YUV420P,
     AV_PIX_FMT_YUYV422,
     AV_PIX_FMT_RGB24,
     AV_PIX_FMT_BGR24,
     AV_PIX_FMT_YUV422P,
     AV_PIX_FMT_YUV444P,
     AV_PIX_FMT_YUV410P,
     AV_PIX_FMT_YUV411P,
     AV_PIX_FMT_GRAY8,
     AV_PIX_FMT_MONOWHITE,
     AV_PIX_FMT_MONOBLACK,
     AV_PIX_FMT_PAL8,
     AV_PIX_FMT_YUVJ420P,
     AV_PIX_FMT_YUVJ422P,
     AV_PIX_FMT_YUVJ444P,
     AV_PIX_FMT_UYVY422,
     AV_PIX_FMT_UYYVYY411,
     AV_PIX_FMT_BGR8,
     AV_PIX_FMT_BGR4,
     AV_PIX_FMT_BGR4_BYTE,
     AV_PIX_FMT_RGB8,
     AV_PIX_FMT_RGB4,
     AV_PIX_FMT_RGB4_BYTE,
     AV_PIX_FMT_NV12,
     AV_PIX_FMT_NV21,
     AV_PIX_FMT_ARGB,
     AV_PIX_FMT_RGBA,
     AV_PIX_FMT_ABGR,
     AV_PIX_FMT_BGRA,
     AV_PIX_FMT_GRAY16BE,
     AV_PIX_FMT_GRAY16LE,
     AV_PIX_FMT_YUV440P,
     AV_PIX_FMT_YUVJ440P,
     AV_PIX_FMT_YUVA420P,
     AV_PIX_FMT_RGB48BE,
     AV_PIX_FMT_RGB48LE,
     AV_PIX_FMT_RGB565BE,
     AV_PIX_FMT_RGB565LE,
     AV_PIX_FMT_RGB555BE,
     AV_PIX_FMT_RGB555LE,
     AV_PIX_FMT_BGR565BE,
     AV_PIX_FMT_BGR565LE,
     AV_PIX_FMT_BGR555BE,
     AV_PIX_FMT_BGR555LE,
     AV_PIX_FMT_VAAPI_MOCO,
     AV_PIX_FMT_VAAPI_IDCT,
     AV_PIX_FMT_VAAPI_VLD,
     AV_PIX_FMT_VAAPI = AV_PIX_FMT_VAAPI_VLD,
     AV_PIX_FMT_YUV420P16LE,
     AV_PIX_FMT_YUV420P16BE,
     AV_PIX_FMT_YUV422P16LE,
     AV_PIX_FMT_YUV422P16BE,
     AV_PIX_FMT_YUV444P16LE,
     AV_PIX_FMT_YUV444P16BE,
     AV_PIX_FMT_DXVA2_VLD,
     AV_PIX_FMT_RGB444LE,
     AV_PIX_FMT_RGB444BE,
     AV_PIX_FMT_BGR444LE,
     AV_PIX_FMT_BGR444BE,
     AV_PIX_FMT_YA8,
     AV_PIX_FMT_Y400A = AV_PIX_FMT_YA8,
     AV_PIX_FMT_GRAY8A = AV_PIX_FMT_YA8,
     AV_PIX_FMT_BGR48BE,
     AV_PIX_FMT_BGR48LE,
     AV_PIX_FMT_YUV420P9BE,
     AV_PIX_FMT_YUV420P9LE,
     AV_PIX_FMT_YUV420P10BE,
     AV_PIX_FMT_YUV420P10LE,
     AV_PIX_FMT_YUV422P10BE,
     AV_PIX_FMT_YUV422P10LE,
     AV_PIX_FMT_YUV444P9BE,
     AV_PIX_FMT_YUV444P9LE,
     AV_PIX_FMT_YUV444P10BE,
     AV_PIX_FMT_YUV444P10LE,
     AV_PIX_FMT_YUV422P9BE,
     AV_PIX_FMT_YUV422P9LE,
     AV_PIX_FMT_VDA_VLD,
     AV_PIX_FMT_GBRP,
     AV_PIX_FMT_GBR24P = AV_PIX_FMT_GBRP,
     AV_PIX_FMT_GBRP9BE,
     AV_PIX_FMT_GBRP9LE,
     AV_PIX_FMT_GBRP10BE,
     AV_PIX_FMT_GBRP10LE,
     AV_PIX_FMT_GBRP16BE,
     AV_PIX_FMT_GBRP16LE,
     AV_PIX_FMT_YUVA422P,
     AV_PIX_FMT_YUVA444P,
     AV_PIX_FMT_YUVA420P9BE,
     AV_PIX_FMT_YUVA420P9LE,
     AV_PIX_FMT_YUVA422P9BE,
     AV_PIX_FMT_YUVA422P9LE,
     AV_PIX_FMT_YUVA444P9BE,
     AV_PIX_FMT_YUVA444P9LE,
     AV_PIX_FMT_YUVA420P10BE,
     AV_PIX_FMT_YUVA420P10LE,
     AV_PIX_FMT_YUVA422P10BE,
     AV_PIX_FMT_YUVA422P10LE,
     AV_PIX_FMT_YUVA444P10BE,
     AV_PIX_FMT_YUVA444P10LE,
     AV_PIX_FMT_YUVA420P16BE,
     AV_PIX_FMT_YUVA420P16LE,
     AV_PIX_FMT_YUVA422P16BE,
     AV_PIX_FMT_YUVA422P16LE,
     AV_PIX_FMT_YUVA444P16BE,
     AV_PIX_FMT_YUVA444P16LE,
     AV_PIX_FMT_VDPAU,
     AV_PIX_FMT_XYZ12LE,
     AV_PIX_FMT_XYZ12BE,
     AV_PIX_FMT_NV16,
     AV_PIX_FMT_NV20LE,
     AV_PIX_FMT_NV20BE,
     AV_PIX_FMT_RGBA64BE,
     AV_PIX_FMT_RGBA64LE,
     AV_PIX_FMT_BGRA64BE,
     AV_PIX_FMT_BGRA64LE,
     AV_PIX_FMT_YVYU422,
     AV_PIX_FMT_VDA,
     AV_PIX_FMT_YA16BE,
     AV_PIX_FMT_YA16LE,
     AV_PIX_FMT_GBRAP,
     AV_PIX_FMT_GBRAP16BE,
     AV_PIX_FMT_GBRAP16LE,
     AV_PIX_FMT_QSV,
     AV_PIX_FMT_MMAL,
     AV_PIX_FMT_D3D11VA_VLD,
     AV_PIX_FMT_CUDA, AV_PIX_FMT_0RGB = $123 + 4,
     AV_PIX_FMT_RGB0,
     AV_PIX_FMT_0BGR,
     AV_PIX_FMT_BGR0,
     AV_PIX_FMT_YUV420P12BE,
     AV_PIX_FMT_YUV420P12LE,
     AV_PIX_FMT_YUV420P14BE,
     AV_PIX_FMT_YUV420P14LE,
     AV_PIX_FMT_YUV422P12BE,
     AV_PIX_FMT_YUV422P12LE,
     AV_PIX_FMT_YUV422P14BE,
     AV_PIX_FMT_YUV422P14LE,
     AV_PIX_FMT_YUV444P12BE,
     AV_PIX_FMT_YUV444P12LE,
     AV_PIX_FMT_YUV444P14BE,
     AV_PIX_FMT_YUV444P14LE,
     AV_PIX_FMT_GBRP12BE,
     AV_PIX_FMT_GBRP12LE,
     AV_PIX_FMT_GBRP14BE,
     AV_PIX_FMT_GBRP14LE,
     AV_PIX_FMT_YUVJ411P,
     AV_PIX_FMT_BAYER_BGGR8,
     AV_PIX_FMT_BAYER_RGGB8,
     AV_PIX_FMT_BAYER_GBRG8,
     AV_PIX_FMT_BAYER_GRBG8,
     AV_PIX_FMT_BAYER_BGGR16LE,
     AV_PIX_FMT_BAYER_BGGR16BE,
     AV_PIX_FMT_BAYER_RGGB16LE,
     AV_PIX_FMT_BAYER_RGGB16BE,
     AV_PIX_FMT_BAYER_GBRG16LE,
     AV_PIX_FMT_BAYER_GBRG16BE,
     AV_PIX_FMT_BAYER_GRBG16LE,
     AV_PIX_FMT_BAYER_GRBG16BE,
     AV_PIX_FMT_XVMC,
     AV_PIX_FMT_YUV440P10LE,
     AV_PIX_FMT_YUV440P10BE,
     AV_PIX_FMT_YUV440P12LE,
     AV_PIX_FMT_YUV440P12BE,
     AV_PIX_FMT_AYUV64LE,
     AV_PIX_FMT_AYUV64BE,
     AV_PIX_FMT_VIDEOTOOLBOX,
     AV_PIX_FMT_P010LE,
     AV_PIX_FMT_P010BE,
     AV_PIX_FMT_GBRAP12BE,
     AV_PIX_FMT_GBRAP12LE,
     AV_PIX_FMT_GBRAP10BE,
     AV_PIX_FMT_GBRAP10LE,
     AV_PIX_FMT_MEDIACODEC,
     AV_PIX_FMT_GRAY12BE,
     AV_PIX_FMT_GRAY12LE,
     AV_PIX_FMT_GRAY10BE,
     AV_PIX_FMT_GRAY10LE,
     AV_PIX_FMT_P016LE,
     AV_PIX_FMT_P016BE,
     AV_PIX_FMT_D3D11,
     AV_PIX_FMT_GRAY9BE,
     AV_PIX_FMT_GRAY9LE,
     AV_PIX_FMT_GBRPF32BE,
     AV_PIX_FMT_GBRPF32LE,
     AV_PIX_FMT_GBRAPF32BE,
     AV_PIX_FMT_GBRAPF32LE,
     AV_PIX_FMT_DRM_PRIME,
     AV_PIX_FMT_OPENCL,
     AV_PIX_FMT_GRAY14BE,
     AV_PIX_FMT_GRAY14LE,
     AV_PIX_FMT_GRAYF32BE,
     AV_PIX_FMT_GRAYF32LE,
     AV_PIX_FMT_YUVA422P12BE,
     AV_PIX_FMT_YUVA422P12LE,
     AV_PIX_FMT_YUVA444P12BE,
     AV_PIX_FMT_YUVA444P12LE,
     AV_PIX_FMT_NV24,
     AV_PIX_FMT_NV42,
     AV_PIX_FMT_NB
     ); 
   TAVPixelFormat = AVPixelFormat;

Const
  AVPALETTE_SIZE = 1024;
  AVPALETTE_COUNT = 256;
  AV_PIX_FMT_RGB32: AVPixelFormat = AV_PIX_FMT_BGRA;
  AV_PIX_FMT_RGB32_1: AVPixelFormat = AV_PIX_FMT_ABGR;
  AV_PIX_FMT_BGR32: AVPixelFormat = AV_PIX_FMT_RGBA;
  AV_PIX_FMT_BGR32_1: AVPixelFormat = AV_PIX_FMT_ARGB;
  AV_PIX_FMT_0RGB32: AVPixelFormat = AV_PIX_FMT_BGR0;
  AV_PIX_FMT_0BGR32: AVPixelFormat = AV_PIX_FMT_RGB0;
  AV_PIX_FMT_GRAY10: AVPixelFormat = AV_PIX_FMT_GRAY10LE;
  AV_PIX_FMT_GRAY12: AVPixelFormat = AV_PIX_FMT_GRAY12LE;
  AV_PIX_FMT_GRAY16: AVPixelFormat = AV_PIX_FMT_GRAY16LE;
  AV_PIX_FMT_YA16: AVPixelFormat = AV_PIX_FMT_YA16LE;
  AV_PIX_FMT_RGB48: AVPixelFormat = AV_PIX_FMT_RGB48LE;
  AV_PIX_FMT_RGB565: AVPixelFormat = AV_PIX_FMT_RGB565LE;
  AV_PIX_FMT_RGB555: AVPixelFormat = AV_PIX_FMT_RGB555LE;
  AV_PIX_FMT_RGB444: AVPixelFormat = AV_PIX_FMT_RGB444LE;
  AV_PIX_FMT_RGBA64: AVPixelFormat = AV_PIX_FMT_RGBA64LE;
  AV_PIX_FMT_BGR48: AVPixelFormat = AV_PIX_FMT_BGR48LE;
  AV_PIX_FMT_BGR565: AVPixelFormat = AV_PIX_FMT_BGR565LE;
  AV_PIX_FMT_BGR555: AVPixelFormat = AV_PIX_FMT_BGR555LE;
  AV_PIX_FMT_BGR444: AVPixelFormat = AV_PIX_FMT_BGR444LE;
  AV_PIX_FMT_BGRA64: AVPixelFormat = AV_PIX_FMT_BGRA64LE;
  AV_PIX_FMT_YUV420P9: AVPixelFormat = AV_PIX_FMT_YUV420P9LE;
  AV_PIX_FMT_YUV422P9: AVPixelFormat = AV_PIX_FMT_YUV422P9LE;
  AV_PIX_FMT_YUV444P9: AVPixelFormat = AV_PIX_FMT_YUV444P9LE;
  AV_PIX_FMT_YUV420P10: AVPixelFormat = AV_PIX_FMT_YUV420P10LE;
  AV_PIX_FMT_YUV422P10: AVPixelFormat = AV_PIX_FMT_YUV422P10LE;
  AV_PIX_FMT_YUV440P10: AVPixelFormat = AV_PIX_FMT_YUV440P10LE;
  AV_PIX_FMT_YUV444P10: AVPixelFormat = AV_PIX_FMT_YUV444P10LE;
  AV_PIX_FMT_YUV420P12: AVPixelFormat = AV_PIX_FMT_YUV420P12LE;
  AV_PIX_FMT_YUV422P12: AVPixelFormat = AV_PIX_FMT_YUV422P12LE;
  AV_PIX_FMT_YUV440P12: AVPixelFormat = AV_PIX_FMT_YUV440P12LE;
  AV_PIX_FMT_YUV444P12: AVPixelFormat = AV_PIX_FMT_YUV444P12LE;
  AV_PIX_FMT_YUV420P14: AVPixelFormat = AV_PIX_FMT_YUV420P14LE;
  AV_PIX_FMT_YUV422P14: AVPixelFormat = AV_PIX_FMT_YUV422P14LE;
  AV_PIX_FMT_YUV444P14: AVPixelFormat = AV_PIX_FMT_YUV444P14LE;
  AV_PIX_FMT_YUV420P16: AVPixelFormat = AV_PIX_FMT_YUV420P16LE;
  AV_PIX_FMT_YUV422P16: AVPixelFormat = AV_PIX_FMT_YUV422P16LE;
  AV_PIX_FMT_YUV444P16: AVPixelFormat = AV_PIX_FMT_YUV444P16LE;
  AV_PIX_FMT_GBRP9: AVPixelFormat = AV_PIX_FMT_GBRP9LE;
  AV_PIX_FMT_GBRP10: AVPixelFormat = AV_PIX_FMT_GBRP10LE;
  AV_PIX_FMT_GBRP12: AVPixelFormat = AV_PIX_FMT_GBRP12LE;
  AV_PIX_FMT_GBRP14: AVPixelFormat = AV_PIX_FMT_GBRP14LE;
  AV_PIX_FMT_GBRP16: AVPixelFormat = AV_PIX_FMT_GBRP16LE;
  AV_PIX_FMT_GBRAP10: AVPixelFormat = AV_PIX_FMT_GBRAP10LE;
  AV_PIX_FMT_GBRAP12: AVPixelFormat = AV_PIX_FMT_GBRAP12LE;
  AV_PIX_FMT_GBRAP16: AVPixelFormat = AV_PIX_FMT_GBRAP16LE;
  AV_PIX_FMT_BAYER_BGGR16: AVPixelFormat = AV_PIX_FMT_BAYER_BGGR16LE;
  AV_PIX_FMT_BAYER_RGGB16: AVPixelFormat = AV_PIX_FMT_BAYER_RGGB16LE;
  AV_PIX_FMT_BAYER_GBRG16: AVPixelFormat = AV_PIX_FMT_BAYER_GBRG16LE;
  AV_PIX_FMT_BAYER_GRBG16: AVPixelFormat = AV_PIX_FMT_BAYER_GRBG16LE;
  AV_PIX_FMT_YUVA420P9: AVPixelFormat = AV_PIX_FMT_YUVA420P9LE;
  AV_PIX_FMT_YUVA422P9: AVPixelFormat = AV_PIX_FMT_YUVA422P9LE;
  AV_PIX_FMT_YUVA444P9: AVPixelFormat = AV_PIX_FMT_YUVA444P9LE;
  AV_PIX_FMT_YUVA420P10: AVPixelFormat = AV_PIX_FMT_YUVA420P10LE;
  AV_PIX_FMT_YUVA422P10: AVPixelFormat = AV_PIX_FMT_YUVA422P10LE;
  AV_PIX_FMT_YUVA444P10: AVPixelFormat = AV_PIX_FMT_YUVA444P10LE;
  AV_PIX_FMT_YUVA422P12: AVPixelFormat = AV_PIX_FMT_YUVA422P12LE;
  AV_PIX_FMT_YUVA444P12: AVPixelFormat = AV_PIX_FMT_YUVA444P12LE;
  AV_PIX_FMT_YUVA420P16: AVPixelFormat = AV_PIX_FMT_YUVA420P16LE;
  AV_PIX_FMT_YUVA422P16: AVPixelFormat = AV_PIX_FMT_YUVA422P16LE;
  AV_PIX_FMT_YUVA444P16: AVPixelFormat = AV_PIX_FMT_YUVA444P16LE;
  AV_PIX_FMT_XYZ12: AVPixelFormat = AV_PIX_FMT_XYZ12LE;
  AV_PIX_FMT_NV20: AVPixelFormat = AV_PIX_FMT_NV20LE;
  AV_PIX_FMT_AYUV64: AVPixelFormat = AV_PIX_FMT_AYUV64LE;
  AV_PIX_FMT_P010: AVPixelFormat = AV_PIX_FMT_P010LE;
  AV_PIX_FMT_P016: AVPixelFormat = AV_PIX_FMT_P016LE;
Type
  AVColorPrimaries = (
     AVCOL_PRI_RESERVED0 = 0, AVCOL_PRI_BT709 = 1,
     AVCOL_PRI_UNSPECIFIED = 2, AVCOL_PRI_RESERVED = 3, AVCOL_PRI_BT470M = 4,
     AVCOL_PRI_BT470BG = 5,
     AVCOL_PRI_SMPTE170M = 6,
     AVCOL_PRI_SMPTE240M = 7,
     AVCOL_PRI_FILM = 8,
     AVCOL_PRI_BT2020 = 9,
     AVCOL_PRI_SMPTE428 = 10,
     AVCOL_PRI_SMPTEST428_1 = AVCOL_PRI_SMPTE428,
     AVCOL_PRI_SMPTE431 = 11,
     AVCOL_PRI_SMPTE432 = 12,
     AVCOL_PRI_JEDEC_P22 = 22,
     AVCOL_PRI_NB
     );
  AVColorTransferCharacteristic = (
     AVCOL_TRC_RESERVED0 = 0, AVCOL_TRC_BT709 = 1,
     AVCOL_TRC_UNSPECIFIED = 2,
     AVCOL_TRC_RESERVED = 3,
     AVCOL_TRC_GAMMA22 = 4,
     AVCOL_TRC_GAMMA28 = 5,
     AVCOL_TRC_SMPTE170M = 6,
     AVCOL_TRC_SMPTE240M = 7,
     AVCOL_TRC_LINEAR = 8,
     AVCOL_TRC_LOG = 9,
     AVCOL_TRC_LOG_SQRT = 10,
     AVCOL_TRC_IEC61966_2_4 = 11,
     AVCOL_TRC_BT1361_ECG = 12,
     AVCOL_TRC_IEC61966_2_1 = 13,
     AVCOL_TRC_BT2020_10 = 14,
     AVCOL_TRC_BT2020_12 = 15,
     AVCOL_TRC_SMPTE2084 = 16,
     AVCOL_TRC_SMPTEST2084 = AVCOL_TRC_SMPTE2084,
     AVCOL_TRC_SMPTE428 = 17,
     AVCOL_TRC_SMPTEST428_1 = AVCOL_TRC_SMPTE428,
     AVCOL_TRC_ARIB_STD_B67 = 18,
     AVCOL_TRC_NB
     );
  AVColorSpace = (
     AVCOL_SPC_RGB = 0,
     AVCOL_SPC_BT709 = 1,
     AVCOL_SPC_UNSPECIFIED = 2, AVCOL_SPC_RESERVED = 3, AVCOL_SPC_FCC = 4,
     AVCOL_SPC_BT470BG = 5,
     AVCOL_SPC_SMPTE170M = 6,
     AVCOL_SPC_SMPTE240M = 7,
     AVCOL_SPC_YCGCO = 8,
     AVCOL_SPC_YCOCG = AVCOL_SPC_YCGCO, AVCOL_SPC_BT2020_NCL = 9,
     AVCOL_SPC_BT2020_CL = 10,
     AVCOL_SPC_SMPTE2085 = 11,
     AVCOL_SPC_NB
     );
  AVColorRange = (
     AVCOL_RANGE_UNSPECIFIED = 0, AVCOL_RANGE_MPEG = 1,
     AVCOL_RANGE_JPEG = 2,
     AVCOL_RANGE_NB
     );
  AVChromaLocation = (
     AVCHROMA_LOC_UNSPECIFIED = 0, AVCHROMA_LOC_LEFT = 1,
     AVCHROMA_LOC_CENTER = 2,
     AVCHROMA_LOC_TOPLEFT = 3,
     AVCHROMA_LOC_TOP = 4, AVCHROMA_LOC_BOTTOMLEFT = 5, AVCHROMA_LOC_BOTTOM = 6, AVCHROMA_LOC_NB
     );
Const
  AV_NUM_DATA_POINTERS = 8;
Type
  TAVNDPArray = Array[0..AV_NUM_DATA_POINTERS - 1] Of int;
  pAVNDPArray = ^TAVNDPArray;
  TAVNDPArray_int = TAVNDPArray;
  pAVNDPArray_int = ^TAVNDPArray_int;
  TAVNDPArray_puint8_t = Array[0..AV_NUM_DATA_POINTERS - 1] Of puint8_t;
  pAVNDPArray_puint8_t = ^TAVNDPArray_puint8_t;
  TAVNDPArray_uint64_t = Array[0..AV_NUM_DATA_POINTERS - 1] Of uint64_t;
  TAVNDPArray_pAVBufferRef = Array[0..AV_NUM_DATA_POINTERS - 1] Of pAVBufferRef;
  pAVNDPArray_pAVBufferRef = ^TAVNDPArray_pAVBufferRef;
  Tuint8_t_array_4 = Array[0..3] Of uint8_t;
  puint8_t_array_4 = ^Tuint8_t_array_4;
  Tint_array_4 = Array[0..3] Of int;
  AVFrameSideDataType = (
     AV_FRAME_DATA_PANSCAN,
     AV_FRAME_DATA_A53_CC,
     AV_FRAME_DATA_STEREO3D,
     AV_FRAME_DATA_MATRIXENCODING,
     AV_FRAME_DATA_DOWNMIX_INFO,
     AV_FRAME_DATA_REPLAYGAIN,
     AV_FRAME_DATA_DISPLAYMATRIX,
     AV_FRAME_DATA_AFD,
     AV_FRAME_DATA_MOTION_VECTORS,
     AV_FRAME_DATA_SKIP_SAMPLES,
     AV_FRAME_DATA_AUDIO_SERVICE_TYPE,
     AV_FRAME_DATA_MASTERING_DISPLAY_METADATA,
     AV_FRAME_DATA_GOP_TIMECODE,
     AV_FRAME_DATA_SPHERICAL,
     AV_FRAME_DATA_CONTENT_LIGHT_LEVEL,
     AV_FRAME_DATA_ICC_PROFILE,
     AV_FRAME_DATA_QP_TABLE_PROPERTIES,
     AV_FRAME_DATA_QP_TABLE_DATA,
     AV_FRAME_DATA_S12M_TIMECODE,
     AV_FRAME_DATA_DYNAMIC_HDR_PLUS,
     AV_FRAME_DATA_REGIONS_OF_INTEREST
     );
  AVActiveFormatDescription = (
     AV_AFD_SAME = 8, AV_AFD_4_3 = 9, AV_AFD_16_9 = 10, AV_AFD_14_9 = 11, AV_AFD_4_3_SP_14_9 = 13, AV_AFD_16_9_SP_14_9 = 14, AV_AFD_SP_4_3 = 15);
  AVFrameSideData = Record
     _type: AVFrameSideDataType;
     data: puint8_t;
     size: int;
     metadata: pAVDictionary;
     buf: pAVBufferRef;
  End;
  pAVFrameSideData = ^AVFrameSideData;
  ppAVFrameSideData = ^pAVFrameSideData;
  AVRegionOfInterest = Record
     self_size: uint32_t;
     top: int;
     bottom: int;
     left: int;
     right: int;
     qoffset: AVRational;
  End;
  pAVRegionOfInterest = ^AVRegionOfInterest;
Const
  AV_FRAME_FLAG_CORRUPT = (1 Shl 0);
  AV_FRAME_FLAG_DISCARD = (1 Shl 2);
  FF_DECODE_ERROR_INVALID_BITSTREAM = 1;
  FF_DECODE_ERROR_MISSING_REFERENCE = 2;
  FF_DECODE_ERROR_CONCEALMENT_ACTIVE = 4;
  FF_DECODE_ERROR_DECODE_SLICES = 8;

Type               
  ppAVFrame = ^pAVFrame;
  pAVFrame = ^AVFrame;
  AVFrame = Record
     data: TAVNDPArray_puint8_t;
     linesize: TAVNDPArray_int;
     extended_data: ppuint8_t;
     width, height: int;
     nb_samples: int;
     format: int;
     key_frame: int;
     pict_type: AVPictureType;
     sample_aspect_ratio: AVRational;
     pts: int64_t;
     pkt_pts: int64_t;
     pkt_dts: int64_t;
     coded_picture_number: int;
     display_picture_number: int;
     quality: int;
     opaque: Pointer;
     error: TAVNDPArray_puint8_t;
     repeat_pict: int;
     interlaced_frame: int;
     top_field_first: int;
     palette_has_changed: int;
     reordered_opaque: int64_t;
     sample_rate: int;
     channel_layout: uint64_t;
     buf: TAVNDPArray_pAVBufferRef;
     extended_buf: ppAVBufferRef;
     nb_extended_buf: int;
     side_data: ppAVFrameSideData;
     nb_side_data: int;
     flags: int;
     color_range: AVColorRange;
     color_primaries: AVColorPrimaries;
     color_trc: AVColorTransferCharacteristic;
     colorspace: AVColorSpace;
     chroma_location: AVChromaLocation;
     best_effort_timestamp: int64_t;
     pkt_pos: int64_t;
     pkt_duration: int64_t;
     metadata: pAVDictionary;
     decode_error_flags: int;
     channels: int;
     pkt_size: int;
     qscale_table: pint8_t;
     qstride: int;
     qscale_type: int;
     qp_table_buf: pAVBufferRef;
     hw_frames_ctx: pAVBufferRef;
     opaque_ref: pAVBufferRef;
     crop_top: size_t;
     crop_bottom: size_t;
     crop_left: size_t;
     crop_right: size_t;
     private_ref: pAVBufferRef;
  End;
  TAVFrame = AVFrame;

Const
  AV_FRAME_CROP_UNALIGNED = 1 Shl 0;
Type
  pFFFrameBucket = ^FFFrameBucket;
  FFFrameBucket = Record
     frame: pAVFrame;
  End;

  pFFFrameQueueGlobal = ^FFFrameQueueGlobal;
  FFFrameQueueGlobal = Record
     dummy: AnsiChar;
  End;

  pFFFrameQueue = ^FFFrameQueue;
  FFFrameQueue = Record
     queue: pFFFrameBucket;
     allocated: size_t;
     tail: size_t;
     queued: size_t;
     first_bucket: FFFrameBucket;
     total_frames_head: uint64_t;
     total_frames_tail: uint64_t;
     total_samples_head: uint64_t;
     total_samples_tail: uint64_t;
     samples_skipped: int;
  End;

  AVOptionType = (
     AV_OPT_TYPE_FLAGS, AV_OPT_TYPE_INT, AV_OPT_TYPE_INT64, AV_OPT_TYPE_DOUBLE, AV_OPT_TYPE_FLOAT, AV_OPT_TYPE_STRING, AV_OPT_TYPE_RATIONAL, AV_OPT_TYPE_BINARY,
     AV_OPT_TYPE_DICT, AV_OPT_TYPE_UINT64, AV_OPT_TYPE_CONST, AV_OPT_TYPE_IMAGE_SIZE,
     AV_OPT_TYPE_PIXEL_FMT, AV_OPT_TYPE_SAMPLE_FMT, AV_OPT_TYPE_VIDEO_RATE,
     AV_OPT_TYPE_DURATION, AV_OPT_TYPE_COLOR, AV_OPT_TYPE_CHANNEL_LAYOUT, AV_OPT_TYPE_BOOL);
  Tdefault_val = Record
     Case int Of
        0:
        (i64: int64_t);
        1:
        (dbl: double);
        2:
        (str: PAnsiChar);
        3:
        (q: AVRational);
  End;

  AVOption = Record
     name: PAnsiChar;
     help: PAnsiChar;
     offset: int;
     _type: AVOptionType;
     default_val: Tdefault_val;
     min: double;
     max: double;
     flags: int;
     _unit: PAnsiChar;
  End;
  pAVOption = ^AVOption;
  AVOptionRange = Record
     str: PAnsiChar;
     value_min, value_max: double;
     component_min, component_max: double;
     is_range: int;
  End;
  pAVOptionRange = ^AVOptionRange;
  ppAVOptionRange = ^pAVOptionRange;
  AVOptionRanges = Record
     range: ppAVOptionRange;
     nb_ranges: int;
     nb_components: int;
  End;
  pAVOptionRanges = ^AVOptionRanges;
Const
  AV_OPT_FLAG_ENCODING_PARAM = 1;
  AV_OPT_FLAG_DECODING_PARAM = 2;
  AV_OPT_FLAG_AUDIO_PARAM = 8;
  AV_OPT_FLAG_VIDEO_PARAM = 16;
  AV_OPT_FLAG_SUBTITLE_PARAM = 32;
  AV_OPT_FLAG_EXPORT = 64;
  AV_OPT_FLAG_READONLY = 128;
  AV_OPT_FLAG_BSF_PARAM = (1 Shl 8);
  AV_OPT_FLAG_FILTERING_PARAM = (1 Shl 16);
Type
  AVClassCategory = (
     AV_CLASS_CATEGORY_NA = 0,
     AV_CLASS_CATEGORY_INPUT,
     AV_CLASS_CATEGORY_OUTPUT,
     AV_CLASS_CATEGORY_MUXER,
     AV_CLASS_CATEGORY_DEMUXER,
     AV_CLASS_CATEGORY_ENCODER,
     AV_CLASS_CATEGORY_DECODER,
     AV_CLASS_CATEGORY_FILTER,
     AV_CLASS_CATEGORY_BITSTREAM_FILTER,
     AV_CLASS_CATEGORY_SWSCALER,
     AV_CLASS_CATEGORY_SWRESAMPLER,
     AV_CLASS_CATEGORY_DEVICE_VIDEO_OUTPUT = 40,
     AV_CLASS_CATEGORY_DEVICE_VIDEO_INPUT,
     AV_CLASS_CATEGORY_DEVICE_AUDIO_OUTPUT,
     AV_CLASS_CATEGORY_DEVICE_AUDIO_INPUT,
     AV_CLASS_CATEGORY_DEVICE_OUTPUT,
     AV_CLASS_CATEGORY_DEVICE_INPUT,
     AV_CLASS_CATEGORY_NB
     );
  pAVClassCategory = ^AVClassCategory;
  pAVClass = ^avclass;
  avclass = Record
     class_name: PAnsiChar;
     item_name: Function(ctx: Pointer): PAnsiChar; cdecl;
     option: pAVOption;
     version: int;
     log_level_offset_offset: int;
     parent_log_context_offset: int;
     child_next: Function(obj: Pointer; prev: Pointer): Pointer; cdecl;
     child_class_next: Function(Const prev: pAVClass): pAVClass; cdecl;
     category: AVClassCategory;
     get_category: Function(ctx: Pointer): pAVClassCategory; cdecl;
     query_ranges: Function(Var ranges: pAVOptionRanges; obj: Pointer; Const key: PAnsiChar; flags: int): int; cdecl;
  End;
  PVA_LIST = ^VA_LIST;
  VA_LIST = Array[0..0] Of Pointer;
Const
  AV_LOG_QUIET = -8;
  AV_LOG_PANIC = 0;
  AV_LOG_FATAL = 8;
  AV_LOG_ERROR = 16;
  AV_LOG_WARNING = 24;
  AV_LOG_INFO = 32;
  AV_LOG_VERBOSE = 40;
  AV_LOG_DEBUG = 48;
  AV_LOG_TRACE = 56;
  AV_LOG_MAX_OFFSET = (AV_LOG_TRACE - AV_LOG_QUIET);
Type
  pAVSampleFormat = ^AVSampleFormat;
  AVSampleFormat = (
     AV_SAMPLE_FMT_NONE = -1,
     AV_SAMPLE_FMT_U8,
     AV_SAMPLE_FMT_S16,
     AV_SAMPLE_FMT_S32,
     AV_SAMPLE_FMT_FLT,
     AV_SAMPLE_FMT_DBL,
     AV_SAMPLE_FMT_U8P,
     AV_SAMPLE_FMT_S16P,
     AV_SAMPLE_FMT_S32P,
     AV_SAMPLE_FMT_FLTP,
     AV_SAMPLE_FMT_DBLP,
     AV_SAMPLE_FMT_S64,
     AV_SAMPLE_FMT_S64P,
     AV_SAMPLE_FMT_NB
     );
  TAVSampleFormat = AVSampleFormat;

Const
  AV_OPT_FLAG_IMPLICIT_KEY = 1;

  AV_OPT_SEARCH_CHILDREN = (1 Shl 0);
  AV_OPT_SEARCH_FAKE_OBJ = (1 Shl 1);
  AV_OPT_ALLOW_NULL = (1 Shl 2);
  AV_OPT_MULTI_COMPONENT_RANGE = (1 Shl 12);

  AV_OPT_SERIALIZE_SKIP_DEFAULTS = $00000001;
  AV_OPT_SERIALIZE_OPT_FLAGS_EXACT = $00000002;

Type
  Tav_log_callback = Procedure(p: Pointer; lvl: Integer; fmt: PAnsiChar; vl: PVA_LIST);cdecl;

Const
  AV_LOG_SKIP_REPEATED = 1;
  AV_LOG_PRINT_LEVEL = 2;

  AV_FOURCC_MAX_STRING_SIZE = 32;

  AVERROR_BSF_NOT_FOUND = -($F8 Or (Ord('B') Shl 8) Or (Ord('S') Shl 16) Or (Ord('F') Shl 24));
  AVERROR_BUG = -(Ord('B') Or (Ord('U') Shl 8) Or (Ord('G') Shl 16) Or (Ord('!') Shl 24));
  AVERROR_BUFFER_TOO_SMALL = -(Ord('B') Or (Ord('U') Shl 8) Or (Ord('F') Shl 16) Or (Ord('S') Shl 24));
  AVERROR_DECODER_NOT_FOUND = -($F8 Or (Ord('D') Shl 8) Or (Ord('E') Shl 16) Or (Ord('C') Shl 24));
  AVERROR_DEMUXER_NOT_FOUND = -($F8 Or (Ord('D') Shl 8) Or (Ord('E') Shl 16) Or (Ord('M') Shl 24));
  AVERROR_ENCODER_NOT_FOUND = -($F8 Or (Ord('E') Shl 8) Or (Ord('N') Shl 16) Or (Ord('C') Shl 24));
  AVERROR_EOF = -(Ord('E') Or (Ord('O') Shl 8) Or (Ord('F') Shl 16) Or (Ord(' ') Shl 24));
  AVERROR_EXIT = -(Ord('E') Or (Ord('X') Shl 8) Or (Ord('I') Shl 16) Or (Ord('T') Shl 24));
  AVERROR_EXTERNAL = -(Ord('E') Or (Ord('X') Shl 8) Or (Ord('T') Shl 16) Or (Ord(' ') Shl 24));
  AVERROR_FILTER_NOT_FOUND = -($F8 Or (Ord('F') Shl 8) Or (Ord('I') Shl 16) Or (Ord('L') Shl 24));
  AVERROR_INVALIDDATA = -(Ord('I') Or (Ord('N') Shl 8) Or (Ord('D') Shl 16) Or (Ord('A') Shl 24));
  AVERROR_MUXER_NOT_FOUND = -($F8 Or (Ord('M') Shl 8) Or (Ord('U') Shl 16) Or (Ord('X') Shl 24));
  AVERROR_OPTION_NOT_FOUND = -($F8 Or (Ord('O') Shl 8) Or (Ord('P') Shl 16) Or (Ord('T') Shl 24));
  AVERROR_PATCHWELCOME = -(Ord('P') Or (Ord('A') Shl 8) Or (Ord('W') Shl 16) Or (Ord('E') Shl 24));
  AVERROR_PROTOCOL_NOT_FOUND = -($F8 Or (Ord('P') Shl 8) Or (Ord('R') Shl 16) Or (Ord('O') Shl 24));
  AVERROR_STREAM_NOT_FOUND = -($F8 Or (Ord('S') Shl 8) Or (Ord('T') Shl 16) Or (Ord('R') Shl 24));
  AVERROR_BUG2 = -(Ord('B') Or (Ord('U') Shl 8) Or (Ord('G') Shl 16) Or (Ord(' ') Shl 24));
  AVERROR_UNKNOWN = -(Ord('U') Or (Ord('N') Shl 8) Or (Ord('K') Shl 16) Or (Ord('N') Shl 24));
  AVERROR_EXPERIMENTAL = -$2BB2AFA8;
  AVERROR_INPUT_CHANGED = -$636E6701;
  AVERROR_OUTPUT_CHANGED = -$636E6702;
  AVERROR_HTTP_BAD_REQUEST = -($F8 Or (Ord('4') Shl 8) Or (Ord('0') Shl 16) Or (Ord('0') Shl 24));
  AVERROR_HTTP_UNAUTHORIZED = -($F8 Or (Ord('4') Shl 8) Or (Ord('0') Shl 16) Or (Ord('1') Shl 24));
  AVERROR_HTTP_FORBIDDEN = -($F8 Or (Ord('4') Shl 8) Or (Ord('0') Shl 16) Or (Ord('3') Shl 24));
  AVERROR_HTTP_NOT_FOUND = -($F8 Or (Ord('4') Shl 8) Or (Ord('0') Shl 16) Or (Ord('4') Shl 24));
  AVERROR_HTTP_OTHER_4XX = -($F8 Or (Ord('4') Shl 8) Or (Ord('X') Shl 16) Or (Ord('X') Shl 24));
  AVERROR_HTTP_SERVER_ERROR = -($F8 Or (Ord('5') Shl 8) Or (Ord('X') Shl 16) Or (Ord('X') Shl 24));
  AV_ERROR_MAX_STRING_SIZE = 64;
  AVERROR_EPERM = -1;
  AVERROR_ENOENT = -2;
  AVERROR_ESRCH = -3;
  AVERROR_EINTR = -4;
  AVERROR_EIO = -5;
  AVERROR_ENXIO = -6;
  AVERROR_E2BIG = -7;
  AVERROR_ENOEXEC = -8;
  AVERROR_EBADF = -9;
  AVERROR_ECHILD = -10;
  AVERROR_EAGAIN = -11;
  AVERROR_ENOMEM = -12;
  AVERROR_EACCES = -13;
  AVERROR_EFAULT = -14;
  AVERROR_ENOTBLK = -15;
  AVERROR_EBUSY = -16;
  AVERROR_EEXIST = -17;
  AVERROR_EXDEV = -18;
  AVERROR_ENODEV = -19;
  AVERROR_ENOTDIR = -20;
  AVERROR_EISDIR = -21;
  AVERROR_EINVAL = -22;
  AVERROR_ENFILE = -23;
  AVERROR_EMFILE = -24;
  AVERROR_ENOTTY = -25;
  AVERROR_ETXTBSY = -26;
  AVERROR_EFBIG = -27;
  AVERROR_ENOSPC = -28;
  AVERROR_ESPIPE = -29;
  AVERROR_EROFS = -30;
  AVERROR_EMLINK = -31;
  AVERROR_EPIPE = -32;
  AVERROR_EDOM = -33;
  AVERROR_ERANGE = -34;
  AVERROR_EDEADLK = -36;
  AVERROR_ENAMETOOLONG = -38;
  AVERROR_ENOLCK = -39;
  AVERROR_ENOSYS = -40;
  AVERROR_ENOTEMPTY = -41;
  AVERROR_ELOOP = -114;
  AVERROR_ENOMSG = -91;
  AVERROR_EIDRM = -90;
  AVERROR_ENOSTR = -99;
  AVERROR_ENODATA = -96;
  AVERROR_ETIME = -101;
  AVERROR_ENOSR = -98;
  AVERROR_EREMOTE = -71;
  AVERROR_ENOLINK = -97;
  AVERROR_EMULTIHOP = -95;
  AVERROR_EBADMSG = -94;
  AVERROR_EPROTO = -134;
  AVERROR_EOVERFLOW = -132;
  AVERROR_EILSEQ = -42;
  AVERROR_EUSERS = -68;
  AVERROR_ENOTSOCK = -128;
  AVERROR_EDESTADDRREQ = -109;
  AVERROR_EMSGSIZE = -115;
  AVERROR_EPROTOTYPE = -136;
  AVERROR_ENOPROTOOPT = -123;
  AVERROR_EPROTONOSUPPORT = -135;
  AVERROR_ESOCKTNOSUPPORT = -44;
  AVERROR_EOPNOTSUPP = -130;
  AVERROR_EPFNOSUPPORT = -46;
  AVERROR_EAFNOSUPPORT = -102;
  AVERROR_EADDRINUSE = -100;
  AVERROR_EADDRNOTAVAIL = -101;
  AVERROR_ENETDOWN = -116;
  AVERROR_ENETUNREACH = -118;
  AVERROR_ENETRESET = -117;
  AVERROR_ECONNABORTED = -106;
  AVERROR_ECONNRESET = -108;
  AVERROR_ENOBUFS = -119;
  AVERROR_EISCONN = -113;
  AVERROR_ENOTCONN = -126;
  AVERROR_ESHUTDOWN = -58;
  AVERROR_ETOOMANYREFS = -59;
  AVERROR_ETIMEDOUT = -138;
  AVERROR_ECONNREFUSED = -107;
  AVERROR_EHOSTDOWN = -64;
  AVERROR_EHOSTUNREACH = -110;
  AVERROR_EALREADY = -103;
  AVERROR_EINPROGRESS = -112;
  AVERROR_ESTALE = -70;
  AVERROR_ECANCELED = -105;
  AVERROR_EOWNERDEAD = -133;
  AVERROR_ENOTRECOVERABLE = -44;
  WSABASEERR = -10000;
  WSAEINTR = WSABASEERR - 4;
  WSAEBADF = WSABASEERR - 9;
  WSAEACCES = WSABASEERR - 13;
  WSAEFAULT = WSABASEERR - 14;
  WSAEINVAL = WSABASEERR - 22;
  WSAEMFILE = WSABASEERR - 24;
  WSAEWOULDBLOCK = WSABASEERR - 35;
  WSAEINPROGRESS = WSABASEERR - 36;
  WSAEALREADY = WSABASEERR - 37;
  WSAENOTSOCK = WSABASEERR - 38;
  WSAEDESTADDRREQ = WSABASEERR - 39;
  WSAEMSGSIZE = WSABASEERR - 40;
  WSAEPROTOTYPE = WSABASEERR - 41;
  WSAENOPROTOOPT = WSABASEERR - 42;
  WSAEPROTONOSUPPORT = WSABASEERR - 43;
  WSAESOCKTNOSUPPORT = WSABASEERR - 44;
  WSAEOPNOTSUPP = WSABASEERR - 45;
  WSAEPFNOSUPPORT = WSABASEERR - 46;
  WSAEAFNOSUPPORT = WSABASEERR - 47;
  WSAEADDRINUSE = WSABASEERR - 48;
  WSAEADDRNOTAVAIL = WSABASEERR - 49;
  WSAENETDOWN = WSABASEERR - 50;
  WSAENETUNREACH = WSABASEERR - 51;
  WSAENETRESET = WSABASEERR - 52;
  WSAECONNABORTED = WSABASEERR - 53;
  WSAECONNRESET = WSABASEERR - 54;
  WSAENOBUFS = WSABASEERR - 55;
  WSAEISCONN = WSABASEERR - 56;
  WSAENOTCONN = WSABASEERR - 57;
  WSAESHUTDOWN = WSABASEERR - 58;
  WSAETOOMANYREFS = WSABASEERR - 59;
  WSAETIMEDOUT = WSABASEERR - 60;
  WSAECONNREFUSED = WSABASEERR - 61;
  WSAELOOP = WSABASEERR - 62;
  WSAENAMETOOLONG = WSABASEERR - 63;
  WSAEHOSTDOWN = WSABASEERR - 64;
  WSAEHOSTUNREACH = WSABASEERR - 65;
  WSAENOTEMPTY = WSABASEERR - 66;
  WSAEPROCLIM = WSABASEERR - 67;
  WSAEUSERS = WSABASEERR - 68;
  WSAEDQUOT = WSABASEERR - 69;
  WSAESTALE = WSABASEERR - 70;
  WSAEREMOTE = WSABASEERR - 71;
  WSAEDISCON = WSABASEERR - 101;
  WSASYSNOTREADY = WSABASEERR - 91;
  WSAVERNOTSUPPORTED = WSABASEERR - 92;
  WSANOTINITIALISED = WSABASEERR - 93;
  WSAHOST_NOT_FOUND = WSABASEERR - 1001;
  WSATRY_AGAIN = WSABASEERR - 1002;
  WSANO_RECOVERY = WSABASEERR - 1003;
  WSANO_DATA = WSABASEERR - 1004;
  WSAENOMORE = WSABASEERR - 102;
  WSAECANCELLED = WSABASEERR - 103;
  WSAEINVALIDPROCTABLE = WSABASEERR - 104;
  WSAEINVALIDPROVIDER = WSABASEERR - 105;
  WSAEPROVIDERFAILEDINIT = WSABASEERR - 106;
  WSASYSCALLFAILURE = WSABASEERR - 107;
  WSASERVICE_NOT_FOUND = WSABASEERR - 108;
  WSATYPE_NOT_FOUND = WSABASEERR - 109;
  WSA_E_NO_MORE = WSABASEERR - 110;
  WSA_E_CANCELLED = WSABASEERR - 111;
  WSAEREFUSED = WSABASEERR - 112;
  WSA_QOS_RECEIVERS = WSABASEERR - 1005;
  WSA_QOS_SENDERS = WSABASEERR - 1006;
  WSA_QOS_NO_SENDERS = WSABASEERR - 1007;
  WSA_QOS_NO_RECEIVERS = WSABASEERR - 1008;
  WSA_QOS_REQUEST_CONFIRMED = WSABASEERR - 1009;
  WSA_QOS_ADMISSION_FAILURE = WSABASEERR - 1010;
  WSA_QOS_POLICY_FAILURE = WSABASEERR - 1011;
  WSA_QOS_BAD_STYLE = WSABASEERR - 1012;
  WSA_QOS_BAD_OBJECT = WSABASEERR - 1013;
  WSA_QOS_TRAFFIC_CTRL_ERROR = WSABASEERR - 1014;
  WSA_QOS_GENERIC_ERROR = WSABASEERR - 1015;
  WSA_QOS_ESERVICETYPE = WSABASEERR - 1016;
  WSA_QOS_EFLOWSPEC = WSABASEERR - 1017;
  WSA_QOS_EPROVSPECBUF = WSABASEERR - 1018;
  WSA_QOS_EFILTERSTYLE = WSABASEERR - 1019;
  WSA_QOS_EFILTERTYPE = WSABASEERR - 1020;
  WSA_QOS_EFILTERCOUNT = WSABASEERR - 1021;
  WSA_QOS_EOBJLENGTH = WSABASEERR - 1022;
  WSA_QOS_EFLOWCOUNT = WSABASEERR - 1023;
  WSA_QOS_EUNKOWNPSOBJ = WSABASEERR - 1024;
  WSA_QOS_EPOLICYOBJ = WSABASEERR - 1025;
  WSA_QOS_EFLOWDESC = WSABASEERR - 1026;
  WSA_QOS_EPSFLOWSPEC = WSABASEERR - 1027;
  WSA_QOS_EPSFILTERSPEC = WSABASEERR - 1028;
  WSA_QOS_ESDMODEOBJ = WSABASEERR - 1029;
  WSA_QOS_ESHAPERATEOBJ = WSABASEERR - 1030;
  WSA_QOS_RESERVED_PETYPE = WSABASEERR - 1031;
Type
  TErrorItem = Record
     err: Integer;
     msg: String;
  End;
Const
  CErrorList: Array[0..173] Of TErrorItem = ((err: WSAEINTR; msg: 'Interrupted function call'), (err: WSAEBADF; msg: 'Bad file number'), (err: WSAEACCES;
     msg: 'Permission denied'), (err: WSAEFAULT; msg: 'Bad address'), (err: WSAEINVAL; msg: 'Invalid argument / Invalid data found when processing input'),
     (err: WSAEMFILE; msg: 'Too many open files'), (err: WSAENAMETOOLONG; msg: 'File name too long'), (err: WSAENOTEMPTY; msg: 'Directory not empty'),
     (err: WSAELOOP; msg: 'Too many symbolic links encountered'), (err: WSAEREMOTE; msg: 'Too many levels of remote in path'), (err: WSAEUSERS;
     msg: 'Too many users'), (err: WSAENOTSOCK; msg: 'Socket operation on non-socket'), (err: WSAEDESTADDRREQ; msg: 'Destination address required'),
     (err: WSAEMSGSIZE; msg: 'Message too long'), (err: WSAEPROTOTYPE; msg: 'Protocol wrong type for socket'), (err: WSAENOPROTOOPT;
     msg: 'Protocol not available'), (err: WSAEPROTONOSUPPORT; msg: 'Protocol not supported'), (err: WSAESOCKTNOSUPPORT; msg: 'Socket type not supported'),
     (err: WSAEOPNOTSUPP; msg: 'Operation not supported on transport endpoint'), (err: WSAEPFNOSUPPORT; msg: 'Protocol family not supported'),
     (err: WSAEAFNOSUPPORT; msg: 'Address family not supported by protocol'), (err: WSAEADDRINUSE; msg: 'Address already in use'), (err: WSAEADDRNOTAVAIL;
     msg: 'Cannot assign requested address'), (err: WSAENETDOWN; msg: 'Network is down'), (err: WSAENETUNREACH; msg: 'Network is unreachable'),
     (err: WSAENETRESET; msg: 'Network dropped connection because of reset'), (err: WSAECONNABORTED; msg: 'Software caused connection abort'),
     (err: WSAECONNRESET; msg: 'Connection reset by peer'), (err: WSAENOBUFS; msg: 'No buffer space available'), (err: WSAEISCONN;
     msg: 'Transport endpoint is already connected'), (err: WSAENOTCONN; msg: 'Transport endpoint is not connected'), (err: WSAESHUTDOWN;
     msg: 'Cannot send after transport endpoint shutdown'), (err: WSAETOOMANYREFS; msg: 'Too many references: cannot splice'), (err: WSAETIMEDOUT;
     msg: 'Connection timed out'), (err: WSAECONNREFUSED; msg: 'Connection refused'), (err: WSAEHOSTDOWN; msg: 'Host is down'), (err: WSAEHOSTUNREACH;
     msg: 'No route to host'), (err: WSAEALREADY; msg: 'Operation already in progress'), (err: WSAEINPROGRESS; msg: 'Operation now in progress'),
     (err: WSAESTALE; msg: 'Stale NFS file handle'), (err: WSAEDQUOT; msg: 'Quota exceeded'), (err: WSAEWOULDBLOCK; msg: 'WSAEWOULDBLOCK'), (err: WSAEPROCLIM;
     msg: 'WSAEPROCLIM'), (err: WSAEDISCON; msg: 'WSAEDISCON'), (err: WSASYSNOTREADY; msg: 'WSASYSNOTREADY'), (err: WSAVERNOTSUPPORTED;
     msg: 'WSAVERNOTSUPPORTED'), (err: WSANOTINITIALISED; msg: 'WSANOTINITIALISED'), (err: WSAHOST_NOT_FOUND; msg: 'WSAHOST_NOT_FOUND'), (err: WSATRY_AGAIN;
     msg: 'WSATRY_AGAIN'), (err: WSANO_RECOVERY; msg: 'WSANO_RECOVERY'), (err: WSANO_DATA; msg: 'WSANO_DATA'), (err: WSAENOMORE; msg: 'WSAENOMORE'),
     (err: WSAECANCELLED; msg: 'WSAECANCELLED'), (err: WSAEINVALIDPROCTABLE; msg: 'WSAEINVALIDPROCTABLE'), (err: WSAEINVALIDPROVIDER;
     msg: 'WSAEINVALIDPROVIDER'), (err: WSAEPROVIDERFAILEDINIT; msg: 'WSAEPROVIDERFAILEDINIT'), (err: WSASYSCALLFAILURE; msg: 'WSASYSCALLFAILURE'),
     (err: WSASERVICE_NOT_FOUND; msg: 'WSASERVICE_NOT_FOUND'), (err: WSATYPE_NOT_FOUND; msg: 'WSATYPE_NOT_FOUND'), (err: WSA_E_NO_MORE; msg: 'WSA_E_NO_MORE'),
     (err: WSA_E_CANCELLED; msg: 'WSA_E_CANCELLED'), (err: WSAEREFUSED; msg: 'WSAEREFUSED'),
     (err: AVERROR_BSF_NOT_FOUND; msg: 'Bitstream filter not found'), (err: AVERROR_BUG; msg: 'Internal bug, should not have happened'), (err: AVERROR_BUG2;
     msg: 'Internal bug, should not have happened'), (err: AVERROR_BUFFER_TOO_SMALL; msg: 'Buffer too small'), (err: AVERROR_DECODER_NOT_FOUND;
     msg: 'Decoder not found'), (err: AVERROR_DEMUXER_NOT_FOUND; msg: 'Demuxer not found'), (err: AVERROR_ENCODER_NOT_FOUND; msg: 'Encoder not found'),
     (err: AVERROR_EOF; msg: 'End of file'), (err: AVERROR_EXIT; msg: 'Immediate exit requested'), (err: AVERROR_EXTERNAL;
     msg: 'Generic error in an external library'), (err: AVERROR_FILTER_NOT_FOUND; msg: 'Filter not found'), (err: AVERROR_INVALIDDATA;
     msg: 'Invalid data found when processing input'), (err: AVERROR_MUXER_NOT_FOUND; msg: 'Muxer not found'), (err: AVERROR_OPTION_NOT_FOUND;
     msg: 'Option not found'), (err: AVERROR_PATCHWELCOME; msg: 'Not yet implemented in FFmpeg, patches welcome'), (err: AVERROR_PROTOCOL_NOT_FOUND;
     msg: 'Protocol not found'), (err: AVERROR_STREAM_NOT_FOUND; msg: 'Stream not found'), (err: AVERROR_UNKNOWN; msg: 'Unknown error occurred'),
     (err: AVERROR_EXPERIMENTAL; msg: 'Requested feature is flagged experimental. Set strict_std_compliance if you really want to use it.'),
     (err: AVERROR_INPUT_CHANGED; msg: 'Input changed between calls. Reconfiguration is required. (can be OR-ed with AVERROR_OUTPUT_CHANGED)'),
     (err: AVERROR_OUTPUT_CHANGED; msg: 'Output changed between calls. Reconfiguration is required. (can be OR-ed with AVERROR_INPUT_CHANGED)'),
     (err: AVERROR_HTTP_BAD_REQUEST; msg: 'HTTP or RTSP error: bad request(400)'), (err: AVERROR_HTTP_UNAUTHORIZED;
     msg: 'HTTP or RTSP error: unauthorized(401)'), (err: AVERROR_HTTP_FORBIDDEN; msg: 'HTTP or RTSP error: forbidden(403)'), (err: AVERROR_HTTP_NOT_FOUND;
     msg: 'HTTP or RTSP error: not found(404)'), (err: AVERROR_HTTP_OTHER_4XX; msg: 'HTTP or RTSP error: other error(4xx)'), (err: AVERROR_HTTP_SERVER_ERROR;
     msg: 'HTTP or RTSP error: server error(5xx)'), (err: AVERROR_ENOENT; msg: 'No such file or directory'), (err: AVERROR_ESRCH; msg: 'No such process'),
     (err: AVERROR_EINTR; msg: 'Interrupted function call'), (err: AVERROR_EIO; msg: 'I/O error'), (err: AVERROR_ENXIO; msg: 'No such device or address'),
     (err: AVERROR_E2BIG; msg: 'Argument list too long'), (err: AVERROR_ENOEXEC; msg: 'Exec format error'), (err: AVERROR_EBADF; msg: 'Bad file number'),
     (err: AVERROR_ECHILD; msg: 'No child processes'), (err: AVERROR_EAGAIN; msg: 'Resource temporarily unavailable / Try again'), (err: AVERROR_ENOMEM;
     msg: 'Not enough space / Out of memory'), (err: AVERROR_EACCES; msg: 'Permission denied'), (err: AVERROR_EFAULT; msg: 'Bad address'), (err: AVERROR_ENOTBLK;
     msg: 'Unknown error'), (err: AVERROR_EBUSY; msg: 'Device or resource busy'), (err: AVERROR_EEXIST; msg: 'File exists'), (err: AVERROR_EXDEV;
     msg: 'Cross-device link'), (err: AVERROR_ENODEV; msg: 'No such device'), (err: AVERROR_ENOTDIR; msg: 'Not a directory'), (err: AVERROR_EISDIR;
     msg: 'Is a directory'), (err: AVERROR_EINVAL; msg: 'Invalid argument / Invalid data found when processing input'), (err: AVERROR_ENFILE;
     msg: 'Too many open files in system / File table overflow'), (err: AVERROR_EMFILE; msg: 'Too many open files'), (err: AVERROR_ENOTTY;
     msg: 'Inappropriate I/O control operation / Not a typewriter'), (err: AVERROR_ETXTBSY; msg: 'Unknown error'), (err: AVERROR_EFBIG; msg: 'File too large'),
     (err: AVERROR_ENOSPC; msg: 'No space left on device'), (err: AVERROR_ESPIPE; msg: 'Illegal seek'), (err: AVERROR_EROFS; msg: 'Read-only file system'),
     (err: AVERROR_EMLINK; msg: 'Too many links'), (err: AVERROR_EPIPE; msg: 'Broken pipe'), (err: AVERROR_EDOM; msg: 'Math argument out of domain of func'),
     (err: AVERROR_ERANGE; msg: 'Math result not representable'), (err: AVERROR_EDEADLK; msg: 'Resource deadlock avoided'), (err: AVERROR_ENAMETOOLONG;
     msg: 'File name too long'), (err: AVERROR_ENOLCK; msg: 'No locks available'), (err: AVERROR_ENOSYS; msg: 'Function not implemented'),
     (err: AVERROR_ENOTEMPTY; msg: 'Directory not empty'), (err: AVERROR_ELOOP; msg: 'Too many symbolic links encountered'), (err: AVERROR_ENOMSG;
     msg: 'Unknown error'), (err: AVERROR_EIDRM; msg: 'Unknown error'), (err: AVERROR_ENOSTR; msg: 'Unknown error'), (err: AVERROR_ENODATA;
     msg: 'Unknown error'), (err: AVERROR_ETIME; msg: 'Unknown error'), (err: AVERROR_ENOSR; msg: 'Unknown error'), (err: AVERROR_EREMOTE; msg: 'Unknown error'),
     (err: AVERROR_ENOLINK; msg: 'Unknown error'), (err: AVERROR_EPROTO; msg: 'Protocol error'), (err: AVERROR_EMULTIHOP; msg: 'Unknown error'),
     (err: AVERROR_EBADMSG; msg: 'Unknown error'), (err: AVERROR_EOVERFLOW; msg: 'Value too large for defined data type'), (err: AVERROR_EILSEQ;
     msg: 'Illegal byte sequence'), (err: AVERROR_EUSERS; msg: 'Unknown error'), (err: AVERROR_ENOTSOCK; msg: 'Socket operation on non-socket'),
     (err: AVERROR_EDESTADDRREQ; msg: 'Destination address required'), (err: AVERROR_EMSGSIZE; msg: 'Message too long'), (err: AVERROR_EPROTOTYPE;
     msg: 'Protocol wrong type for socket'), (err: AVERROR_ENOPROTOOPT; msg: 'Protocol not available'), (err: AVERROR_EPROTONOSUPPORT;
     msg: 'Protocol not supported'), (err: AVERROR_ESOCKTNOSUPPORT; msg: 'Unknown error'), (err: AVERROR_EOPNOTSUPP;
     msg: 'Operation not supported on transport endpoint'), (err: AVERROR_EPFNOSUPPORT; msg: 'Unknown error'), (err: AVERROR_EAFNOSUPPORT;
     msg: 'Address family not supported by protocol'), (err: AVERROR_EADDRINUSE; msg: 'Address already in use'), (err: AVERROR_EADDRNOTAVAIL;
     msg: 'Cannot assign requested address'), (err: AVERROR_ENETDOWN; msg: 'Network is down'), (err: AVERROR_ENETUNREACH; msg: 'Network is unreachable'),
     (err: AVERROR_ENETRESET; msg: 'Network dropped connection because of reset'), (err: AVERROR_ECONNABORTED; msg: 'Software caused connection abort'),
     (err: AVERROR_ECONNRESET; msg: 'Connection reset by peer'), (err: AVERROR_ENOBUFS; msg: 'No buffer space available'), (err: AVERROR_EISCONN;
     msg: 'Transport endpoint is already connected'), (err: AVERROR_ENOTCONN; msg: 'Transport endpoint is not connected'), (err: AVERROR_ESHUTDOWN;
     msg: 'Unknown error'), (err: AVERROR_ETOOMANYREFS; msg: 'Unknown error'), (err: AVERROR_ETIMEDOUT; msg: 'Connection timed out'), (err: AVERROR_ECONNREFUSED;
     msg: 'Connection refused'), (err: AVERROR_EHOSTDOWN; msg: 'Unknown error'), (err: AVERROR_EHOSTUNREACH; msg: 'No route to host'), (err: AVERROR_EALREADY;
     msg: 'Operation already in progress'), (err: AVERROR_EINPROGRESS; msg: 'Operation now in progress'), (err: AVERROR_ESTALE; msg: 'Unknown error'),
     (err: AVERROR_ECANCELED; msg: 'Operation Canceled'), (err: AVERROR_EOWNERDEAD; msg: 'Owner died'), (err: AVERROR_ENOTRECOVERABLE;
     msg: 'State not recoverable'));

Const
  AV_CPU_FLAG_FORCE = $80000000;
  AV_CPU_FLAG_MMX = $0001;
  AV_CPU_FLAG_MMXEXT = $0002;
  AV_CPU_FLAG_MMX2 = $0002;
  AV_CPU_FLAG_3DNOW = $0004;
  AV_CPU_FLAG_SSE = $0008;
  AV_CPU_FLAG_SSE2 = $0010;
  AV_CPU_FLAG_SSE2SLOW = $40000000;
  AV_CPU_FLAG_3DNOWEXT = $0020;
  AV_CPU_FLAG_SSE3 = $0040;
  AV_CPU_FLAG_SSE3SLOW = $20000000;
  AV_CPU_FLAG_SSSE3 = $0080;
  AV_CPU_FLAG_SSSE3SLOW = $4000000;
  AV_CPU_FLAG_ATOM = $10000000;
  AV_CPU_FLAG_SSE4 = $0100;
  AV_CPU_FLAG_SSE42 = $0200;
  AV_CPU_FLAG_AESNI = $80000;
  AV_CPU_FLAG_AVX = $4000;
  AV_CPU_FLAG_AVXSLOW = $8000000;
  AV_CPU_FLAG_XOP = $0400;
  AV_CPU_FLAG_FMA4 = $0800;
  AV_CPU_FLAG_CMOV = $1000;
  AV_CPU_FLAG_AVX2 = $8000;
  AV_CPU_FLAG_FMA3 = $10000;
  AV_CPU_FLAG_BMI1 = $20000;
  AV_CPU_FLAG_BMI2 = $40000;
  AV_CPU_FLAG_AVX512 = $100000;
  AV_CPU_FLAG_ALTIVEC = $0001;
  AV_CPU_FLAG_VSX = $0002;
  AV_CPU_FLAG_POWER8 = $0004;
  AV_CPU_FLAG_ARMV5TE = (1 Shl 0);
  AV_CPU_FLAG_ARMV6 = (1 Shl 1);
  AV_CPU_FLAG_ARMV6T2 = (1 Shl 2);
  AV_CPU_FLAG_VFP = (1 Shl 3);
  AV_CPU_FLAG_VFPV3 = (1 Shl 4);
  AV_CPU_FLAG_NEON = (1 Shl 5);
  AV_CPU_FLAG_ARMV8 = (1 Shl 6);
  AV_CPU_FLAG_VFP_VM = (1 Shl 7);
  AV_CPU_FLAG_SETEND = (1 Shl 16);

Type
  pAVAudioFifo = ^AVAudioFifo;
  AVAudioFifo = Record
  End;

Type
  AVEscapeMode = (
     AV_ESCAPE_MODE_AUTO,
     AV_ESCAPE_MODE_BACKSLASH,
     AV_ESCAPE_MODE_QUOTE
     );

Const
  AV_ESCAPE_FLAG_WHITESPACE = (1 Shl 0);
  AV_ESCAPE_FLAG_STRICT = (1 Shl 1);

  AV_UTF8_FLAG_ACCEPT_INVALID_BIG_CODES = 1;
  AV_UTF8_FLAG_ACCEPT_NON_CHARACTERS = 2;
  AV_UTF8_FLAG_ACCEPT_SURROGATES = 4;
  AV_UTF8_FLAG_EXCLUDE_XML_INVALID_CONTROL_CODES = 8;
  AV_UTF8_FLAG_ACCEPT_ALL = AV_UTF8_FLAG_ACCEPT_INVALID_BIG_CODES Or AV_UTF8_FLAG_ACCEPT_NON_CHARACTERS Or AV_UTF8_FLAG_ACCEPT_SURROGATES;

  AV_BPRINT_SIZE_UNLIMITED = ((max_unsigned) - 1);
  AV_BPRINT_SIZE_AUTOMATIC = 1;
  AV_BPRINT_SIZE_COUNT_ONLY = 0;

Type
  ptm = ^tm;
  tm = Record
  End;

Type
  Tav_display_matrix = Array[0..8] Of int32_t;

  pAVExpr = ^AVExpr;
  AVExpr = Record
  End;

  Tav_expr_funcs1 = Function(p1: Pointer; p2: double): ppDouble; cdecl;
  Tav_expr_funcs2 = Function(p1: Pointer; p2: double; p3: double): ppDouble; cdecl;

  pAVFifoBuffer = ^AVFifoBuffer;
  AVFifoBuffer = Record
     buffer: puint8_t;
     rptr, wptr, _end: puint8_t;
     rndx, wndx: uint32_t;
  End;

  Tav_fifo_proc = Procedure(p1: Pointer; p2: Pointer; p3: int); cdecl;
  Tav_fifo_func = Function(p1: Pointer; p2: Pointer; p3: int): int; cdecl;

Type
   AVHWDeviceType = (
      AV_HWDEVICE_TYPE_NONE,
      AV_HWDEVICE_TYPE_VDPAU,
      AV_HWDEVICE_TYPE_CUDA,
      AV_HWDEVICE_TYPE_VAAPI,
      AV_HWDEVICE_TYPE_DXVA2,
      AV_HWDEVICE_TYPE_QSV,
      AV_HWDEVICE_TYPE_VIDEOTOOLBOX,
      AV_HWDEVICE_TYPE_D3D11VA,
      AV_HWDEVICE_TYPE_DRM,
      AV_HWDEVICE_TYPE_OPENCL,
      AV_HWDEVICE_TYPE_MEDIACODEC,
      AV_HWDEVICE_TYPE_VULKAN
      );
   TAVHWDeviceType= AVHWDeviceType;

  pAVHWDeviceInternal = ^AVHWDeviceInternal;
  AVHWDeviceInternal = Record
  End;
  pAVHWDeviceContext = ^AVHWDeviceContext;
  AVHWDeviceContext = Record
     av_class: pAVClass;
     internal: pAVHWDeviceInternal;
     _type: AVHWDeviceType;
     hwctx: Pointer;
     free: Procedure(ctx: pAVHWDeviceContext); cdecl;
     user_opaque: Pointer;
  End;
  pAVHWFramesInternal = ^AVHWFramesInternal;
  AVHWFramesInternal = Record
  End;
  pAVHWFramesContext = ^AVHWFramesContext;
  AVHWFramesContext = Record
     av_class: pAVClass;
     internal: pAVHWFramesInternal;
     device_ref: pAVBufferRef;
     device_ctx: pAVHWDeviceContext;
     hwctx: Pointer;
     free: Procedure(ctx: pAVHWFramesContext); cdecl;
     user_opaque: Pointer;
     pool: pAVBufferPool;
     initial_pool_size: int;
     format: AVPixelFormat;
     sw_format: AVPixelFormat;
     width, height: int;
  End;
Type
  AVHWFrameTransferDirection = (
     AV_HWFRAME_TRANSFER_DIRECTION_FROM,
     AV_HWFRAME_TRANSFER_DIRECTION_TO);
Type
  pAVHWFramesConstraints = ^AVHWFramesConstraints;
  AVHWFramesConstraints = Record
     valid_hw_formats: pAVPixelFormat;
     valid_sw_formats: pAVPixelFormat;
     min_width: int;
     min_height: int;
     max_width: int;
     max_height: int;
  End;
Const
  AV_HWFRAME_MAP_READ = 1 Shl 0;
  AV_HWFRAME_MAP_WRITE = 1 Shl 1;
  AV_HWFRAME_MAP_OVERWRITE = 1 Shl 2;
  AV_HWFRAME_MAP_DIRECT = 1 Shl 3;
Type
  pAVMediaCodecDeviceContext = ^AVMediaCodecDeviceContext;
  AVMediaCodecDeviceContext = Record
     surface: Pointer;
  End;
Const
  AV_DRM_MAX_PLANES = 4;
Type
  pAVDRMObjectDescriptor = ^AVDRMObjectDescriptor;
  AVDRMObjectDescriptor = Record
     fd: int;
     size: size_t;
     format_modifier: uint64_t;
  End;
  pAVDRMPlaneDescriptor = ^AVDRMPlaneDescriptor;
  AVDRMPlaneDescriptor = Record
     object_index: int;
     offset: ptrdiff_t;
     pitch: ptrdiff_t;
  End;
  pAVDRMLayerDescriptor = ^AVDRMLayerDescriptor;
  AVDRMLayerDescriptor = Record
     format: uint32_t;
     nb_planes: int;
     planes: Array[0..AV_DRM_MAX_PLANES - 1] Of AVDRMPlaneDescriptor;
  End;
  pAVDRMFrameDescriptor = ^AVDRMFrameDescriptor;
  AVDRMFrameDescriptor = Record
     nb_objects: int;
     objects: Array[0..AV_DRM_MAX_PLANES - 1] Of AVDRMObjectDescriptor;
     nb_layers: int;
     layers: Array[0..AV_DRM_MAX_PLANES - 1] Of AVDRMLayerDescriptor;
  End;
  pAVDRMDeviceContext = ^AVDRMDeviceContext;
  AVDRMDeviceContext = Record
     fd: int;
  End;
Type
  pAVComponentDescriptor = ^AVComponentDescriptor;
  AVComponentDescriptor = Record
     plane: int;
     step: int;
     offset: int;
     shift: int;
     depth: int;
     step_minus1: int deprecated;
     depth_minus1: int deprecated;
     offset_plus1: int deprecated;
  End;
  pAVPixFmtDescriptor = ^AVPixFmtDescriptor;
  AVPixFmtDescriptor = Record
     name: PAnsiChar;
     nb_components: uint8_t;
     log2_chroma_w: uint8_t;
     log2_chroma_h: uint8_t;
     flags: uint64_t;
     comp: Array[0..3] Of AVComponentDescriptor;
     alias: PAnsiChar;
  End;
Const
  AV_PIX_FMT_FLAG_BE = (1 Shl 0);
  AV_PIX_FMT_FLAG_PAL = (1 Shl 1);
  AV_PIX_FMT_FLAG_BITSTREAM = (1 Shl 2);
  AV_PIX_FMT_FLAG_HWACCEL = (1 Shl 3);
  AV_PIX_FMT_FLAG_PLANAR = (1 Shl 4);
  AV_PIX_FMT_FLAG_RGB = (1 Shl 5);
  AV_PIX_FMT_FLAG_PSEUDOPAL = (1 Shl 6);
  AV_PIX_FMT_FLAG_ALPHA = (1 Shl 7);
  AV_PIX_FMT_FLAG_BAYER = (1 Shl 8);
  AV_PIX_FMT_FLAG_FLOAT = (1 Shl 9);
Type
  Tav_read_array4_puint8_t = Record
  End;
  pav_read_array4_puint8_t = ^Tav_read_array4_puint8_t;
  Tav_read_array4_int = Record
  End;
  pav_read_array4_int = ^Tav_read_array4_int;
Const
  FF_LOSS_RESOLUTION = $0001;
  FF_LOSS_DEPTH = $0002;
  FF_LOSS_COLORSPACE = $0004;
  FF_LOSS_ALPHA = $0008;
  FF_LOSS_COLORQUANT = $0010;
  FF_LOSS_CHROMA = $0020;
Type
  Tav_image_array4_int = array4_int;
  pav_image_array4_int = ^Tav_image_array4_int;
  Tav_image_array4_puint8_t = array4_puint8_t;
  pav_image_array4_puint8_t = ^Tav_image_array4_puint8_t;
  Tav_image_array4_ptrdiff_t = array4_ptrdiff_t;
  pav_image_array4_ptrdiff_t = ^Tav_image_array4_ptrdiff_t;
Const
  AV_TS_MAX_STRING_SIZE = 32;

//Function av_size_mult(a: size_t; b: size_t; Var r: size_t): int; Inline;

Const
  AV_TIMECODE_STR_SIZE = 23;
Type
  AVTimecodeFlag = (
     AV_TIMECODE_FLAG_DROPFRAME = 1 Shl 0,
     AV_TIMECODE_FLAG_24HOURSMAX = 1 Shl 1,
     AV_TIMECODE_FLAG_ALLOWNEGATIVE = 1 Shl 2
     );
  pAVTimecode = ^AVTimecode;
  AVTimecode = Record
     start: int;
     flags: uint32_t;
     rate: AVRational;
     fps: unsigned;
  End;
Const
  M_E = 2.7182818284590452354;
  M_LN2 = 0.69314718055994530942;
  M_LN10 = 2.30258509299404568402;
  M_LOG2_10 = 3.32192809488736234787;
  M_PHI = 1.61803398874989484820;
  M_PI = 3.14159265358979323846;
  M_PI_2 = 1.57079632679489661923;
  M_SQRT1_2 = 0.70710678118654752440;
  M_SQRT2 = 1.41421356237309504880;

Type
  AVRounding = int;

Const
  AV_ROUND_ZERO = 0;
  AV_ROUND_INF = 1;
  AV_ROUND_DOWN = 2;
  AV_ROUND_UP = 3;
  AV_ROUND_NEAR_INF = 5;
  AV_ROUND_PASS_MINMAX = 8192;

Type
  pAVMotionVector = ^AVMotionVector;
  AVMotionVector = Record
     source: int32_t;
     w, h: uint8_t;
     src_x, src_y: int16_t;
     dst_x, dst_y: int16_t;
     flags: uint64_t;
     motion_x, motion_y: int32_t;
     motion_scale: uint16_t;
  End;

  pAVMD5 = ^AVMD5;
  AVMD5 = Record
  End;

  av_intfloat32 = Record
     Case Integer Of
        0:
        (i: uint32_t);
        1:
        (f: float);
  End;

  av_intfloat64 = Record
     Case Integer Of
        0:
        (i: uint64_t);
        1:
        (f: double);
  End;

type
  pAVMasteringDisplayMetadata = ^AVMasteringDisplayMetadata;
  AVMasteringDisplayMetadata = Record
     display_primaries: Array[0..2, 0..1] Of AVRational;
     white_point: Array[0..1] Of AVRational;
     min_luminance: AVRational;
     max_luminance: AVRational;
     has_primaries: int;
     has_luminance: int;
  End;

  pAVContentLightMetadata = ^AVContentLightMetadata;
  AVContentLightMetadata = Record
     MaxCLL: unsigned;
     MaxFALL: unsigned;
  End;

  av_pixelutils_sad_fn = Function(Const src1: puint8_t; stride1: ptrdiff_t; Const src2: puint8_t; stride2: ptrdiff_t): int; cdecl;

  pAVAES = ^AVAES;
  AVAES = Record
  End;

  pAVTWOFISH = ^AVTWOFISH;
  AVTWOFISH = Record
  End;

  pAVTEA = ^AVTEA;
  AVTEA = Record
  End;

  AVTXContext = Record
  End;
  pAVTXContext = ^AVTXContext;
  AVComplexFloat = Record
     re, im: float;
  End;

  AVTXType = (
     AV_TX_FLOAT_FFT = 0,
     AV_TX_FLOAT_MDCT = 1
     );
  av_tx_fn = Procedure(s: pAVTXContext; &out, &in: Pointer; stride: ptrdiff_t); cdecl;

  pAVLFG = ^AVLFG;
  AVLFG = Record
     state: Array[0..63] Of uint;
     index: int;
  End;

   Tav_bmg_get_arrayofdouble = Array[0..1] Of double;

    AVAESCTR = record
      end;

const
    AV_BF_ROUNDS = 16;

type
    PAVBlowfish = ^AVBlowfish;
    AVBlowfish = record
        p : array[0..(AV_BF_ROUNDS+2)-1] of uint32_t;
        s : array[0..3] of array[0..255] of uint32_t;
      end;

  TallocCall = function(size: Size_t): PAVBufferRef; cdecl;
  TfreeCall = procedure(opaque: Pointer; data: PByte); cdecl;
  Talloc2Call = function(opaque: Pointer; size: Size_t): PAVBufferRef; cdecl;
  Tfree2Call = procedure(opaque: POinter); cdecl;


    PAVCAMELLIA = ^AVCAMELLIA;
    AVCAMELLIA = record
      end;
    PAVCAST5 = ^AVCAST5;
    AVCAST5 = record
      end;

  type
    AVChannel =  Longint;
    Const
      AV_CHAN_NONE = -(1);
      AV_CHAN_FRONT_LEFT = (-(1))+1;
      AV_CHAN_FRONT_RIGHT = (-(1))+2;
      AV_CHAN_FRONT_CENTER = (-(1))+3;
      AV_CHAN_LOW_FREQUENCY = (-(1))+4;
      AV_CHAN_BACK_LEFT = (-(1))+5;
      AV_CHAN_BACK_RIGHT = (-(1))+6;
      AV_CHAN_FRONT_LEFT_OF_CENTER = (-(1))+7;
      AV_CHAN_FRONT_RIGHT_OF_CENTER = (-(1))+8;
      AV_CHAN_BACK_CENTER = (-(1))+9;
      AV_CHAN_SIDE_LEFT = (-(1))+10;
      AV_CHAN_SIDE_RIGHT = (-(1))+11;
      AV_CHAN_TOP_CENTER = (-(1))+12;
      AV_CHAN_TOP_FRONT_LEFT = (-(1))+13;
      AV_CHAN_TOP_FRONT_CENTER = (-(1))+14;
      AV_CHAN_TOP_FRONT_RIGHT = (-(1))+15;
      AV_CHAN_TOP_BACK_LEFT = (-(1))+16;
      AV_CHAN_TOP_BACK_CENTER = (-(1))+17;
      AV_CHAN_TOP_BACK_RIGHT = (-(1))+18;
      AV_CHAN_STEREO_LEFT = 29;
      AV_CHAN_STEREO_RIGHT = 30;
      AV_CHAN_WIDE_LEFT = 31;
      AV_CHAN_WIDE_RIGHT = 32;
      AV_CHAN_SURROUND_DIRECT_LEFT = 33;
      AV_CHAN_SURROUND_DIRECT_RIGHT = 34;
      AV_CHAN_LOW_FREQUENCY_2 = 35;
      AV_CHAN_TOP_SIDE_LEFT = 36;
      AV_CHAN_TOP_SIDE_RIGHT = 37;
      AV_CHAN_BOTTOM_FRONT_CENTER = 38;
      AV_CHAN_BOTTOM_FRONT_LEFT = 39;
      AV_CHAN_BOTTOM_FRONT_RIGHT = 40;
      AV_CHAN_UNUSED = $200;
      AV_CHAN_UNKNOWN = $300;
      AV_CHAN_AMBISONIC_BASE = $400;
      AV_CHAN_AMBISONIC_END = $7ff;
               
  type
    AVChannelOrder =  Longint;
    Const
      AV_CHANNEL_ORDER_UNSPEC = 0;
      AV_CHANNEL_ORDER_NATIVE = 1;
      AV_CHANNEL_ORDER_CUSTOM = 2;
      AV_CHANNEL_ORDER_AMBISONIC = 3;
Type

    PAVChannelCustom = ^AVChannelCustom;
    AVChannelCustom = record
        id : AVChannel;
        name : array[0..15] of char;
        opaque : pointer;
      end;
    PAVChannelLayout = ^AVChannelLayout;

    AVChannelLayout = record
        order : AVChannelOrder;
        nb_channels : longint;
        u : record
            case longint of
              0 : ( mask : uint64_t );
              1 : ( map : PAVChannelCustom );
            end;
        opaque : pointer;
      end;
                     
  type
    PAVDOVIDecoderConfigurationRecord = ^AVDOVIDecoderConfigurationRecord;
    AVDOVIDecoderConfigurationRecord = record
        dv_version_major : uint8_t;
        dv_version_minor : uint8_t;
        dv_profile : uint8_t;
        dv_level : uint8_t;
        rpu_present_flag : uint8_t;
        el_present_flag : uint8_t;
        bl_present_flag : uint8_t;
        dv_bl_signal_compatibility_id : uint8_t;
      end;

  type
    AVDownmixType =  Longint;
    Const
      AV_DOWNMIX_TYPE_UNKNOWN = 0;
      AV_DOWNMIX_TYPE_LORO = 1;
      AV_DOWNMIX_TYPE_LTRT = 2;
      AV_DOWNMIX_TYPE_DPLII = 3;
      AV_DOWNMIX_TYPE_NB = 4;

  type
    PAVDownmixInfo = ^AVDownmixInfo;
    AVDownmixInfo = record
        preferred_downmix_type : AVDownmixType;
        center_mix_level : double;
        center_mix_level_ltrt : double;
        surround_mix_level : double;
        surround_mix_level_ltrt : double;
        lfe_mix_level : double;
      end;

  type
    PAVSubsampleEncryptionInfo = ^AVSubsampleEncryptionInfo;
    AVSubsampleEncryptionInfo = record
        bytes_of_clear_data : dword;
        bytes_of_protected_data : dword;
      end;

    PAVEncryptionInfo = ^AVEncryptionInfo;
    AVEncryptionInfo = record
        scheme : uint32_t;
        crypt_byte_block : uint32_t;
        skip_byte_block : uint32_t;
        key_id : Puint8_t;
        key_id_size : uint32_t;
        iv : Puint8_t;
        iv_size : uint32_t;
        subsamples : PAVSubsampleEncryptionInfo;
        subsample_count : uint32_t;
      end;

  type
    PAVEncryptionInitInfo = ^AVEncryptionInitInfo;
    AVEncryptionInitInfo = record
        system_id : Puint8_t;
        system_id_size : uint32_t;
        key_ids : ^Puint8_t;
        num_key_ids : uint32_t;
        key_id_size : uint32_t;
        data : Puint8_t;
        data_size : uint32_t;
        next : PAVEncryptionInitInfo;
      end;

  type
    AVFilmGrainParamsType =  Longint;
    Const
      AV_FILM_GRAIN_PARAMS_NONE = 0;
      AV_FILM_GRAIN_PARAMS_AV1 = 1;
      AV_FILM_GRAIN_PARAMS_H274 = 2;

  type
    PAVFilmGrainAOMParams = ^AVFilmGrainAOMParams;
    AVFilmGrainAOMParams = record
        num_y_points : longint;
        y_points : array[0..13] of array[0..1] of uint8_t;
        chroma_scaling_from_luma : longint;
        num_uv_points : array[0..1] of longint;
        uv_points : array[0..1] of array[0..9] of array[0..1] of uint8_t;
        scaling_shift : longint;
        ar_coeff_lag : longint;
        ar_coeffs_y : array[0..23] of int8_t;
        ar_coeffs_uv : array[0..1] of array[0..24] of int8_t;
        ar_coeff_shift : longint;
        grain_scale_shift : longint;
        uv_mult : array[0..1] of longint;
        uv_mult_luma : array[0..1] of longint;
        uv_offset : array[0..1] of longint;
        overlap_flag : longint;
        limit_output_range : longint;
      end;

    PAVFilmGrainH274Params = ^AVFilmGrainH274Params;
    AVFilmGrainH274Params = record
        model_id : longint;
        bit_depth_luma : longint;
        bit_depth_chroma : longint;
        color_range : AVColorRange;
        color_primaries : AVColorPrimaries;
        color_trc : AVColorTransferCharacteristic;
        color_space : AVColorSpace;
        blending_mode_id : longint;
        log2_scale_factor : longint;
        component_model_present : array[0..2] of longint;
        num_intensity_intervals : array[0..2] of uint16_t;
        num_model_values : array[0..2] of uint8_t;
        intensity_interval_lower_bound : array[0..2] of array[0..255] of uint8_t;
        intensity_interval_upper_bound : array[0..2] of array[0..255] of uint8_t;
        comp_model_value : array[0..2] of array[0..255] of array[0..5] of int16_t;
      end;

    PAVFilmGrainParams = ^AVFilmGrainParams;
    AVFilmGrainParams = record
        _type : AVFilmGrainParamsType;
        seed : uint64_t;
        codec : record
            case longint of
              0 : ( aom : AVFilmGrainAOMParams );
              1 : ( h274 : AVFilmGrainH274Params );
            end;
      end;

        
  type
    PAVHashContext = ^AVHashContext;
    AVHashContext = record
      end;

  type
    AVHDRPlusOverlapProcessOption =  Longint;
    Const
      AV_HDR_PLUS_OVERLAP_PROCESS_WEIGHTED_AVERAGING = 0;
      AV_HDR_PLUS_OVERLAP_PROCESS_LAYERING = 1;

                 
  type
    PAVHDRPlusPercentile = ^AVHDRPlusPercentile;
    AVHDRPlusPercentile = record
        percentage : uint8_t;
        percentile : AVRational;
      end;

    PAVHDRPlusColorTransformParams = ^AVHDRPlusColorTransformParams;
    AVHDRPlusColorTransformParams = record
        window_upper_left_corner_x : AVRational;
        window_upper_left_corner_y : AVRational;
        window_lower_right_corner_x : AVRational;
        window_lower_right_corner_y : AVRational;
        center_of_ellipse_x : uint16_t;
        center_of_ellipse_y : uint16_t;
        rotation_angle : uint8_t;
        semimajor_axis_internal_ellipse : uint16_t;
        semimajor_axis_external_ellipse : uint16_t;
        semiminor_axis_external_ellipse : uint16_t;
        overlap_process_option : AVHDRPlusOverlapProcessOption;
        maxscl : array[0..2] of AVRational;
        average_maxrgb : AVRational;
        num_distribution_maxrgb_percentiles : uint8_t;
        distribution_maxrgb : array[0..14] of AVHDRPlusPercentile;
        fraction_bright_pixels : AVRational;
        tone_mapping_flag : uint8_t;
        knee_point_x : AVRational;
        knee_point_y : AVRational;
        num_bezier_curve_anchors : uint8_t;
        bezier_curve_anchors : array[0..14] of AVRational;
        color_saturation_mapping_flag : uint8_t;
        color_saturation_weight : AVRational;
      end;

    PAVDynamicHDRPlus = ^AVDynamicHDRPlus;
    AVDynamicHDRPlus = record
        itu_t_t35_country_code : uint8_t;
        application_version : uint8_t;
        num_windows : uint8_t;
        params : array[0..2] of AVHDRPlusColorTransformParams;
        targeted_system_display_maximum_luminance : AVRational;
        targeted_system_display_actual_peak_luminance_flag : uint8_t;
        num_rows_targeted_system_display_actual_peak_luminance : uint8_t;
        num_cols_targeted_system_display_actual_peak_luminance : uint8_t;
        targeted_system_display_actual_peak_luminance : array[0..24] of array[0..24] of AVRational;
        mastering_display_actual_peak_luminance_flag : uint8_t;
        num_rows_mastering_display_actual_peak_luminance : uint8_t;
        num_cols_mastering_display_actual_peak_luminance : uint8_t;
        mastering_display_actual_peak_luminance : array[0..24] of array[0..24] of AVRational;
      end;

  type
    PAVHDRVividColorToneMappingParams = ^AVHDRVividColorToneMappingParams;
    AVHDRVividColorToneMappingParams = record
        targeted_system_display_maximum_luminance : AVRational;
        base_enable_flag : longint;
        base_param_m_p : AVRational;
        base_param_m_m : AVRational;
        base_param_m_a : AVRational;
        base_param_m_b : AVRational;
        base_param_m_n : AVRational;
        base_param_k1 : longint;
        base_param_k2 : longint;
        base_param_k3 : longint;
        base_param_Delta_enable_mode : longint;
        base_param_Delta : AVRational;
        three_Spline_enable_flag : longint;
        three_Spline_num : longint;
        three_Spline_TH_mode : longint;
        three_Spline_TH_enable_MB : AVRational;
        three_Spline_TH_enable : AVRational;
        three_Spline_TH_Delta1 : AVRational;
        three_Spline_TH_Delta2 : AVRational;
        three_Spline_enable_Strength : AVRational;
      end;

    PAVHDRVividColorTransformParams = ^AVHDRVividColorTransformParams;
    AVHDRVividColorTransformParams = record
        minimum_maxrgb : AVRational;
        average_maxrgb : AVRational;
        variance_maxrgb : AVRational;
        maximum_maxrgb : AVRational;
        tone_mapping_mode_flag : longint;
        tone_mapping_param_num : longint;
        tm_params : array[0..1] of AVHDRVividColorToneMappingParams;
        color_saturation_mapping_flag : longint;
        color_saturation_num : longint;
        color_saturation_gain : array[0..7] of AVRational;
      end;

    PAVDynamicHDRVivid = ^AVDynamicHDRVivid;
    AVDynamicHDRVivid = record
        system_start_code : uint8_t;
        num_windows : uint8_t;
        params : array[0..2] of AVHDRVividColorTransformParams;
      end;


  type
    PAVHMAC  = ^AVHMAC;
    AVHMAC =  Longint;

    PAVHMACType  = ^AVHMACType;
    AVHMACType =  Longint;
    Const
      AV_HMAC_MD5 = 0;
      AV_HMAC_SHA1 = 1;
      AV_HMAC_SHA224 = 2;
      AV_HMAC_SHA256 = 3;
      AV_HMAC_SHA384 = 4;
      AV_HMAC_SHA512 = 5;

Type
   Tav_vlogCall = Procedure(avcl: Pointer; level: Integer; Const fmt: PAnsiChar; vl: PAnsiChar); cdecl;

   PPPByte = ^PPByte;
            
  type
    PAVCRC = ^AVCRC;
    AVCRC = uint32_t;

    PAVCRCId = ^AVCRCId;
    AVCRCId =  Longint;

    Const
      AV_CRC_8_ATM = 0;
      AV_CRC_16_ANSI = 1;
      AV_CRC_16_CCITT = 2;
      AV_CRC_32_IEEE = 3;
      AV_CRC_32_IEEE_LE = 4;
      AV_CRC_16_ANSI_LE = 5;
      AV_CRC_24_IEEE = 6;
      AV_CRC_8_EBU = 7;
      AV_CRC_MAX = 8;

  type
    PAVLumaCoefficients = ^AVLumaCoefficients;
    AVLumaCoefficients = record
        cr : AVRational;
        cg : AVRational;
        cb : AVRational;
      end;
            
    PAVCIExy = ^AVCIExy;
    AVCIExy = record
        x : AVRational;
        y : AVRational;
      end;


    PAVPrimaryCoefficients = ^AVPrimaryCoefficients;
    AVPrimaryCoefficients = record
        r : AVCIExy;
        g : AVCIExy;
        b : AVCIExy;
      end;


    PAVWhitepointCoefficients = ^AVWhitepointCoefficients;
    AVWhitepointCoefficients = AVCIExy;


    PAVColorPrimariesDesc = ^AVColorPrimariesDesc;
    AVColorPrimariesDesc = record
        wp : AVWhitepointCoefficients;
        prim : AVPrimaryCoefficients;
      end;
         
  type
    PAVDetectionBBox = ^AVDetectionBBox;
    AVDetectionBBox = record
      end;

    PAVDetectionBBoxHeader = ^AVDetectionBBoxHeader;
    AVDetectionBBoxHeader = record
        source : array[0..255] of char;
        nb_bboxes : uint32_t;
        bboxes_offset : size_t;
        bbox_size : size_t;
      end;


//-------------------------------------------------
 Type
                             
   Tav_log2 = Function(v: unsigned): int; cdecl;
   Tav_log2_16bit = Function(v: unsigned): int; cdecl;
   Tav_get_channel_layout = Function(Const name: PAnsiChar): uint64_t; cdecl;
   Tav_get_extended_channel_layout = Function(Const name: PAnsiChar; Var channel_layout: uint64_t; Var nb_channels: int): int; cdecl;
   Tav_get_channel_layout_string = Procedure(buf: PAnsiChar; buf_size: int; nb_channels: int; channel_layout: uint64_t); cdecl;
   Tav_bprint_channel_layout = Procedure(bp: pAVBPrint; nb_channels: int; channel_layout: uint64_t); cdecl;
   Tav_get_channel_layout_nb_channels = Function(channel_layout: uint64_t): int; cdecl;
   Tav_get_default_channel_layout = Function(nb_channels: int): int64_t; cdecl;
   Tav_get_channel_layout_channel_index = Function(channel_layout: uint64_t; channel: uint64_t): int; cdecl;
   Tav_channel_layout_extract_channel = Function(channel_layout: uint64_t; index: int): uint64_t; cdecl;
   Tav_get_channel_name = Function(channel: uint64_t): PAnsiChar; cdecl;
   Tav_get_channel_description = Function(channel: uint64_t): PAnsiChar; cdecl;
   Tav_get_standard_channel_layout = Function(index: unsigned; Var layout: uint64_t; Const name: ppAnsiChar): int; cdecl;
   Tav_dict_get = Function(Const m: pAVDictionary; Const key: PAnsiChar; Const prev: pAVDictionaryEntry; flags: int): pAVDictionaryEntry; cdecl;
   Tav_dict_count = Function(Const m: pAVDictionary): int; cdecl;
   Tav_dict_set = Function(Var pm: pAVDictionary; Const key: PAnsiChar; Const value: PAnsiChar; flags: int): int; cdecl;
   Tav_dict_set_int = Function(Var pm: pAVDictionary; Const key: PAnsiChar; value: int64_t; flags: int): int; cdecl;
   Tav_dict_parse_string = Function(Var pm: pAVDictionary; Const str: PAnsiChar; Const key_val_sep: PAnsiChar; Const pairs_sep: PAnsiChar; flags: int): int; cdecl;
   Tav_dict_copy = Function(Var dst: pAVDictionary; Const src: pAVDictionary; flags: int): int; cdecl;
   Tav_dict_free = Procedure(Var m: pAVDictionary); cdecl;
   Tav_dict_get_string = Function(Const m: pAVDictionary; Var buffer: PAnsiChar; Const key_val_sep: AnsiChar; Const pairs_sep: AnsiChar): int; cdecl;
   Tav_buffer_alloc = Function(size: int): pAVBufferRef; cdecl;
   Tav_buffer_allocz = Function(size: int): pAVBufferRef; cdecl;
   Tav_buffer_create = Function(data: puint8_t; size: int; freeproc: TFreeProc; opaque: Pointer; flags: int): AVBufferRef; cdecl;
   Tav_buffer_default_free = Procedure(opaque: Pointer; data: puint8_t); cdecl;
   Tav_buffer_ref = Function(buf: pAVBufferRef): pAVBufferRef; cdecl;
   Tav_buffer_unref = Procedure(Var buf: pAVBufferRef); cdecl;
   Tav_buffer_is_writable = Function(Const buf: pAVBufferRef): int; cdecl;
   Tav_buffer_get_opaque = Function(Const buf: pAVBufferRef): Pointer; cdecl;
   Tav_buffer_get_ref_count = Function(Const buf: pAVBufferRef): int; cdecl;
   Tav_buffer_make_writable = Function(Var buf: pAVBufferRef): int; cdecl;
   Tav_buffer_realloc = Function(Var buf: pAVBufferRef; size: int): int; cdecl;
   Tav_buffer_pool_init = Function(size: int; alloc: Tbuffer_pool_init_proc): pAVBufferPool; cdecl;
   Tav_buffer_pool_init2 = Function(size: int; opaque: Pointer; alloc: Tav_buffer_pool_init2_alloc_proc; pool_free: Tav_buffer_pool_init2_pool_free_proc): pAVBufferPool; cdecl;
   Tav_buffer_pool_uninit = Procedure(Var pool: pAVBufferPool); cdecl;
   Tav_buffer_pool_get = Function(pool: pAVBufferPool): pAVBufferRef; cdecl;
   Tav_reduce = Function(Var dst_num: int; Var dst_den: int; num: int64_t; den: int64_t; max: int64_t): int; cdecl;
   Tav_mul_q = Function(b, c: AVRational): AVRational; cdecl;
   Tav_div_q = Function(b, c: AVRational): AVRational; cdecl;
   Tav_add_q = Function(b, c: AVRational): AVRational; cdecl;
   Tav_sub_q = Function(b, c: AVRational): AVRational; cdecl;
   Tav_d2q = Function(d: double; max: int): AVRational; cdecl;
   Tav_nearer_q = Function(q: AVRational; q1: AVRational; q2: AVRational): int; cdecl;
   Tav_find_nearest_q_idx = Function(q: AVRational; Const q_list: pAVRational): int; cdecl;
   Tav_q2intfloat = Function(q: AVRational): uint32_t; cdecl;
   Tav_frame_get_best_effort_timestamp = Function(Const frame: pAVFrame): int64_t; cdecl;
   Tav_frame_set_best_effort_timestamp = Procedure(frame: pAVFrame; val: int64_t); cdecl;
   Tav_frame_get_pkt_duration = Function(Const frame: pAVFrame): int64_t; cdecl;
   Tav_frame_set_pkt_duration = Procedure(frame: pAVFrame; val: int64_t); cdecl;
   Tav_frame_get_pkt_pos = Function(Const frame: pAVFrame): int64_t; cdecl;
   Tav_frame_set_pkt_pos = Procedure(frame: pAVFrame; val: int64_t); cdecl;
   Tav_frame_get_channel_layout = Function(Const frame: pAVFrame): int64_t; cdecl;
   Tav_frame_set_channel_layout = Procedure(frame: pAVFrame; val: int64_t); cdecl;
   Tav_frame_get_channels = Function(Const frame: pAVFrame): int; cdecl;
   Tav_frame_set_channels = Procedure(frame: pAVFrame; val: int); cdecl;
   Tav_frame_get_sample_rate = Function(Const frame: pAVFrame): int; cdecl;
   Tav_frame_set_sample_rate = Procedure(frame: pAVFrame; val: int); cdecl;
   Tav_frame_get_metadata = Function(Const frame: AVFrame): pAVDictionary; cdecl;
   Tav_frame_set_metadata = Procedure(frame: pAVFrame; val: pAVDictionary); cdecl;
   Tav_frame_get_decode_error_flags = Function(Const frame: pAVFrame): int; cdecl;
   Tav_frame_set_decode_error_flags = Procedure(frame: pAVFrame; val: int); cdecl;
   Tav_frame_get_pkt_size = Function(Const frame: pAVFrame): int; cdecl;
   Tav_frame_set_pkt_size = Procedure(frame: AVFrame; val: int); cdecl;
   Tav_frame_get_qp_table = Function(f: pAVFrame; stride: pint; _type: pint): pint8_t; cdecl;
   Tav_frame_set_qp_table = Function(f: pAVFrame; buf: pAVBufferRef; stride: int; _type: int): int; cdecl;
   Tav_frame_get_colorspace = Function(Const frame: pAVFrame): AVColorSpace; cdecl;
   Tav_frame_set_colorspace = Procedure(frame: pAVFrame; val: AVColorSpace); cdecl;
   Tav_frame_get_color_range = Function(Const frame: pAVFrame): AVColorRange; cdecl;
   Tav_frame_set_color_range = Procedure(frame: pAVFrame; val: AVColorRange); cdecl;
   Tav_get_colorspace_name = Function(val: AVColorSpace): PAnsiChar; cdecl;
   Tav_frame_alloc = Function(): pAVFrame; cdecl;
   Tav_frame_free = Procedure(Var frame: pAVFrame); cdecl;
   Tav_frame_ref = Function(dst: pAVFrame; Const src: pAVFrame): int; cdecl;
   Tav_frame_clone = Function(Const src: pAVFrame): pAVFrame; cdecl;
   Tav_frame_unref = Procedure(frame: pAVFrame); cdecl;
   Tav_frame_move_ref = Procedure(dst: pAVFrame; src: pAVFrame); cdecl;
   Tav_frame_get_buffer = Function(frame: pAVFrame; align: int): int; cdecl;
   Tav_frame_is_writable = Function(frame: pAVFrame): int; cdecl;
   Tav_frame_make_writable = Function(frame: pAVFrame): int; cdecl;
   Tav_frame_copy = Function(dst: pAVFrame; Const src: pAVFrame): int; cdecl;
   Tav_frame_copy_props = Function(dst: pAVFrame; Const src: pAVFrame): int; cdecl;
   Tav_frame_get_plane_buffer = Function(frame: pAVFrame; plane: int): pAVBufferRef; cdecl;
   Tav_frame_new_side_data = Function(frame: AVFrame; _type: AVFrameSideDataType; size: int): pAVFrameSideData; cdecl;
   Tav_frame_new_side_data_from_buf = Function(frame: pAVFrame; _type: AVFrameSideDataType; buf: pAVBufferRef): pAVFrameSideData; cdecl;
   Tav_frame_get_side_data = Function(Const frame: pAVFrame; _type: AVFrameSideDataType): pAVFrameSideData; cdecl;
   Tav_frame_remove_side_data = Procedure(frame: pAVFrame; _type: AVFrameSideDataType); cdecl;
   Tav_frame_apply_cropping = Function(frame: pAVFrame; flags: int): int; cdecl;
   Tav_frame_side_data_name = Function(_type: AVFrameSideDataType): PAnsiChar; cdecl;
   Tav_get_sample_fmt_name = Function(sample_fmt: AVSampleFormat): PAnsiChar; cdecl;
   Tav_get_sample_fmt = Function(Const name: PAnsiChar): AVSampleFormat; cdecl;
   Tav_get_alt_sample_fmt = Function(sample_fmt: AVSampleFormat; planar: int): AVSampleFormat; cdecl;
   Tav_get_packed_sample_fmt = Function(sample_fmt: AVSampleFormat): AVSampleFormat; cdecl;
   Tav_get_planar_sample_fmt = Function(sample_fmt: AVSampleFormat): AVSampleFormat; cdecl;
   Tav_get_sample_fmt_string = Function(buf: PAnsiChar; buf_size: int; sample_fmt: AVSampleFormat): PAnsiChar; cdecl;
   Tav_get_bytes_per_sample = Function(sample_fmt: AVSampleFormat): int; cdecl;
   Tav_sample_fmt_is_planar = Function(sample_fmt: AVSampleFormat): int; cdecl;
   Tav_samples_get_buffer_size = Function(linesize: int; nb_channels: int; nb_samples: int; sample_fmt: AVSampleFormat; align: int): int; cdecl;
   Tav_samples_fill_arrays = Function(Var audio_data: puint8_t; Var linesize: int; Const buf: puint8_t; nb_channels: int; nb_samples: int; sample_fmt: AVSampleFormat; align: int): int; cdecl;
   Tav_samples_alloc = Function(Var audio_data: puint8_t; linesize: pint; nb_channels: int; nb_samples: int; sample_fmt: AVSampleFormat; align: int): int; cdecl;
   Tav_samples_alloc_array_and_samples = Function(Var audio_data: ppuint8_t; Var linesize: int; nb_channels: int; nb_samples: int; sample_fmt: AVSampleFormat; align: int): int; cdecl;
   Tav_samples_copy = Function(Var dst: puint8_t; Const src: ppuint8_t; dst_offset: int; src_offset: int; nb_samples: int; nb_channels: int; sample_fmt: AVSampleFormat): int; cdecl;
   Tav_samples_set_silence = Function(Var audio_data: puint8_t; offset: int; nb_samples: int; nb_channels: int; sample_fmt: AVSampleFormat): int; cdecl;
   Tav_opt_show2 = Function(obj, av_log_obj: Pointer; req_flags, rej_flags: int): int; cdecl;
   Tav_opt_set_defaults = Procedure(s: Pointer); cdecl;
   Tav_opt_set_defaults2 = Procedure(s: Pointer; mask, flags: int); cdecl;
   Tav_set_options_string = Function(ctx: Pointer; Const opts: PAnsiChar; Const key_val_sep: PAnsiChar; Const pairs_sep: PAnsiChar): int; cdecl;
   Tav_opt_set_from_string = Function(ctx: Pointer; Const opts: PAnsiChar; Const shorthand: ppAnsiChar; Const key_val_sep: PAnsiChar; Const pairs_sep: PAnsiChar):int; cdecl;
   Tav_opt_free = Procedure(obj: Pointer); cdecl;
   Tav_opt_flag_is_set = Function(obj: Pointer; Const field_name: PAnsiChar; Const flag_name: PAnsiChar): int; cdecl;
   Tav_opt_set_dict = Function(obj: Pointer; Var options: pAVDictionary): int; cdecl;
   Tav_opt_set_dict2 = Function(obj: Pointer; Var options: pAVDictionary; search_flags: int): int; cdecl;
   Tav_opt_get_key_value = Function(Const ropts: ppAnsiChar; Const key_val_sep: PAnsiChar; Const pairs_sep: PAnsiChar; flags: unsigned; rkey: ppAnsiChar; rval: ppAnsiChar): int; cdecl;
   Tav_opt_eval_flags = Function(obj: Pointer; Const o: pAVOption; Const val: PAnsiChar; Var flags_out: int): int; cdecl;
   Tav_opt_find = Function(obj: Pointer; Const name: PAnsiChar; Const _unit: PAnsiChar; opt_flags: int; search_flags: int): pAVOption; cdecl;
   Tav_opt_find2 = Function(obj: Pointer; Const name: PAnsiChar; Const _unit: PAnsiChar; opt_flags: int; search_flags: int; Var target_obj: Pointer): pAVOption; cdecl;
   Tav_opt_next = Function(Const obj: Pointer; Const prev: pAVOption): pAVOption; cdecl;
   Tav_opt_child_next = Function(obj: Pointer; prev: Pointer): Pointer; cdecl;
   Tav_opt_child_class_next = Function(Const parent: pAVClass; Const prev: pAVClass): pAVClass; cdecl;
   Tav_opt_set = Function(obj: Pointer; Const name: PAnsiChar; Const val: PAnsiChar; search_flags: int): int; cdecl;
   Tav_opt_set_int = Function(obj: Pointer; Const name: PAnsiChar; val: int64_t; search_flags: int): int; cdecl;
   Tav_opt_set_double = Function(obj: Pointer; Const name: PAnsiChar; val: double; search_flags: int): int; cdecl;
   Tav_opt_set_q = Function(obj: Pointer; Const name: PAnsiChar; val: AVRational; search_flags: int): int; cdecl;
   Tav_opt_set_bin = Function(obj: Pointer; Const name: PAnsiChar; Const val: puint8_t; size: int; search_flags: int): int; cdecl;
   Tav_opt_set_image_size = Function(obj: Pointer; Const name: PAnsiChar; w, h, search_flags: int): int; cdecl;
   Tav_opt_set_pixel_fmt = Function(obj: Pointer; Const name: PAnsiChar; fmt: AVPixelFormat; search_flags: int): int; cdecl;
   Tav_opt_set_sample_fmt = Function(obj: Pointer; Const name: PAnsiChar; fmt: AVSampleFormat; search_flags: int): int; cdecl;
   Tav_opt_set_video_rate = Function(obj: Pointer; Const name: PAnsiChar; val: AVRational; search_flags: int): int; cdecl;
   Tav_opt_set_channel_layout = Function(obj: Pointer; Const name: PAnsiChar; ch_layout: int64_t; search_flags: int): int; cdecl;
   Tav_opt_set_dict_val = Function(obj: Pointer; Const name: PAnsiChar; Const val: pAVDictionary; search_flags: int): int; cdecl;
   Tav_opt_get = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var out_val: puint8_t): int; cdecl;
   Tav_opt_get_int = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var out_val: int64_t): int; cdecl;
   Tav_opt_get_double = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; out_val: double): int; cdecl;
   Tav_opt_get_q = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var out_val: AVRational): int; cdecl;
   Tav_opt_get_image_size = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var w_out, h_out: int): int; cdecl;
   Tav_opt_get_pixel_fmt = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var out_fmt: AVPixelFormat): int; cdecl;
   Tav_opt_get_sample_fmt = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var out_fmt: AVSampleFormat): int; cdecl;
   Tav_opt_get_video_rate = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var out_val: AVRational): int; cdecl;
   Tav_opt_get_channel_layout = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var ch_layout: int64_t): int; cdecl;
   Tav_opt_get_dict_val = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int; Var out_val: pAVDictionary): int; cdecl;
   Tav_opt_ptr = Function(Const avclass: pAVClass; obj: Pointer; Const name: PAnsiChar): Pointer; cdecl;
   Tav_opt_freep_ranges = Procedure(Var ranges: pAVOptionRanges); cdecl;
   Tav_opt_query_ranges = Function(Var ranges: pAVOptionRanges; obj: Pointer; Const key: PAnsiChar; flags: int): int; cdecl;
   Tav_opt_copy = Function(dest: Pointer; Const src: Pointer): int; cdecl;
   Tav_opt_query_ranges_default = Function(Var ranges: pAVOptionRanges; obj: Pointer; Const key: PAnsiChar; flags: int): int; cdecl;
   Tav_opt_is_set_to_default = Function(obj: Pointer; Const o: pAVOption): int; cdecl;
   Tav_opt_is_set_to_default_by_name = Function(obj: Pointer; Const name: PAnsiChar; search_flags: int): int; cdecl;
   Tav_opt_serialize = Function(obj: Pointer; opt_flags: int; flags: int; Var buffer: PAnsiChar; Const key_val_sep: AnsiChar; Const pairs_sep: AnsiChar): int; cdecl;
   Tav_log = Procedure(avcl: Pointer; level: int; Const fmt: PAnsiChar); cdecl;
   Tav_vlog = Procedure(avcl: Pointer; level: int; Const fmt: PAnsiChar; vl: PVA_LIST); cdecl;
   Tav_log_get_level = Function(): int; cdecl;
   Tav_log_set_level = Procedure(level: int); cdecl;
   Tav_log_set_callback = Procedure(callbackproc: Tav_log_callback); cdecl;
   Tav_log_default_callback = Procedure(avcl: Pointer; level: int; Const fmt: PAnsiChar; vl: PVA_LIST); cdecl;
   Tav_default_item_name = Function(ctx: Pointer): PAnsiChar; cdecl;
   Tav_default_get_category = Function(ptr: Pointer): AVClassCategory; cdecl;
   Tav_log_format_line = Procedure(ptr: Pointer; level: int; Const fmt: PAnsiChar; vl: PVA_LIST; line: PAnsiChar; line_size: int; Var print_prefix: int); cdecl;
   Tav_log_format_line2 = Function(ptr: Pointer; level: int; Const fmt: PAnsiChar; vl: PVA_LIST; line: PAnsiChar; line_size: int; Var print_prefix: int): int; cdecl;
   Tav_log_set_flags = Procedure(arg: int); cdecl;
   Tav_log_get_flags = Function(): int; cdecl;
   Tavutil_version = Function(): unsigned; cdecl;
   Tav_version_info = Function(): PAnsiChar; cdecl;
   Tavutil_configuration = Function(): PAnsiChar; cdecl;
   Tavutil_license = Function(): PAnsiChar; cdecl;
   Tav_get_media_type_string = Function(media_type: AVMediaType): PAnsiChar; cdecl;
   Tav_get_picture_type_char = Function(pict_type: AVPictureType): AnsiChar; cdecl;
   Tav_int_list_length_for_size = Function(elsize: unsigned; Const list: Pointer; term: uint64_t): unsigned; cdecl;
   Tav_fopen_utf8 = Function(Const path: PAnsiChar; Const mode: PAnsiChar): pFile; cdecl;
   Tav_get_time_base_q = Function(): AVRational; cdecl;
   Tav_fourcc_make_string = Function(buf: PAnsiChar; fourcc: uint32_t): PAnsiChar; cdecl;
   Tav_file_map = Function(Const filename: PAnsiChar; Var bufptr: puint8_t; Var size: size_t; log_offset: int; log_ctx: Pointer): int; cdecl;
   Tav_file_unmap = Procedure(bufptr: puint8_t; size: size_t); cdecl;
   Tav_tempfile = Function(Const prefix: PAnsiChar; Var filename: PAnsiChar; log_offset: int; log_ctx: Pointer): int; cdecl;
   Tav_strerror = Function(errnum: int; errbuf: PAnsiChar; errbuf_size: size_t): int; cdecl;
   Tav_get_cpu_flags = Function(): int; cdecl;
   Tav_force_cpu_flags = Procedure(flags: int); cdecl;
   Tav_parse_cpu_caps = Function(Var flags: unsigned; Const s: PAnsiChar): int; cdecl;
   Tav_cpu_count = Function(): int; cdecl;
   Tav_cpu_max_align = Function(): size_t; cdecl;
   Tav_audio_fifo_free = Procedure(af: pAVAudioFifo); cdecl;
   Tav_audio_fifo_alloc = Function(sample_fmt: AVSampleFormat; channels: int; nb_samples: int): pAVAudioFifo; cdecl;
   Tav_audio_fifo_realloc = Function(af: pAVAudioFifo; nb_samples: int): int; cdecl;
   Tav_audio_fifo_write = Function(af: pAVAudioFifo; Var data: puint8_t; nb_samples: int): int; cdecl;
   Tav_audio_fifo_peek = Function(af: pAVAudioFifo; Var data: puint8_t; nb_samples: int): int; cdecl;
   Tav_audio_fifo_peek_at = Function(af: pAVAudioFifo; Var data: Pointer; nb_samples: int; offset: int): int; cdecl;
   Tav_audio_fifo_read = Function(af: pAVAudioFifo; Var data: Pointer; nb_samples: int): int; cdecl;
   Tav_audio_fifo_drain = Function(af: pAVAudioFifo; nb_samples: int): int; cdecl;
   Tav_audio_fifo_reset = Procedure(af: pAVAudioFifo); cdecl;
   Tav_audio_fifo_size = Function(af: pAVAudioFifo): int; cdecl;
   Tav_audio_fifo_space = Function(af: pAVAudioFifo): int; cdecl;
   Tav_strstart = Function(Const str: PAnsiChar; Const pfx: PAnsiChar; Const ptr: ppAnsiChar): int; cdecl;
   Tav_stristart = Function(Const str: PAnsiChar; Const pfx: PAnsiChar; Const ptr: ppAnsiChar): int; cdecl;
   Tav_stristr = Function(Const haystack: PAnsiChar; Const needle: PAnsiChar): PAnsiChar; cdecl;
   Tav_strnstr = Function(Const haystack: PAnsiChar; Const needle: PAnsiChar; hay_length: size_t): PAnsiChar; cdecl;
   Tav_strlcpy = Function(dst: PAnsiChar; Const src: PAnsiChar; size: size_t): size_t; cdecl;
   Tav_strlcat = Function(dst: PAnsiChar; Const src: PAnsiChar; size: size_t): size_t; cdecl;
   Tav_d2str = Function(d: double): PAnsiChar; cdecl;
   Tav_get_token = Function(Const buf: ppAnsiChar; Const term: PAnsiChar): PAnsiChar; cdecl;
   Tav_strtok = Function(s: PAnsiChar; Const delim: PAnsiChar; saveptr: ppAnsiChar): PAnsiChar; cdecl;
   Tav_strcasecmp = Function(Const a: PAnsiChar; Const b: PAnsiChar): int; cdecl;
   Tav_strncasecmp = Function(Const a: PAnsiChar; Const b: PAnsiChar; n: size_t): int; cdecl;
   Tav_strireplace = Function(Const str: PAnsiChar; Const from: PAnsiChar; Const _to: PAnsiChar): PAnsiChar; cdecl;
   Tav_basename = Function(Const path: PAnsiChar): PAnsiChar; cdecl;
   Tav_dirname = Function(path: PAnsiChar): PAnsiChar; cdecl;
   Tav_match_name = Function(Const name: PAnsiChar; Const names: PAnsiChar): int; cdecl;
   Tav_append_path_component = Function(Const path: PAnsiChar; Const component: PAnsiChar): PAnsiChar; cdecl;
   Tav_escape = Function(Var dst: PAnsiChar; Const src: PAnsiChar; Const special_chars: PAnsiChar; mode: AVEscapeMode; flags: int): int; cdecl;
   Tav_utf8_decode = Function(Var codep: int32_t; Const bufp: ppuint8_t; Const buf_end: puint8_t; flags: unsigned_int): int; cdecl;
   Tav_match_list = Function(Const name: PAnsiChar; Const list: PAnsiChar; separator: AnsiChar): int; cdecl;
   Tav_bprint_init = Procedure(buf: pAVBPrint; size_init: unsigned; size_max: unsigned); cdecl;
   Tav_bprint_init_for_buffer = Procedure(buf: pAVBPrint; buffer: PAnsiChar; size: unsigned); cdecl;
   Tav_vbprintf = Procedure(buf: pAVBPrint; Const fmt: PAnsiChar; vl_arg: PVA_LIST); cdecl;
   Tav_bprint_chars = Procedure(buf: pAVBPrint; c: AnsiChar; n: unsigned); cdecl;
   Tav_bprint_append_data = Procedure(buf: pAVBPrint; Const data: PAnsiChar; size: unsigned); cdecl;
   Tav_bprint_strftime = Procedure(buf: pAVBPrint; Const fmt: PAnsiChar; Const tm: ptm); cdecl;
   Tav_bprint_get_buffer = Procedure(buf: pAVBPrint; size: unsigned; Var mem: punsigned_char; Var actual_size: unsigned); cdecl;
   Tav_bprint_clear = Procedure(buf: pAVBPrint); cdecl;
   Tav_bprint_finalize = Function(buf: pAVBPrint; Var ret_str: PAnsiChar): int; cdecl;
   Tav_bprint_escape = Procedure(dstbuf: pAVBPrint; Const src: PAnsiChar; Const special_chars: PAnsiChar; mode: AVEscapeMode; flags: int); cdecl;
   Tav_display_rotation_get = Function(Const matrix: Tav_display_matrix): double; cdecl;
   Tav_display_rotation_set = Procedure(matrix: Tav_display_matrix; angle: double); cdecl;
   Tav_display_matrix_flip = Procedure(matrix: Tav_display_matrix; hflip: int; vflip: int); cdecl;
   Tav_expr_parse_and_eval = Function(Var res: double; Const s: PAnsiChar; const_names: ppAnsiChar; Const const_values: pdouble; func1_names: ppAnsiChar;
                                      funcs1: Tav_expr_funcs1; func2_names: ppAnsiChar; funcs2: Tav_expr_funcs2; opaque: Pointer; log_offset: int; log_ctx: Pointer): int; cdecl;
   Tav_expr_parse = Function(Var expr: pAVExpr; Const s: PAnsiChar; const_names: ppAnsiChar; func1_names: ppAnsiChar; funcs1: Tav_expr_funcs1;
                             func2_names: ppAnsiChar; funcs2: Tav_expr_funcs2; log_offset: int; log_ctx: Pointer): int; cdecl;
   Tav_expr_eval = Function(e: pAVExpr; Const const_values: pdouble; opaque: Pointer): double; cdecl;
   Tav_expr_free = Procedure(e: pAVExpr); cdecl;
   Tav_strtod = Function(Const numstr: PAnsiChar; Var tail: PAnsiChar): double; cdecl;
   Tav_fifo_alloc = Function(size: unsigned_int): pAVFifoBuffer; cdecl;
   Tav_fifo_alloc_array = Function(nmemb: size_t; size: size_t): pAVFifoBuffer; cdecl;
   Tav_fifo_free = Procedure(f: pAVFifoBuffer); cdecl;
   Tav_fifo_freep = Procedure(Var f: pAVFifoBuffer); cdecl;
   Tav_fifo_reset = Procedure(f: pAVFifoBuffer); cdecl;
   Tav_fifo_size = Function(Const f: pAVFifoBuffer): int; cdecl;
   Tav_fifo_space = Function(Const f: pAVFifoBuffer): int; cdecl;
   Tav_fifo_generic_peek_at = Function(f: pAVFifoBuffer; dest: Pointer; offset: int; buf_size: int; func: Tav_fifo_proc): int; cdecl;
   Tav_fifo_generic_peek = Function(f: pAVFifoBuffer; dest: Pointer; buf_size: int; func: Tav_fifo_proc): int; cdecl;
   Tav_fifo_generic_read = Function(f: pAVFifoBuffer; dest: Pointer; buf_size: int; func: Tav_fifo_proc): int; cdecl;
   Tav_fifo_generic_write = Function(f: pAVFifoBuffer; src: Pointer; size: int; func: Tav_fifo_func): int; cdecl;
   Tav_fifo_realloc2 = Function(f: pAVFifoBuffer; size: unsigned_int): int; cdecl;
   Tav_fifo_grow = Function(f: pAVFifoBuffer; additional_space: unsigned_int): int; cdecl;
   Tav_fifo_drain = Procedure(f: pAVFifoBuffer; size: int); cdecl;
   Tav_hwdevice_find_type_by_name = Function(Const name: PAnsiChar): AVHWDeviceType; cdecl;
   Tav_hwdevice_get_type_name = Function(_type: AVHWDeviceType): PAnsiChar; cdecl;
   Tav_hwdevice_iterate_types = Function(prev: AVHWDeviceType): AVHWDeviceType; cdecl;
   Tav_hwdevice_ctx_alloc = Function(_type: AVHWDeviceType): pAVBufferRef; cdecl;
   Tav_hwdevice_ctx_init = Function(ref: pAVBufferRef): int; cdecl;
   Tav_hwdevice_ctx_create = Function(Var device_ctx: pAVBufferRef; _type: AVHWDeviceType; Const device: PAnsiChar; opts: pAVDictionary; flags: int): int; cdecl;
   Tav_hwdevice_ctx_create_derived = Function(Var dst_ctx: pAVBufferRef; _type: AVHWDeviceType; src_ctx: pAVBufferRef; flags: int): int; cdecl;
   Tav_hwframe_ctx_alloc = Function(device_ctx: pAVBufferRef): pAVBufferRef; cdecl;
   Tav_hwframe_ctx_init = Function(ref: pAVBufferRef): int; cdecl;
   Tav_hwframe_get_buffer = Function(hwframe_ctx: pAVBufferRef; frame: pAVFrame; flags: int): int; cdecl;
   Tav_hwframe_transfer_data = Function(dst: pAVFrame; Const src: pAVFrame; flags: int): int; cdecl;
   Tav_hwframe_transfer_get_formats = Function(hwframe_ctx: pAVBufferRef; dir: AVHWFrameTransferDirection; Var formats: pAVPixelFormat; flags: int): int; cdecl;
   Tav_hwdevice_hwconfig_alloc = Function(device_ctx: pAVBufferRef): Pointer; cdecl;
   Tav_hwdevice_get_hwframe_constraints = Function(ref: pAVBufferRef; Const hwconfig: Pointer): pAVHWFramesConstraints; cdecl;
   Tav_hwframe_constraints_free = Procedure(Var constraints: pAVHWFramesConstraints); cdecl;
   Tav_hwframe_map = Function(dst: pAVFrame; Const src: pAVFrame; flags: int): int; cdecl;
   Tav_hwframe_ctx_create_derived = Function(Var derived_frame_ctx: pAVBufferRef; format: AVPixelFormat; derived_device_ctx: pAVBufferRef; source_frame_ctx: pAVBufferRef;
                                             flags: int): int; cdecl;
   Tav_get_bits_per_pixel = Function(Const pixdesc: pAVPixFmtDescriptor): int; cdecl;
   Tav_get_padded_bits_per_pixel = Function(Const pixdesc: pAVPixFmtDescriptor): int; cdecl;
   Tav_pix_fmt_desc_get = Function(pix_fmt: AVPixelFormat): pAVPixFmtDescriptor; cdecl;
   Tav_pix_fmt_desc_next = Function(Const prev: pAVPixFmtDescriptor): pAVPixFmtDescriptor; cdecl;
   Tav_pix_fmt_desc_get_id = Function(Const desc: pAVPixFmtDescriptor): AVPixelFormat; cdecl;
   Tav_pix_fmt_get_chroma_sub_sample = Function(pix_fmt: AVPixelFormat; Var h_shift: int; Var v_shift: int): int; cdecl;
   Tav_pix_fmt_count_planes = Function(pix_fmt: AVPixelFormat): int; cdecl;
   Tav_color_range_name = Function(range: AVColorRange): PAnsiChar; cdecl;
   Tav_color_range_from_name = Function(Const name: PAnsiChar): int; cdecl;
   Tav_color_primaries_name = Function(primaries: AVColorPrimaries): PAnsiChar; cdecl;
   Tav_color_primaries_from_name = Function(Const name: PAnsiChar): int; cdecl;
   Tav_color_transfer_name = Function(transfer: AVColorTransferCharacteristic): PAnsiChar; cdecl;
   Tav_color_transfer_from_name = Function(Const name: PAnsiChar): int; cdecl;
   Tav_color_space_name = Function(space: AVColorSpace): PAnsiChar; cdecl;
   Tav_color_space_from_name = Function(Const name: PAnsiChar): int; cdecl;
   Tav_chroma_location_name = Function(location: AVChromaLocation): PAnsiChar; cdecl;
   Tav_chroma_location_from_name = Function(Const name: PAnsiChar): int; cdecl;
   Tav_get_pix_fmt = Function(Const name: PAnsiChar): AVPixelFormat; cdecl;
   Tav_get_pix_fmt_name = Function(pix_fmt: AVPixelFormat): PAnsiChar; cdecl;
   Tav_get_pix_fmt_string = Function(buf: PAnsiChar; buf_size: int; pix_fmt: AVPixelFormat): PAnsiChar; cdecl;
   Tav_read_image_line2 = Procedure(dst: Pointer; Const data: pav_read_array4_puint8_t; Const linesize: pav_read_array4_int;
                                    Const desc: pAVPixFmtDescriptor; x, y, c, w, read_pal_component, dst_element_size: int); cdecl;
   Tav_read_image_line = Procedure(dst: puint16_t; Const data: pav_read_array4_puint8_t; Const linesize: pav_read_array4_int;
                                    Const desc: pAVPixFmtDescriptor; x, y, c, w: int; read_pal_component: int); cdecl;
   Tav_write_image_line2 = Procedure(Const src: puint16_t; data: pav_read_array4_puint8_t; Const linesize: pav_read_array4_int;
                                    Const desc: pAVPixFmtDescriptor; x: int; y: int; c: int; w: int; src_element_size: int); cdecl;
   Tav_write_image_line = Procedure(Const src: puint16_t; data: pav_read_array4_puint8_t; Const linesize: pav_read_array4_int;
                                    Const desc: pAVPixFmtDescriptor; x: int; y: int; c: int; w: int); cdecl;
   Tav_pix_fmt_swap_endianness = Function(pix_fmt: AVPixelFormat): AVPixelFormat; cdecl;
   Tav_get_pix_fmt_loss = Function(dst_pix_fmt: AVPixelFormat; src_pix_fmt: AVPixelFormat; has_alpha: int): int; cdecl;
   Tav_find_best_pix_fmt_of_2 = Function(dst_pix_fmt1: AVPixelFormat; dst_pix_fmt2: AVPixelFormat; src_pix_fmt: AVPixelFormat; has_alpha: int;
                                    Var loss_ptr: int): AVPixelFormat; cdecl;
   Tav_image_fill_max_pixsteps = Procedure(max_pixsteps: pav_image_array4_int; max_pixstep_comps: pav_image_array4_int; Const pixdesc: pAVPixFmtDescriptor); cdecl;
   Tav_image_get_linesize = Function(pix_fmt: AVPixelFormat; width: int; plane: int): int; cdecl;
   Tav_image_fill_linesizes = Function(linesizes: pav_image_array4_int; pix_fmt: AVPixelFormat; width: int): int; cdecl;
   Tav_image_fill_pointers = Function(data: pav_image_array4_puint8_t; pix_fmt: AVPixelFormat; height: int; ptr: puint8_t;
                                    Const linesizes: pav_image_array4_int): int; cdecl;
   Tav_image_alloc = Function(pointers: pav_image_array4_puint8_t;linesizes: pav_image_array4_int;w: int;h: int;pix_fmt: AVPixelFormat; align: int): int; cdecl;
   Tav_image_copy_plane = Procedure(dst: puint8_t; dst_linesize: int; Const src: puint8_t; src_linesize: int; bytewidth: int; height: int); cdecl;
   Tav_image_copy = Procedure(dst_data: pav_image_array4_puint8_t; dst_linesizes: pav_image_array4_int; Const src_data: pav_image_array4_puint8_t;
                              Const src_linesizes: pav_image_array4_int; pix_fmt: AVPixelFormat; width: int; height: int); cdecl;
   Tav_image_copy_uc_from = Procedure(dst_data: pav_image_array4_puint8_t; Const dst_linesizes: pav_image_array4_ptrdiff_t; Const src_data: pav_image_array4_puint8_t;
                                      Const src_linesizes: pav_image_array4_ptrdiff_t; pix_fmt: AVPixelFormat; width: int; height: int); cdecl;
   Tav_image_fill_arrays = Function(dst_data: pav_image_array4_puint8_t; dst_linesize: pav_image_array4_int; Const src: puint8_t; pix_fmt: AVPixelFormat; width: int;
                                    height: int; align: int): int; cdecl;
   Tav_image_get_buffer_size = Function(pix_fmt: AVPixelFormat; width: int; height: int; align: int): int; cdecl;
   Tav_image_copy_to_buffer = Function(dst: puint8_t; dst_size: int; Const src_data: pav_image_array4_puint8_t; Const src_linesize: pav_image_array4_int;
                                       pix_fmt: AVPixelFormat; width: int; height: int; align: int): int; cdecl;
   Tav_image_check_size = Function(w: unsigned_int; h: unsigned_int; log_offset: int; log_ctx: Pointer): int; cdecl;
   Tav_image_check_size2 = Function(w: unsigned_int; h: unsigned_int; max_pixels: int64_t; pix_fmt: AVPixelFormat; log_offset: int; log_ctx: Pointer): int; cdecl;

   Tav_image_check_sar = Function(w: unsigned_int; h: unsigned_int; sar: AVRational): int; cdecl;
   Tav_image_fill_black = Function(dst_data: pav_image_array4_puint8_t; Const dst_linesize: pav_image_array4_ptrdiff_t; pix_fmt: AVPixelFormat; range: AVColorRange;
                                   width: int; height: int): int; cdecl;
   Tav_gettime = Function(): int64_t; cdecl;
   Tav_gettime_relative = Function(): int64_t; cdecl;
   Tav_gettime_relative_is_monotonic = Function(): int; cdecl;
   Tav_usleep = Function(usec: unsigned): int; cdecl;
   Tav_malloc = Function(size: size_t): Pointer; cdecl;
   Tav_mallocz = Function(size: size_t): Pointer; cdecl;
   Tav_malloc_array = Function(nmemb: size_t; size: size_t): Pointer; cdecl;
   Tav_mallocz_array = Function(nmemb: size_t; size: size_t): Pointer; cdecl;
   Tav_calloc = Function(nmemb: size_t; size: size_t): Pointer; cdecl;
   Tav_realloc = Function(ptr: Pointer; size: size_t): Pointer; cdecl;
   Tav_reallocp = Function(ptr: Pointer; size: size_t): int; cdecl;
   Tav_realloc_f = Function(ptr: Pointer; nelem: size_t; elsize: size_t): Pointer; cdecl;
   Tav_realloc_array = Function(ptr: Pointer; nmemb: size_t; size: size_t): Pointer; cdecl;
   Tav_reallocp_array = Function(ptr: Pointer; nmemb: size_t; size: size_t): int; cdecl;
   Tav_fast_realloc = Function(ptr: Pointer; Var size: unsigned_int; min_size: size_t): Pointer; cdecl;
   Tav_fast_malloc = Procedure(ptr: Pointer; Var size: unsigned_int; min_size: size_t); cdecl;
   Tav_fast_mallocz = Procedure(ptr: Pointer; Var size: unsigned_int; min_size: size_t); cdecl;
   Tav_free = Procedure(ptr: Pointer); cdecl;
   Tav_freep = Procedure(ptr: Pointer); cdecl;
   Tav_strdup = Function(Const s: PAnsiChar): PAnsiChar; cdecl;
   Tav_strndup = Function(Const s: PAnsiChar; len: size_t): PAnsiChar; cdecl;
   Tav_memdup = Function(Const p: Pointer; size: size_t): Pointer; cdecl;
   Tav_memcpy_backptr = Procedure(dst: puint8_t; back: int; cnt: int); cdecl;
   Tav_dynarray_add = Procedure(tab_ptr: Pointer; Var nb_ptr: int; elem: Pointer); cdecl;
   Tav_dynarray_add_nofree = Function(tab_ptr: Pointer; Var nb_ptr: int; elem: Pointer): int; cdecl;
   Tav_dynarray2_add = Function(Var tab_ptr: Pointer; Var nb_ptr: int; elem_size: size_t; Const elem_data: puint8_t): Pointer; cdecl;
   Tav_max_alloc = Procedure(max: size_t); cdecl;
   Tav_timecode_adjust_ntsc_framenum2 = Function(framenum: int; fps: int): int; cdecl;
   Tav_timecode_get_smpte_from_framenum = Function(Const tc: pAVTimecode; framenum: int): uint32_t; cdecl;
   Tav_timecode_make_string = Function(Const tc: pAVTimecode; buf: PAnsiChar; framenum: int): PAnsiChar; cdecl;
   Tav_timecode_make_smpte_tc_string = Function(buf: PAnsiChar; tcsmpte: uint32_t; prevent_df: int): PAnsiChar; cdecl;
   Tav_timecode_make_mpeg_tc_string = Function(buf: PAnsiChar; tc25bit: uint32_t): PAnsiChar; cdecl;
   Tav_timecode_init = Function(tc: pAVTimecode; rate: AVRational; flags: int; rame_start: int; log_ctx: Pointer): int; cdecl;
   Tav_timecode_init_from_string = Function(tc: pAVTimecode; rate: AVRational; Const str: PAnsiChar; log_ctx: Pointer): int; cdecl;
   Tav_timecode_check_frame_rate = Function(rate: AVRational): int; cdecl;
   Tav_gcd = Function(a, b: int64_t): int64_t; cdecl;
   Tav_rescale = Function(a, b, c: int64_t): int64_t; cdecl;
   Tav_rescale_rnd = Function(a, b, c: int64_t; rnd: AVRounding): int64_t; cdecl;
   Tav_rescale_q = Function(a: int64_t; bq: AVRational; cq: AVRational): int64_t; cdecl;
   Tav_rescale_q_rnd = Function(a: int64_t; bq: AVRational; cq: AVRational; rnd: AVRounding): int64_t; cdecl;
   Tav_compare_ts = Function(ts_a: int64_t; tb_a: AVRational; ts_b: int64_t; tb_b: AVRational): int; cdecl;
   Tav_compare_mod = Function(a: uint64_t; b: uint64_t; _mod: uint64_t): int64_t; cdecl;
   Tav_rescale_delta = Function(in_tb: AVRational; in_ts: int64_t; fs_tb: AVRational; duration: int; Var last: int64_t; out_tb: AVRational): int64_t; cdecl;
   Tav_add_stable = Function(ts_tb: AVRational; ts: int64_t; inc_tb: AVRational; inc: int64_t): int64_t; cdecl;
   Tav_parse_ratio = Function(q: pAVRational; Const str: PAnsiChar; max, log_offset: int; log_ctx: Pointer): int; cdecl;
   Tav_parse_video_size = Function(Var width_ptr: int; Var height_ptr: int; Const str: PAnsiChar): int; cdecl;
   Tav_parse_video_rate = Function(rate: pAVRational; Const str: PAnsiChar): int; cdecl;
   Tav_parse_color = Function(rgba_color: puint8_t; Const color_string: PAnsiChar; slen: int; log_ctx: Pointer): int; cdecl;
   Tav_get_known_color_name = Function(color_idx: int; Const rgb: ppuint8_t): PAnsiChar; cdecl;
   Tav_parse_time = Function(timeval: pint64_t; Const timestr: PAnsiChar; duration: int): int; cdecl;
   Tav_find_info_tag = Function(arg: PAnsiChar; arg_size: int; Const tag1: PAnsiChar; Const info: PAnsiChar): int; cdecl;
   Tav_small_strptime = Function(Const p: PAnsiChar; Const fmt: PAnsiChar; dt: ptm): PAnsiChar; cdecl;
   Tav_timegm = Function(tm: ptm): time_t; cdecl;
   Tav_md5_alloc = Function(): pAVMD5; cdecl;
   Tav_md5_init = Procedure(ctx: pAVMD5); cdecl;
   Tav_md5_update = Procedure(ctx: pAVMD5; Const src: puint8_t; len: int); cdecl;
   Tav_md5_final = Procedure(ctx: pAVMD5; dst: puint8_t); cdecl;
   Tav_md5_sum = Procedure(dst: puint8_t; Const src: puint8_t; Const len: int); cdecl;
   Tav_assert0_fpu = Procedure(); cdecl;
   Tav_mastering_display_metadata_alloc = Function(): pAVMasteringDisplayMetadata; cdecl;
   Tav_mastering_display_metadata_create_side_data = Function(frame: pAVFrame): pAVMasteringDisplayMetadata; cdecl;
   Tav_content_light_metadata_alloc = Function(Var size: size_t): pAVContentLightMetadata; cdecl;
   Tav_content_light_metadata_create_side_data = Function(frame: pAVFrame): pAVContentLightMetadata; cdecl;
   Tav_pixelutils_get_sad_fn = Function(w_bits: int; h_bits: int; aligned: int; log_ctx: Pointer): av_pixelutils_sad_fn; cdecl;
   Tav_get_random_seed = Function(): uint32_t; cdecl;
   Tav_aes_alloc = Function(): pAVAES; cdecl;
   Tav_aes_init = Function(a: pAVAES; Const key: puint8_t; key_bits: int; decrypt: int): int; cdecl;
   Tav_aes_crypt = Procedure(a: pAVAES; dst: puint8_t; Const src: puint8_t; count: int; iv: puint8_t; decrypt: int); cdecl;
   Tav_twofish_alloc = Function(): pAVTWOFISH; cdecl;
   Tav_twofish_init = Function(ctx: pAVTWOFISH; Const key: puint8_t; key_bits: int): int; cdecl;
   Tav_twofish_crypt = Procedure(ctx: pAVTWOFISH; dst: puint8_t; Const src: puint8_t; count: int; iv: puint8_t; decrypt: int); cdecl;
   Tav_tea_alloc = Function(): pAVTEA; cdecl;
   Tav_tea_init = Procedure(ctx: pAVTEA; Const key: puint8_t; rounds: int); cdecl;
   Tav_tea_crypt = Procedure(ctx: pAVTEA; dst: puint8_t; Const src: puint8_t; count: int; iv: puint8_t; decrypt: int); cdecl;
   Tav_tx_init = Function(Var ctx: pAVTXContext; tx: av_tx_fn; &type: AVTXType; inv, len: int; Const scale: Pointer; flags: uint64_t): int; cdecl;
   Tav_tx_uninit = Procedure(Var ctx: pAVTXContext); cdecl;
   Tav_lfg_init = Procedure(c: pAVLFG; seed: uint); cdecl;
   Tav_lfg_init_from_data = Function(c: pAVLFG; Const data: puint8_t; length: uint): int; cdecl;
   Tav_bmg_get = Procedure(lfg: pAVLFG; &out: Tav_bmg_get_arrayofdouble); cdecl;
   Tav_aes_ctr_init = Function(var a:AVAESCTR; var key:uint8_t):longint;cdecl;
   Tav_aes_ctr_free = Procedure(var a:AVAESCTR);cdecl;
   Tav_aes_ctr_crypt = Procedure(var a:AVAESCTR; var dst:uint8_t; var src:uint8_t; size:longint);cdecl;
   Tav_aes_ctr_get_iv = Function(var a:AVAESCTR):Puint8_t;cdecl;
   Tav_aes_ctr_set_random_iv = Procedure(var a:AVAESCTR);cdecl;
   Tav_aes_ctr_set_iv = Procedure(var a:AVAESCTR; var iv:uint8_t);cdecl;
   Tav_aes_ctr_set_full_iv = Procedure(var a:AVAESCTR; var iv:uint8_t);cdecl;
   Tav_aes_ctr_increment_iv = Procedure(var a:AVAESCTR);cdecl;     
   Tav_sscanf = Function(_string:Pchar; format:Pchar; args:array of const):longint;cdecl;
   Tav_base64_decode = Function(var _out:uint8_t; ain:Pchar; out_size:longint):longint;cdecl;
   Tav_base64_encode = Function(_out:Pchar; out_size:longint; var ain:uint8_t; in_size:longint):Pchar;cdecl;
   Tav_blowfish_alloc = Function():PAVBlowfish;cdecl;
   Tav_blowfish_init = Procedure(var ctx:AVBlowfish; var key:uint8_t; key_len:longint);cdecl;
   Tav_blowfish_crypt_ecb = Procedure(var ctx:AVBlowfish; var xl:uint32_t; var xr:uint32_t; decrypt:longint);cdecl;
   Tav_blowfish_crypt = Procedure(var ctx:AVBlowfish; var dst:uint8_t; var src:uint8_t; count:longint; var iv:uint8_t;decrypt:longint);cdecl;
   Tav_buffer_replace = Function(dst: PPAVBufferRef; src: PAVBufferRef): Integer; cdecl;   
   Tav_buffer_pool_buffer_get_opaque = Procedure(ref: PAVBufferRef); cdecl;
   Tav_camellia_init = Function(var ctx:AVCAMELLIA; var key:uint8_t; key_bits:longint):longint;cdecl;
   Tav_camellia_crypt = Procedure(var ctx:AVCAMELLIA; var dst:uint8_t; var src:uint8_t; count:longint; var iv:uint8_t;decrypt:longint);cdecl;
   Tav_cast5_init = Function(var ctx:AVCAST5; var key:uint8_t; key_bits:longint):longint;cdecl;
   Tav_cast5_crypt = Procedure(var ctx:AVCAST5; var dst:uint8_t; var src:uint8_t; count:longint; decrypt:longint);cdecl;
   Tav_cast5_crypt2 = Procedure(var ctx:AVCAST5; var dst:uint8_t; var src:uint8_t; count:longint; var iv:uint8_t;decrypt:longint);cdecl;
   Tav_channel_name = Function(buf:Pchar; buf_size:size_t; channel:AVChannel):longint;cdecl;
   Tav_channel_name_bprint = Procedure(var bp:AVBPrint; channel_id:AVChannel);cdecl;
   Tav_channel_description = Function(buf:Pchar; buf_size:size_t; channel:AVChannel):longint;cdecl;
   Tav_channel_description_bprint = Procedure(var bp:AVBPrint; channel_id:AVChannel);cdecl;
   Tav_channel_layout_from_mask = Function(var channel_layout:AVChannelLayout; mask:uint64_t):longint;cdecl;
   Tav_channel_layout_from_string = Function(var channel_layout:AVChannelLayout; str:Pchar):longint;cdecl;
   Tav_channel_layout_default = Procedure(var ch_layout:AVChannelLayout; nb_channels:longint);cdecl;
   Tav_channel_layout_standard = Function(opaque:Ppointer):PAVChannelLayout;cdecl;
   Tav_channel_layout_uninit = Procedure(var channel_layout:AVChannelLayout);cdecl;
   Tav_channel_layout_copy = Function(var dst:AVChannelLayout; var src:AVChannelLayout):longint;cdecl;
   Tav_channel_layout_describe = Function(var channel_layout:AVChannelLayout; buf:Pchar; buf_size:size_t):longint;cdecl;
   Tav_channel_layout_describe_bprint = Function(var channel_layout:AVChannelLayout; var bp:AVBPrint):longint;cdecl;
   Tav_channel_layout_index_from_channel = Function(var channel_layout:AVChannelLayout; channel:AVChannel):longint;cdecl;
   Tav_channel_layout_index_from_string = Function(var channel_layout:AVChannelLayout; name:Pchar):longint;cdecl;
   Tav_channel_layout_subset = Function(var channel_layout:AVChannelLayout; mask:uint64_t):uint64_t;cdecl;
   Tav_channel_layout_check = Function(var channel_layout:AVChannelLayout):longint;cdecl;
   Tav_channel_layout_compare = Function(var chl:AVChannelLayout; var chl1:AVChannelLayout):longint;cdecl; 
   Tav_cpu_force_count = Procedure(count:longint);cdecl;
   Tav_dovi_alloc = Function(var size:size_t):PAVDOVIDecoderConfigurationRecord;cdecl;
   Tav_downmix_info_update_side_data = Function(var frame:AVFrame):PAVDownmixInfo;cdecl;
   Tav_encryption_info_alloc = Function(subsample_count:uint32_t; key_id_size:uint32_t; iv_size:uint32_t):PAVEncryptionInfo;cdecl;
   Tav_encryption_info_clone = Function(var info:AVEncryptionInfo):PAVEncryptionInfo;cdecl;
   Tav_encryption_info_free = Procedure(var info:AVEncryptionInfo);cdecl;
   Tav_encryption_info_get_side_data = Function(var side_data:uint8_t; side_data_size:size_t):PAVEncryptionInfo;cdecl;
   Tav_encryption_info_add_side_data = Function(var info:AVEncryptionInfo; var side_data_size:size_t):Puint8_t;cdecl;
   Tav_encryption_init_info_alloc = Function(system_id_size:uint32_t; num_key_ids:uint32_t; key_id_size:uint32_t; data_size:uint32_t):PAVEncryptionInitInfo;cdecl;
   Tav_encryption_init_info_free = Procedure(var info:AVEncryptionInitInfo);cdecl;
   Tav_encryption_init_info_get_side_data = Function(var side_data:uint8_t; side_data_size:size_t):PAVEncryptionInitInfo;cdecl;
   Tav_encryption_init_info_add_side_data = Function(var info:AVEncryptionInitInfo; var side_data_size:size_t):Puint8_t;cdecl;
   Tav_expr_count_vars = Function(var e:AVExpr; var counter:dword; size:longint):longint;cdecl;
   Tav_expr_count_func = Function(var e:AVExpr; var counter:dword; size:longint; arg:longint):longint;cdecl;   
   Tav_film_grain_params_alloc = Function(var size:size_t):PAVFilmGrainParams;cdecl;
   Tav_film_grain_params_create_side_data = Function(var frame:AVFrame):PAVFilmGrainParams;cdecl;
   Tav_hash_alloc = Function(var ctx:PAVHashContext; name:Pchar):longint;cdecl;
   Tav_hash_names = Function(i:longint):Pchar;cdecl;
   Tav_hash_get_name = Function(var ctx:AVHashContext):Pchar;cdecl;
   Tav_hash_get_size = Function(var ctx:AVHashContext):longint;cdecl;
   Tav_hash_init = Procedure(var ctx:AVHashContext);cdecl;
   Tav_hash_update = Procedure(var ctx:AVHashContext; var src:uint8_t; len:size_t);cdecl;
   Tav_hash_final = Procedure(var ctx:AVHashContext; var dst:uint8_t);cdecl;
   Tav_hash_final_bin = Procedure(var ctx:AVHashContext; var dst:uint8_t; size:longint);cdecl;
   Tav_hash_final_hex = Procedure(var ctx:AVHashContext; var dst:uint8_t; size:longint);cdecl;
   Tav_hash_final_b64 = Procedure(var ctx:AVHashContext; var dst:uint8_t; size:longint);cdecl;
   Tav_hash_freep = Procedure(var ctx:PAVHashContext);cdecl;
   Tav_dynamic_hdr_plus_alloc = Function(var size:size_t):PAVDynamicHDRPlus;cdecl;
   Tav_dynamic_hdr_plus_create_side_data = Function(var frame:AVFrame):PAVDynamicHDRPlus;cdecl;
   Tav_dynamic_hdr_vivid_alloc = Function(var size:size_t):PAVDynamicHDRVivid;cdecl;
   Tav_dynamic_hdr_vivid_create_side_data = Function(var frame:AVFrame):PAVDynamicHDRVivid;cdecl;
   Tav_hmac_alloc = Function(_type:AVHMACType):PAVHMAC;cdecl;
   Tav_hmac_free = Procedure(var ctx:AVHMAC);cdecl;
   Tav_hmac_init = Procedure(var ctx:AVHMAC; var key:uint8_t; keylen:dword);cdecl;
   Tav_hmac_update = Procedure(var ctx:AVHMAC; var data:uint8_t; len:dword);cdecl;
   Tav_hmac_final = Function(var ctx:AVHMAC; var _out:uint8_t; outlen:dword):longint;cdecl;
   Tav_hmac_calc = Function(var ctx:AVHMAC; var data:uint8_t; len:dword; var key:uint8_t; keylen:dword;var _out:uint8_t; outlen:dword):longint;cdecl;
   Tav_log_once = Procedure(avcl: Pointer; initial_level, subsequent_level: Integer; state: PInteger; fmt: PAnsiChar); cdecl varargs;
   Tav_image_fill_plane_sizes = Function(size: PSize_t; pix_fmt: TAVPixelFormat;height: Integer; Const linesizes: PInteger): Integer; cdecl;
   Tav_image_copy_plane_uc_from = Procedure(dst: PByte; dst_linesize: Integer;Const src: PByte; src_linesize: Integer;bytewidth: Integer; height: Integer); cdecl;
   Tav_opt_eval_int = Function(obj: Pointer; Const o: PAVOption; Const val: PAnsiChar; int_out: PInteger): Integer; cdecl;
   Tav_opt_eval_int64 = Function(obj: Pointer; Const o: PAVOption; Const val: PAnsiChar; int64_out: PInt64): Integer; cdecl;
   Tav_opt_eval_float = Function(obj: Pointer; Const o: PAVOption; Const val: PAnsiChar; float_out: PSingle): Integer; cdecl;
   Tav_opt_eval_double = Function(obj: Pointer; Const o: PAVOption; Const val: PAnsiChar; double_out: PDouble): Integer; cdecl;
   Tav_opt_eval_q = Function(obj: Pointer; Const o: PAVOption; Const val: PAnsiChar; q_out: PAVRational): Integer; cdecl;  
   Tav_opt_child_class_iterate = Function(Const parent: Pointer; iter: PPointer): Pointer; cdecl;
   Tav_size_mult = Function(a, b: size_t; r: Psize_t): Integer; cdecl;
   Tav_hwdevice_ctx_create_derived_opts = Function(dst_ctx: PPAVBufferRef;type_: TAVHWDeviceType;src_ctx: PAVBufferRef;options: PAVDictionary; flags: Integer): Integer; cdecl;

   Tav_crc_init = Function(var ctx:AVCRC; le:longint; bits:longint; poly:uint32_t; ctx_size:longint):longint;cdecl;
   Tav_crc_get_table = Function(crc_id:AVCRCId):PAVCRC;cdecl;
   Tav_csp_luma_coeffs_from_avcsp = Function(csp:AVColorSpace):PAVLumaCoefficients;cdecl;
   Tav_csp_primaries_desc_from_id = Function(prm:AVColorPrimaries):PAVColorPrimariesDesc;cdecl;
   Tav_detection_bbox_create_side_data = Function(var frame:AVFrame; nb_bboxes:uint32_t):PAVDetectionBBoxHeader;cdecl;


//-------------------------------------------------
 Var
                            
   av_log2 : Tav_log2 = Nil;
   av_log2_16bit : Tav_log2_16bit = Nil;
   av_get_channel_layout : Tav_get_channel_layout = Nil;
   av_get_extended_channel_layout : Tav_get_extended_channel_layout = Nil;
   av_get_channel_layout_string : Tav_get_channel_layout_string = Nil;
   av_bprint_channel_layout : Tav_bprint_channel_layout = Nil;
   av_get_channel_layout_nb_channels : Tav_get_channel_layout_nb_channels = Nil;
   av_get_default_channel_layout : Tav_get_default_channel_layout = Nil;
   av_get_channel_layout_channel_index : Tav_get_channel_layout_channel_index = Nil;
   av_channel_layout_extract_channel : Tav_channel_layout_extract_channel = Nil;
   av_get_channel_name : Tav_get_channel_name = Nil;
   av_get_channel_description : Tav_get_channel_description = Nil;
   av_get_standard_channel_layout : Tav_get_standard_channel_layout = Nil;
   av_dict_get : Tav_dict_get = Nil;
   av_dict_count : Tav_dict_count = Nil;
   av_dict_set : Tav_dict_set = Nil;
   av_dict_set_int : Tav_dict_set_int = Nil;
   av_dict_parse_string : Tav_dict_parse_string = Nil;
   av_dict_copy : Tav_dict_copy = Nil;
   av_dict_free : Tav_dict_free = Nil;
   av_dict_get_string : Tav_dict_get_string = Nil;
   av_buffer_alloc : Tav_buffer_alloc = Nil;
   av_buffer_allocz : Tav_buffer_allocz = Nil;
   av_buffer_create : Tav_buffer_create = Nil;
   av_buffer_default_free : Tav_buffer_default_free = Nil;
   av_buffer_ref : Tav_buffer_ref = Nil;
   av_buffer_unref : Tav_buffer_unref = Nil;
   av_buffer_is_writable : Tav_buffer_is_writable = Nil;
   av_buffer_get_opaque : Tav_buffer_get_opaque = Nil;
   av_buffer_get_ref_count : Tav_buffer_get_ref_count = Nil;
   av_buffer_make_writable : Tav_buffer_make_writable = Nil;
   av_buffer_realloc : Tav_buffer_realloc = Nil;
   av_buffer_pool_init : Tav_buffer_pool_init = Nil;
   av_buffer_pool_init2 : Tav_buffer_pool_init2 = Nil;
   av_buffer_pool_uninit : Tav_buffer_pool_uninit = Nil;
   av_buffer_pool_get : Tav_buffer_pool_get = Nil;
   av_reduce : Tav_reduce = Nil;
   av_mul_q : Tav_mul_q = Nil;
   av_div_q : Tav_div_q = Nil;
   av_add_q : Tav_add_q = Nil;
   av_sub_q : Tav_sub_q = Nil;
   av_d2q : Tav_d2q = Nil;
   av_nearer_q : Tav_nearer_q = Nil;
   av_find_nearest_q_idx : Tav_find_nearest_q_idx = Nil;
   av_q2intfloat : Tav_q2intfloat = Nil;
   av_frame_get_best_effort_timestamp : Tav_frame_get_best_effort_timestamp = Nil;
   av_frame_set_best_effort_timestamp : Tav_frame_set_best_effort_timestamp = Nil;
   av_frame_get_pkt_duration : Tav_frame_get_pkt_duration = Nil;
   av_frame_set_pkt_duration : Tav_frame_set_pkt_duration = Nil;
   av_frame_get_pkt_pos : Tav_frame_get_pkt_pos = Nil;
   av_frame_set_pkt_pos : Tav_frame_set_pkt_pos = Nil;
   av_frame_get_channel_layout : Tav_frame_get_channel_layout = Nil;
   av_frame_set_channel_layout : Tav_frame_set_channel_layout = Nil;
   av_frame_get_channels : Tav_frame_get_channels = Nil;
   av_frame_set_channels : Tav_frame_set_channels = Nil;
   av_frame_get_sample_rate : Tav_frame_get_sample_rate = Nil;
   av_frame_set_sample_rate : Tav_frame_set_sample_rate = Nil;
   av_frame_get_metadata : Tav_frame_get_metadata = Nil;
   av_frame_set_metadata : Tav_frame_set_metadata = Nil;
   av_frame_get_decode_error_flags : Tav_frame_get_decode_error_flags = Nil;
   av_frame_set_decode_error_flags : Tav_frame_set_decode_error_flags = Nil;
   av_frame_get_pkt_size : Tav_frame_get_pkt_size = Nil;
   av_frame_set_pkt_size : Tav_frame_set_pkt_size = Nil;
   av_frame_get_qp_table : Tav_frame_get_qp_table = Nil;
   av_frame_set_qp_table : Tav_frame_set_qp_table = Nil;
   av_frame_get_colorspace : Tav_frame_get_colorspace = Nil;
   av_frame_set_colorspace : Tav_frame_set_colorspace = Nil;
   av_frame_get_color_range : Tav_frame_get_color_range = Nil;
   av_frame_set_color_range : Tav_frame_set_color_range = Nil;
   av_get_colorspace_name : Tav_get_colorspace_name = Nil;
   av_frame_alloc : Tav_frame_alloc = Nil;
   av_frame_free : Tav_frame_free = Nil;
   av_frame_ref : Tav_frame_ref = Nil;
   av_frame_clone : Tav_frame_clone = Nil;
   av_frame_unref : Tav_frame_unref = Nil;
   av_frame_move_ref : Tav_frame_move_ref = Nil;
   av_frame_get_buffer : Tav_frame_get_buffer = Nil;
   av_frame_is_writable : Tav_frame_is_writable = Nil;
   av_frame_make_writable : Tav_frame_make_writable = Nil;
   av_frame_copy : Tav_frame_copy = Nil;
   av_frame_copy_props : Tav_frame_copy_props = Nil;
   av_frame_get_plane_buffer : Tav_frame_get_plane_buffer = Nil;
   av_frame_new_side_data : Tav_frame_new_side_data = Nil;
   av_frame_new_side_data_from_buf : Tav_frame_new_side_data_from_buf = Nil;
   av_frame_get_side_data : Tav_frame_get_side_data = Nil;
   av_frame_remove_side_data : Tav_frame_remove_side_data = Nil;
   av_frame_apply_cropping : Tav_frame_apply_cropping = Nil;
   av_frame_side_data_name : Tav_frame_side_data_name = Nil;
   av_get_sample_fmt_name : Tav_get_sample_fmt_name = Nil;
   av_get_sample_fmt : Tav_get_sample_fmt = Nil;
   av_get_alt_sample_fmt : Tav_get_alt_sample_fmt = Nil;
   av_get_packed_sample_fmt : Tav_get_packed_sample_fmt = Nil;
   av_get_planar_sample_fmt : Tav_get_planar_sample_fmt = Nil;
   av_get_sample_fmt_string : Tav_get_sample_fmt_string = Nil;
   av_get_bytes_per_sample : Tav_get_bytes_per_sample = Nil;
   av_sample_fmt_is_planar : Tav_sample_fmt_is_planar = Nil;
   av_samples_get_buffer_size : Tav_samples_get_buffer_size = Nil;
   av_samples_fill_arrays : Tav_samples_fill_arrays = Nil;
   av_samples_alloc : Tav_samples_alloc = Nil;
   av_samples_alloc_array_and_samples : Tav_samples_alloc_array_and_samples = Nil;
   av_samples_copy : Tav_samples_copy = Nil;
   av_samples_set_silence : Tav_samples_set_silence = Nil;
   av_opt_show2 : Tav_opt_show2 = Nil;
   av_opt_set_defaults : Tav_opt_set_defaults = Nil;
   av_opt_set_defaults2 : Tav_opt_set_defaults2 = Nil;
   av_set_options_string : Tav_set_options_string = Nil;
   av_opt_set_from_string : Tav_opt_set_from_string = Nil;
   av_opt_free : Tav_opt_free = Nil;
   av_opt_flag_is_set : Tav_opt_flag_is_set = Nil;
   av_opt_set_dict : Tav_opt_set_dict = Nil;
   av_opt_set_dict2 : Tav_opt_set_dict2 = Nil;
   av_opt_get_key_value : Tav_opt_get_key_value = Nil;
   av_opt_eval_flags : Tav_opt_eval_flags = Nil;
   av_opt_find : Tav_opt_find = Nil;
   av_opt_find2 : Tav_opt_find2 = Nil;
   av_opt_next : Tav_opt_next = Nil;
   av_opt_child_next : Tav_opt_child_next = Nil;
   av_opt_child_class_next : Tav_opt_child_class_next = Nil;
   av_opt_set : Tav_opt_set = Nil;
   av_opt_set_int : Tav_opt_set_int = Nil;
   av_opt_set_double : Tav_opt_set_double = Nil;
   av_opt_set_q : Tav_opt_set_q = Nil;
   av_opt_set_bin : Tav_opt_set_bin = Nil;
   av_opt_set_image_size : Tav_opt_set_image_size = Nil;
   av_opt_set_pixel_fmt : Tav_opt_set_pixel_fmt = Nil;
   av_opt_set_sample_fmt : Tav_opt_set_sample_fmt = Nil;
   av_opt_set_video_rate : Tav_opt_set_video_rate = Nil;
   av_opt_set_channel_layout : Tav_opt_set_channel_layout = Nil;
   av_opt_set_dict_val : Tav_opt_set_dict_val = Nil;
   av_opt_get : Tav_opt_get = Nil;
   av_opt_get_int : Tav_opt_get_int = Nil;
   av_opt_get_double : Tav_opt_get_double = Nil;
   av_opt_get_q : Tav_opt_get_q = Nil;
   av_opt_get_image_size : Tav_opt_get_image_size = Nil;
   av_opt_get_pixel_fmt : Tav_opt_get_pixel_fmt = Nil;
   av_opt_get_sample_fmt : Tav_opt_get_sample_fmt = Nil;
   av_opt_get_video_rate : Tav_opt_get_video_rate = Nil;
   av_opt_get_channel_layout : Tav_opt_get_channel_layout = Nil;
   av_opt_get_dict_val : Tav_opt_get_dict_val = Nil;
   av_opt_ptr : Tav_opt_ptr = Nil;
   av_opt_freep_ranges : Tav_opt_freep_ranges = Nil;
   av_opt_query_ranges : Tav_opt_query_ranges = Nil;
   av_opt_copy : Tav_opt_copy = Nil;
   av_opt_query_ranges_default : Tav_opt_query_ranges_default = Nil;
   av_opt_is_set_to_default : Tav_opt_is_set_to_default = Nil;
   av_opt_is_set_to_default_by_name : Tav_opt_is_set_to_default_by_name = Nil;
   av_opt_serialize : Tav_opt_serialize = Nil;
   av_log : Tav_log = Nil;
   av_vlog : Tav_vlog = Nil;
   av_log_get_level : Tav_log_get_level = Nil;
   av_log_set_level : Tav_log_set_level = Nil;
   av_log_set_callback : Tav_log_set_callback = Nil;
   av_log_default_callback : Tav_log_default_callback = Nil;
   av_default_item_name : Tav_default_item_name = Nil;
   av_default_get_category : Tav_default_get_category = Nil;
   av_log_format_line : Tav_log_format_line = Nil;
   av_log_format_line2 : Tav_log_format_line2 = Nil;
   av_log_set_flags : Tav_log_set_flags = Nil;
   av_log_get_flags : Tav_log_get_flags = Nil;
   avutil_version : Tavutil_version = Nil;
   av_version_info : Tav_version_info = Nil;
   avutil_configuration : Tavutil_configuration = Nil;
   avutil_license : Tavutil_license = Nil;
   av_get_media_type_string : Tav_get_media_type_string = Nil;
   av_get_picture_type_char : Tav_get_picture_type_char = Nil;
   av_int_list_length_for_size : Tav_int_list_length_for_size = Nil;
   av_fopen_utf8 : Tav_fopen_utf8 = Nil;
   av_get_time_base_q : Tav_get_time_base_q = Nil;
   av_fourcc_make_string : Tav_fourcc_make_string = Nil;
   av_file_map : Tav_file_map = Nil;
   av_file_unmap : Tav_file_unmap = Nil;
   av_tempfile : Tav_tempfile = Nil;
   av_strerror : Tav_strerror = Nil;
   av_get_cpu_flags : Tav_get_cpu_flags = Nil;
   av_force_cpu_flags : Tav_force_cpu_flags = Nil;
   av_parse_cpu_caps : Tav_parse_cpu_caps = Nil;
   av_cpu_count : Tav_cpu_count = Nil;
   av_cpu_max_align : Tav_cpu_max_align = Nil;
   av_audio_fifo_free : Tav_audio_fifo_free = Nil;
   av_audio_fifo_alloc : Tav_audio_fifo_alloc = Nil;
   av_audio_fifo_realloc : Tav_audio_fifo_realloc = Nil;
   av_audio_fifo_write : Tav_audio_fifo_write = Nil;
   av_audio_fifo_peek : Tav_audio_fifo_peek = Nil;
   av_audio_fifo_peek_at : Tav_audio_fifo_peek_at = Nil;
   av_audio_fifo_read : Tav_audio_fifo_read = Nil;
   av_audio_fifo_drain : Tav_audio_fifo_drain = Nil;
   av_audio_fifo_reset : Tav_audio_fifo_reset = Nil;
   av_audio_fifo_size : Tav_audio_fifo_size = Nil;
   av_audio_fifo_space : Tav_audio_fifo_space = Nil;
   av_strstart : Tav_strstart = Nil;
   av_stristart : Tav_stristart = Nil;
   av_stristr : Tav_stristr = Nil;
   av_strnstr : Tav_strnstr = Nil;
   av_strlcpy : Tav_strlcpy = Nil;
   av_strlcat : Tav_strlcat = Nil;
   av_d2str : Tav_d2str = Nil;
   av_get_token : Tav_get_token = Nil;
   av_strtok : Tav_strtok = Nil;
   av_strcasecmp : Tav_strcasecmp = Nil;
   av_strncasecmp : Tav_strncasecmp = Nil;
   av_strireplace : Tav_strireplace = Nil;
   av_basename : Tav_basename = Nil;
   av_dirname : Tav_dirname = Nil;
   av_match_name : Tav_match_name = Nil;
   av_append_path_component : Tav_append_path_component = Nil;
   av_escape : Tav_escape = Nil;
   av_utf8_decode : Tav_utf8_decode = Nil;
   av_match_list : Tav_match_list = Nil;
   av_bprint_init : Tav_bprint_init = Nil;
   av_bprint_init_for_buffer : Tav_bprint_init_for_buffer = Nil;
   av_vbprintf : Tav_vbprintf = Nil;
   av_bprint_chars : Tav_bprint_chars = Nil;
   av_bprint_append_data : Tav_bprint_append_data = Nil;
   av_bprint_strftime : Tav_bprint_strftime = Nil;
   av_bprint_get_buffer : Tav_bprint_get_buffer = Nil;
   av_bprint_clear : Tav_bprint_clear = Nil;
   av_bprint_finalize : Tav_bprint_finalize = Nil;
   av_bprint_escape : Tav_bprint_escape = Nil;
   av_display_rotation_get : Tav_display_rotation_get = Nil;
   av_display_rotation_set : Tav_display_rotation_set = Nil;
   av_display_matrix_flip : Tav_display_matrix_flip = Nil;
   av_expr_parse_and_eval : Tav_expr_parse_and_eval = Nil;
   av_expr_parse : Tav_expr_parse = Nil;
   av_expr_eval : Tav_expr_eval = Nil;
   av_expr_free : Tav_expr_free = Nil;
   av_strtod : Tav_strtod = Nil;
   av_fifo_alloc : Tav_fifo_alloc = Nil;
   av_fifo_alloc_array : Tav_fifo_alloc_array = Nil;
   av_fifo_free : Tav_fifo_free = Nil;
   av_fifo_freep : Tav_fifo_freep = Nil;
   av_fifo_reset : Tav_fifo_reset = Nil;
   av_fifo_size : Tav_fifo_size = Nil;
   av_fifo_space : Tav_fifo_space = Nil;
   av_fifo_generic_peek_at : Tav_fifo_generic_peek_at = Nil;
   av_fifo_generic_peek : Tav_fifo_generic_peek = Nil;
   av_fifo_generic_read : Tav_fifo_generic_read = Nil;
   av_fifo_generic_write : Tav_fifo_generic_write = Nil;
   av_fifo_realloc2 : Tav_fifo_realloc2 = Nil;
   av_fifo_grow : Tav_fifo_grow = Nil;
   av_fifo_drain : Tav_fifo_drain = Nil;
   av_hwdevice_find_type_by_name : Tav_hwdevice_find_type_by_name = Nil;
   av_hwdevice_get_type_name : Tav_hwdevice_get_type_name = Nil;
   av_hwdevice_iterate_types : Tav_hwdevice_iterate_types = Nil;
   av_hwdevice_ctx_alloc : Tav_hwdevice_ctx_alloc = Nil;
   av_hwdevice_ctx_init : Tav_hwdevice_ctx_init = Nil;
   av_hwdevice_ctx_create : Tav_hwdevice_ctx_create = Nil;
   av_hwdevice_ctx_create_derived : Tav_hwdevice_ctx_create_derived = Nil;
   av_hwframe_ctx_alloc : Tav_hwframe_ctx_alloc = Nil;
   av_hwframe_ctx_init : Tav_hwframe_ctx_init = Nil;
   av_hwframe_get_buffer : Tav_hwframe_get_buffer = Nil;
   av_hwframe_transfer_data : Tav_hwframe_transfer_data = Nil;
   av_hwframe_transfer_get_formats : Tav_hwframe_transfer_get_formats = Nil;
   av_hwdevice_hwconfig_alloc : Tav_hwdevice_hwconfig_alloc = Nil;
   av_hwdevice_get_hwframe_constraints : Tav_hwdevice_get_hwframe_constraints = Nil;
   av_hwframe_constraints_free : Tav_hwframe_constraints_free = Nil;
   av_hwframe_map : Tav_hwframe_map = Nil;
   av_hwframe_ctx_create_derived : Tav_hwframe_ctx_create_derived = Nil;
   av_get_bits_per_pixel : Tav_get_bits_per_pixel = Nil;
   av_get_padded_bits_per_pixel : Tav_get_padded_bits_per_pixel = Nil;
   av_pix_fmt_desc_get : Tav_pix_fmt_desc_get = Nil;
   av_pix_fmt_desc_next : Tav_pix_fmt_desc_next = Nil;
   av_pix_fmt_desc_get_id : Tav_pix_fmt_desc_get_id = Nil;
   av_pix_fmt_get_chroma_sub_sample : Tav_pix_fmt_get_chroma_sub_sample = Nil;
   av_pix_fmt_count_planes : Tav_pix_fmt_count_planes = Nil;
   av_color_range_name : Tav_color_range_name = Nil;
   av_color_range_from_name : Tav_color_range_from_name = Nil;
   av_color_primaries_name : Tav_color_primaries_name = Nil;
   av_color_primaries_from_name : Tav_color_primaries_from_name = Nil;
   av_color_transfer_name : Tav_color_transfer_name = Nil;
   av_color_transfer_from_name : Tav_color_transfer_from_name = Nil;
   av_color_space_name : Tav_color_space_name = Nil;
   av_color_space_from_name : Tav_color_space_from_name = Nil;
   av_chroma_location_name : Tav_chroma_location_name = Nil;
   av_chroma_location_from_name : Tav_chroma_location_from_name = Nil;
   av_get_pix_fmt : Tav_get_pix_fmt = Nil;
   av_get_pix_fmt_name : Tav_get_pix_fmt_name = Nil;
   av_get_pix_fmt_string : Tav_get_pix_fmt_string = Nil;
   av_read_image_line2 : Tav_read_image_line2 = Nil;
   av_read_image_line : Tav_read_image_line = Nil;
   av_write_image_line2 : Tav_write_image_line2 = Nil;
   av_write_image_line : Tav_write_image_line = Nil;
   av_pix_fmt_swap_endianness : Tav_pix_fmt_swap_endianness = Nil;
   av_get_pix_fmt_loss : Tav_get_pix_fmt_loss = Nil;
   av_find_best_pix_fmt_of_2 : Tav_find_best_pix_fmt_of_2 = Nil;
   av_image_fill_max_pixsteps : Tav_image_fill_max_pixsteps = Nil;
   av_image_get_linesize : Tav_image_get_linesize = Nil;
   av_image_fill_linesizes : Tav_image_fill_linesizes = Nil;
   av_image_fill_pointers : Tav_image_fill_pointers = Nil;
   av_image_alloc : Tav_image_alloc = Nil;
   av_image_copy_plane : Tav_image_copy_plane = Nil;
   av_image_copy : Tav_image_copy = Nil;
   av_image_copy_uc_from : Tav_image_copy_uc_from = Nil;
   av_image_fill_arrays : Tav_image_fill_arrays = Nil;
   av_image_get_buffer_size : Tav_image_get_buffer_size = Nil;
   av_image_copy_to_buffer : Tav_image_copy_to_buffer = Nil;
   av_image_check_size : Tav_image_check_size = Nil;
   av_image_check_size2 : Tav_image_check_size2 = Nil;
   av_image_check_sar : Tav_image_check_sar = Nil;
   av_image_fill_black : Tav_image_fill_black = Nil;
   av_gettime : Tav_gettime = Nil;
   av_gettime_relative : Tav_gettime_relative = Nil;
   av_gettime_relative_is_monotonic : Tav_gettime_relative_is_monotonic = Nil;
   av_usleep : Tav_usleep = Nil;
   av_malloc : Tav_malloc = Nil;
   av_mallocz : Tav_mallocz = Nil;
   av_malloc_array : Tav_malloc_array = Nil;
   av_mallocz_array : Tav_mallocz_array = Nil;
   av_calloc : Tav_calloc = Nil;
   av_realloc : Tav_realloc = Nil;
   av_reallocp : Tav_reallocp = Nil;
   av_realloc_f : Tav_realloc_f = Nil;
   av_realloc_array : Tav_realloc_array = Nil;
   av_reallocp_array : Tav_reallocp_array = Nil;
   av_fast_realloc : Tav_fast_realloc = Nil;
   av_fast_malloc : Tav_fast_malloc = Nil;
   av_fast_mallocz : Tav_fast_mallocz = Nil;
   av_free : Tav_free = Nil;
   av_freep : Tav_freep = Nil;
   av_strdup : Tav_strdup = Nil;
   av_strndup : Tav_strndup = Nil;
   av_memdup : Tav_memdup = Nil;
   av_memcpy_backptr : Tav_memcpy_backptr = Nil;
   av_dynarray_add : Tav_dynarray_add = Nil;
   av_dynarray_add_nofree : Tav_dynarray_add_nofree = Nil;
   av_dynarray2_add : Tav_dynarray2_add = Nil;
   av_max_alloc : Tav_max_alloc = Nil;
   av_timecode_adjust_ntsc_framenum2 : Tav_timecode_adjust_ntsc_framenum2 = Nil;
   av_timecode_get_smpte_from_framenum : Tav_timecode_get_smpte_from_framenum = Nil;
   av_timecode_make_string : Tav_timecode_make_string = Nil;
   av_timecode_make_smpte_tc_string : Tav_timecode_make_smpte_tc_string = Nil;
   av_timecode_make_mpeg_tc_string : Tav_timecode_make_mpeg_tc_string = Nil;
   av_timecode_init : Tav_timecode_init = Nil;
   av_timecode_init_from_string : Tav_timecode_init_from_string = Nil;
   av_timecode_check_frame_rate : Tav_timecode_check_frame_rate = Nil;
   av_gcd : Tav_gcd = Nil;
   av_rescale : Tav_rescale = Nil;
   av_rescale_rnd : Tav_rescale_rnd = Nil;
   av_rescale_q : Tav_rescale_q = Nil;
   av_rescale_q_rnd : Tav_rescale_q_rnd = Nil;
   av_compare_ts : Tav_compare_ts = Nil;
   av_compare_mod : Tav_compare_mod = Nil;
   av_rescale_delta : Tav_rescale_delta = Nil;
   av_add_stable : Tav_add_stable = Nil;
   av_parse_ratio : Tav_parse_ratio = Nil;
   av_parse_video_size : Tav_parse_video_size = Nil;
   av_parse_video_rate : Tav_parse_video_rate = Nil;
   av_parse_color : Tav_parse_color = Nil;
   av_get_known_color_name : Tav_get_known_color_name = Nil;
   av_parse_time : Tav_parse_time = Nil;
   av_find_info_tag : Tav_find_info_tag = Nil;
   av_small_strptime : Tav_small_strptime = Nil;
   av_timegm : Tav_timegm = Nil;
   av_md5_alloc : Tav_md5_alloc = Nil;
   av_md5_init : Tav_md5_init = Nil;
   av_md5_update : Tav_md5_update = Nil;
   av_md5_final : Tav_md5_final = Nil;
   av_md5_sum : Tav_md5_sum = Nil;
   av_assert0_fpu : Tav_assert0_fpu = Nil;
   av_mastering_display_metadata_alloc : Tav_mastering_display_metadata_alloc = Nil;
   av_mastering_display_metadata_create_side_data : Tav_mastering_display_metadata_create_side_data = Nil;
   av_content_light_metadata_alloc : Tav_content_light_metadata_alloc = Nil;
   av_content_light_metadata_create_side_data : Tav_content_light_metadata_create_side_data = Nil;
   av_pixelutils_get_sad_fn : Tav_pixelutils_get_sad_fn = Nil;
   av_get_random_seed : Tav_get_random_seed = Nil;
   av_aes_alloc : Tav_aes_alloc = Nil;
   av_aes_init : Tav_aes_init = Nil;
   av_aes_crypt : Tav_aes_crypt = Nil;
   av_twofish_alloc : Tav_twofish_alloc = Nil;
   av_twofish_init : Tav_twofish_init = Nil;
   av_twofish_crypt : Tav_twofish_crypt = Nil;
   av_tea_alloc : Tav_tea_alloc = Nil;
   av_tea_init : Tav_tea_init = Nil;
   av_tea_crypt : Tav_tea_crypt = Nil;
   av_tx_init : Tav_tx_init = Nil;
   av_tx_uninit : Tav_tx_uninit = Nil;
   av_lfg_init : Tav_lfg_init = Nil;
   av_lfg_init_from_data : Tav_lfg_init_from_data = Nil;
   av_bmg_get : Tav_bmg_get = Nil;    
   av_aes_ctr_init : Tav_aes_ctr_init = Nil;
   av_aes_ctr_free : Tav_aes_ctr_free = Nil;
   av_aes_ctr_crypt : Tav_aes_ctr_crypt = Nil;
   av_aes_ctr_get_iv : Tav_aes_ctr_get_iv = Nil;
   av_aes_ctr_set_random_iv : Tav_aes_ctr_set_random_iv = Nil;
   av_aes_ctr_set_iv : Tav_aes_ctr_set_iv = Nil;
   av_aes_ctr_set_full_iv : Tav_aes_ctr_set_full_iv = Nil;
   av_aes_ctr_increment_iv : Tav_aes_ctr_increment_iv = Nil;  
   av_sscanf : Tav_sscanf = Nil;
   av_base64_decode : Tav_base64_decode = Nil;
   av_base64_encode : Tav_base64_encode = Nil;
   av_blowfish_alloc : Tav_blowfish_alloc = Nil;
   av_blowfish_init : Tav_blowfish_init = Nil;
   av_blowfish_crypt_ecb : Tav_blowfish_crypt_ecb = Nil;
   av_blowfish_crypt : Tav_blowfish_crypt = Nil;  
   av_buffer_replace : Tav_buffer_replace = Nil;  
   av_buffer_pool_buffer_get_opaque : Tav_buffer_pool_buffer_get_opaque = Nil;
   av_camellia_init : Tav_camellia_init = Nil;
   av_camellia_crypt : Tav_camellia_crypt = Nil;
   av_cast5_init : Tav_cast5_init = Nil;
   av_cast5_crypt : Tav_cast5_crypt = Nil;
   av_cast5_crypt2 : Tav_cast5_crypt2 = Nil;
   av_channel_name : Tav_channel_name = Nil;
   av_channel_name_bprint : Tav_channel_name_bprint = Nil;
   av_channel_description : Tav_channel_description = Nil;
   av_channel_description_bprint : Tav_channel_description_bprint = Nil;
   av_channel_layout_from_mask : Tav_channel_layout_from_mask = Nil;
   av_channel_layout_from_string : Tav_channel_layout_from_string = Nil;
   av_channel_layout_default : Tav_channel_layout_default = Nil;
   av_channel_layout_standard : Tav_channel_layout_standard = Nil;
   av_channel_layout_uninit : Tav_channel_layout_uninit = Nil;
   av_channel_layout_copy : Tav_channel_layout_copy = Nil;
   av_channel_layout_describe : Tav_channel_layout_describe = Nil;
   av_channel_layout_describe_bprint : Tav_channel_layout_describe_bprint = Nil;
   av_channel_layout_index_from_channel : Tav_channel_layout_index_from_channel = Nil;
   av_channel_layout_index_from_string : Tav_channel_layout_index_from_string = Nil;
   av_channel_layout_subset : Tav_channel_layout_subset = Nil;
   av_channel_layout_check : Tav_channel_layout_check = Nil;
   av_channel_layout_compare : Tav_channel_layout_compare = Nil;   
   av_cpu_force_count : Tav_cpu_force_count = Nil; 
   av_dovi_alloc : Tav_dovi_alloc = Nil;
   av_downmix_info_update_side_data : Tav_downmix_info_update_side_data = Nil;
   av_encryption_info_alloc : Tav_encryption_info_alloc = Nil;
   av_encryption_info_clone : Tav_encryption_info_clone = Nil;
   av_encryption_info_free : Tav_encryption_info_free = Nil;
   av_encryption_info_get_side_data : Tav_encryption_info_get_side_data = Nil;
   av_encryption_info_add_side_data : Tav_encryption_info_add_side_data = Nil;
   av_encryption_init_info_alloc : Tav_encryption_init_info_alloc = Nil;
   av_encryption_init_info_free : Tav_encryption_init_info_free = Nil;
   av_encryption_init_info_get_side_data : Tav_encryption_init_info_get_side_data = Nil;
   av_encryption_init_info_add_side_data : Tav_encryption_init_info_add_side_data = Nil;  
   av_expr_count_vars : Tav_expr_count_vars = Nil;
   av_expr_count_func : Tav_expr_count_func = Nil;  
   av_film_grain_params_alloc : Tav_film_grain_params_alloc = Nil;
   av_film_grain_params_create_side_data : Tav_film_grain_params_create_side_data = Nil;  
   av_hash_alloc : Tav_hash_alloc = Nil;
   av_hash_names : Tav_hash_names = Nil;
   av_hash_get_name : Tav_hash_get_name = Nil;
   av_hash_get_size : Tav_hash_get_size = Nil;
   av_hash_init : Tav_hash_init = Nil;
   av_hash_update : Tav_hash_update = Nil;
   av_hash_final : Tav_hash_final = Nil;
   av_hash_final_bin : Tav_hash_final_bin = Nil;
   av_hash_final_hex : Tav_hash_final_hex = Nil;
   av_hash_final_b64 : Tav_hash_final_b64 = Nil;
   av_hash_freep : Tav_hash_freep = Nil;
   av_dynamic_hdr_plus_alloc : Tav_dynamic_hdr_plus_alloc = Nil;
   av_dynamic_hdr_plus_create_side_data : Tav_dynamic_hdr_plus_create_side_data = Nil;
   av_dynamic_hdr_vivid_alloc : Tav_dynamic_hdr_vivid_alloc = Nil;
   av_dynamic_hdr_vivid_create_side_data : Tav_dynamic_hdr_vivid_create_side_data = Nil;
   av_hmac_alloc : Tav_hmac_alloc = Nil;
   av_hmac_free : Tav_hmac_free = Nil;
   av_hmac_init : Tav_hmac_init = Nil;
   av_hmac_update : Tav_hmac_update = Nil;
   av_hmac_final : Tav_hmac_final = Nil;
   av_hmac_calc : Tav_hmac_calc = Nil;    
   av_log_once : Tav_log_once = Nil; 
   av_image_fill_plane_sizes : Tav_image_fill_plane_sizes = Nil; 
   av_image_copy_plane_uc_from : Tav_image_copy_plane_uc_from = Nil;  
   av_opt_eval_int : Tav_opt_eval_int = Nil;
   av_opt_eval_int64 : Tav_opt_eval_int64 = Nil;
   av_opt_eval_float : Tav_opt_eval_float = Nil;
   av_opt_eval_double : Tav_opt_eval_double = Nil;
   av_opt_eval_q : Tav_opt_eval_q = Nil;   
   av_opt_child_class_iterate : Tav_opt_child_class_iterate = Nil;    
   av_size_mult : Tav_size_mult = Nil;  
   av_hwdevice_ctx_create_derived_opts : Tav_hwdevice_ctx_create_derived_opts = Nil;   
   av_crc_init : Tav_crc_init = Nil;
   av_crc_get_table : Tav_crc_get_table = Nil;
   av_csp_luma_coeffs_from_avcsp : Tav_csp_luma_coeffs_from_avcsp = Nil;
   av_csp_primaries_desc_from_id : Tav_csp_primaries_desc_from_id = Nil;
   av_detection_bbox_create_side_data : Tav_detection_bbox_create_side_data = Nil;

//-------------- Utils -----------------------------------
 
Function RSHIFT(a, b: int): int; Inline;
Function ROUNDED_DIV(a, b: int): int; Inline;
Function FFUDIV(a, b: int): int; Inline;
Function FFUMOD(a, b: int): int; Inline;
Function FFABS(a: int): int; Inline;
Function FFSIGN(a: int): int; Inline;
Function FFNABS(a: int): int; Inline;
Function FFDIFFSIGN(x, y: int): Boolean; Inline;
Function FFMAX(a, b: int): int; Inline;
Function av_clip_c(a: int; amin: int; amax: int): int; Inline;
Function av_clip64_c(a: int64_t; amin: int64_t; amax: int64_t): int64_t; Inline;
Function av_clip_uint8_c(a: int): uint8_t; Inline;
Function av_clip_int8_c(a: int): int8_t; Inline;
Function av_clip_uint16_c(a: int): uint16_t; Inline;
Function av_clip_int16_c(a: int): int16_t; Inline;
Function av_clipl_int32_c(a: int64_t): int32_t; Inline;
Function av_clip_intp2_c(a: int; p: int): int; Inline;
Function av_clip_uintp2_c(a, p: int): unsigned; Inline;
Function av_mod_uintp2_c(a, p: unsigned): unsigned; Inline;
Function av_sat_add32_c(a, b: int): int; Inline;
Function av_sat_dadd32_c(a, b: int): int; Inline;
Function av_sat_sub32_c(a, b: int): int; Inline;
Function av_sat_dsub32_c(a, b: int): int; Inline;
Function av_clipf_c(a, amin, amax: float): float; Inline;
Function av_clipd_c(a, amin, amax: double): double; Inline;
Function av_ceil_log2_c(x: int): int; Inline;
Function av_popcount_c(x: uint32_t): int; Inline;
Function av_parity_c(v: uint32_t): int; Inline;
Function av_popcount64_c(x: uint64_t): int; Inline;
Function av_make_q(_num: int; _den: int): AVRational; Inline;
Function av_cmp_q(a, b: AVRational): int; Inline;
Function av_q2d(a: AVRational): double; Inline;
Function av_inv_q(q: AVRational): AVRational; Inline;
Function av_x_if_null(Const p: Pointer; Const x: Pointer): Pointer; Inline;
Function av_int_list_length(list: Pointer; item_size: int; term: int64_t): int; Inline;
Function av_make_error_string(errbuf: PAnsiChar; errbuf_size: size_t; errnum: int): PAnsiChar; Inline;
Function av_err2str(errnum: int): PAnsiChar;
Function av_strnlen(Const s: PAnsiChar; len: size_t): size_t; Inline;
Function av_isdigit(c: int): Boolean; Inline;
Function av_isgraph(c: int): Boolean; Inline;
Function av_isspace(c1: int): Boolean; Inline;
Function av_toupper(c1: int): int; Inline;
Function av_tolower(c1: int): int; Inline;
Function av_isxdigit(c1: int): Boolean; Inline;
Function av_bprint_is_complete(Const buf: pAVBPrint): Boolean; Inline;
Function av_fifo_peek2(Const f: pAVFifoBuffer; offs: int): puint8_t; Inline;
Function av_parse_ratio_quiet(q: pAVRational; Const str: PAnsiChar; max: int): int;Inline;
Function av_int2float(i: uint32_t): float; Inline;
Function av_float2int(f: float): uint32_t; Inline;
Function av_int2double(i: uint64_t): double; Inline;
Function av_double2int(f: double): uint64_t; Inline;
Function av_lfg_get(c: pAVLFG): uint; Inline;
Function av_mlfg_get(c: pAVLFG): uint; Inline;
Function av_opt_set_int_list(obj: Pointer; name: PAnsiChar; list: Pointer; item_size: int; term: int64_t; flags: int): Integer; Inline;
Function av_ts_make_string(buf: PAnsiChar; ts: int64_t): PAnsiChar;
Function av_ts2str(ts: int64_t): PAnsiChar;
Function av_ts_make_time_string(buf: PAnsiChar; ts: int64_t; tb: pAVRational): PAnsiChar;
Function av_ts2timestr(ts: int64_t; tb: pAVRational): PAnsiChar;

//--------------------------------------------------------


Function  libavutil_Load(Const aLibName: String = avutilLibName): Boolean;
Procedure libavutil_Free;
Function  libavutil_IsLoaded: boolean;

//==============================================================
//==============================================================
implementation

Const
   InvalidLibHandle = 0;

var
  avutil_Handle: TLibHandle;

function libavutil_IsLoaded: boolean;
begin
  Result := (avutil_Handle <> InvalidLibHandle);
end;

Procedure libavutil_Free;
begin
  if libavutil_IsLoaded then
  begin
    DynLibs.UnloadLibrary(avutil_Handle);
    avutil_Handle := InvalidLibHandle;
  end;
end;

function libavutil_Load(Const aLibName: String = avutilLibName): Boolean;
//.............................
   Function _GetProcAddress(Const aProcName: String): {$IFDEF cpui8086}FarPointer{$ELSE}Pointer{$ENDIF};
   Begin
      result := Nil;
      If aProcName = '' Then
         exit;

      result := dynlibs.GetProcedureAddress(avutil_Handle, AnsiString(aProcName));
   End;
   //.....................................
var
  thelib: string;

{$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
  ssl:Integer;
  ssp:Integer;
{$ENDIF}
begin
  Result := False;

  if avutil_Handle <> InvalidLibHandle then Exit(True);

  if aLibName = '' then
     thelib := avutilLibName else
     thelib := aLibName;

  avutil_Handle := dynlibs.LoadLibrary(thelib);

   //.....................................
   // On UNIX If Library NOT load, try to load .so file
  {$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
   If avutil_Handle = 0 Then
   Begin
      ssl := length(thelib);
      ssp := Pos('.so.', thelib);
      If ssp > 0 Then
         delete(thelib, ssp, ssl - ssp + 1);

      avutil_Handle := dynlibs.LoadLibrary(thelib + '.so');
   End;
  {$ENDIF}

    if avutil_Handle = InvalidLibHandle then Exit(False);

    //.........................................
                                     
    Pointer(av_log2) := _GetProcAddress('av_log2');
    Pointer(av_log2_16bit) := _GetProcAddress('av_log2_16bit');
    Pointer(av_get_channel_layout) := _GetProcAddress('av_get_channel_layout');
    Pointer(av_get_extended_channel_layout) := _GetProcAddress('av_get_extended_channel_layout');
    Pointer(av_get_channel_layout_string) := _GetProcAddress('av_get_channel_layout_string');
    Pointer(av_bprint_channel_layout) := _GetProcAddress('av_bprint_channel_layout');
    Pointer(av_get_channel_layout_nb_channels) := _GetProcAddress('av_get_channel_layout_nb_channels');
    Pointer(av_get_default_channel_layout) := _GetProcAddress('av_get_default_channel_layout');
    Pointer(av_get_channel_layout_channel_index) := _GetProcAddress('av_get_channel_layout_channel_index');
    Pointer(av_channel_layout_extract_channel) := _GetProcAddress('av_channel_layout_extract_channel');
    Pointer(av_get_channel_name) := _GetProcAddress('av_get_channel_name');
    Pointer(av_get_channel_description) := _GetProcAddress('av_get_channel_description');
    Pointer(av_get_standard_channel_layout) := _GetProcAddress('av_get_standard_channel_layout');
    Pointer(av_dict_get) := _GetProcAddress('av_dict_get');
    Pointer(av_dict_count) := _GetProcAddress('av_dict_count');
    Pointer(av_dict_set) := _GetProcAddress('av_dict_set');
    Pointer(av_dict_set_int) := _GetProcAddress('av_dict_set_int');
    Pointer(av_dict_parse_string) := _GetProcAddress('av_dict_parse_string');
    Pointer(av_dict_copy) := _GetProcAddress('av_dict_copy');
    Pointer(av_dict_free) := _GetProcAddress('av_dict_free');
    Pointer(av_dict_get_string) := _GetProcAddress('av_dict_get_string');
    Pointer(av_buffer_alloc) := _GetProcAddress('av_buffer_alloc');
    Pointer(av_buffer_allocz) := _GetProcAddress('av_buffer_allocz');
    Pointer(av_buffer_create) := _GetProcAddress('av_buffer_create');
    Pointer(av_buffer_default_free) := _GetProcAddress('av_buffer_default_free');
    Pointer(av_buffer_ref) := _GetProcAddress('av_buffer_ref');
    Pointer(av_buffer_unref) := _GetProcAddress('av_buffer_unref');
    Pointer(av_buffer_is_writable) := _GetProcAddress('av_buffer_is_writable');
    Pointer(av_buffer_get_opaque) := _GetProcAddress('av_buffer_get_opaque');
    Pointer(av_buffer_get_ref_count) := _GetProcAddress('av_buffer_get_ref_count');
    Pointer(av_buffer_make_writable) := _GetProcAddress('av_buffer_make_writable');
    Pointer(av_buffer_realloc) := _GetProcAddress('av_buffer_realloc');
    Pointer(av_buffer_pool_init) := _GetProcAddress('av_buffer_pool_init');
    Pointer(av_buffer_pool_init2) := _GetProcAddress('av_buffer_pool_init2');
    Pointer(av_buffer_pool_uninit) := _GetProcAddress('av_buffer_pool_uninit');
    Pointer(av_buffer_pool_get) := _GetProcAddress('av_buffer_pool_get');
    Pointer(av_reduce) := _GetProcAddress('av_reduce');
    Pointer(av_mul_q) := _GetProcAddress('av_mul_q');
    Pointer(av_div_q) := _GetProcAddress('av_div_q');
    Pointer(av_add_q) := _GetProcAddress('av_add_q');
    Pointer(av_sub_q) := _GetProcAddress('av_sub_q');
    Pointer(av_d2q) := _GetProcAddress('av_d2q');
    Pointer(av_nearer_q) := _GetProcAddress('av_nearer_q');
    Pointer(av_find_nearest_q_idx) := _GetProcAddress('av_find_nearest_q_idx');
    Pointer(av_q2intfloat) := _GetProcAddress('av_q2intfloat');
    Pointer(av_frame_get_best_effort_timestamp) := _GetProcAddress('av_frame_get_best_effort_timestamp');
    Pointer(av_frame_set_best_effort_timestamp) := _GetProcAddress('av_frame_set_best_effort_timestamp');
    Pointer(av_frame_get_pkt_duration) := _GetProcAddress('av_frame_get_pkt_duration');
    Pointer(av_frame_set_pkt_duration) := _GetProcAddress('av_frame_set_pkt_duration');
    Pointer(av_frame_get_pkt_pos) := _GetProcAddress('av_frame_get_pkt_pos');
    Pointer(av_frame_set_pkt_pos) := _GetProcAddress('av_frame_set_pkt_pos');
    Pointer(av_frame_get_channel_layout) := _GetProcAddress('av_frame_get_channel_layout');
    Pointer(av_frame_set_channel_layout) := _GetProcAddress('av_frame_set_channel_layout');
    Pointer(av_frame_get_channels) := _GetProcAddress('av_frame_get_channels');
    Pointer(av_frame_set_channels) := _GetProcAddress('av_frame_set_channels');
    Pointer(av_frame_get_sample_rate) := _GetProcAddress('av_frame_get_sample_rate');
    Pointer(av_frame_set_sample_rate) := _GetProcAddress('av_frame_set_sample_rate');
    Pointer(av_frame_get_metadata) := _GetProcAddress('av_frame_get_metadata');
    Pointer(av_frame_set_metadata) := _GetProcAddress('av_frame_set_metadata');
    Pointer(av_frame_get_decode_error_flags) := _GetProcAddress('av_frame_get_decode_error_flags');
    Pointer(av_frame_set_decode_error_flags) := _GetProcAddress('av_frame_set_decode_error_flags');
    Pointer(av_frame_get_pkt_size) := _GetProcAddress('av_frame_get_pkt_size');
    Pointer(av_frame_set_pkt_size) := _GetProcAddress('av_frame_set_pkt_size');
    Pointer(av_frame_get_qp_table) := _GetProcAddress('av_frame_get_qp_table');
    Pointer(av_frame_set_qp_table) := _GetProcAddress('av_frame_set_qp_table');
    Pointer(av_frame_get_colorspace) := _GetProcAddress('av_frame_get_colorspace');
    Pointer(av_frame_set_colorspace) := _GetProcAddress('av_frame_set_colorspace');
    Pointer(av_frame_get_color_range) := _GetProcAddress('av_frame_get_color_range');
    Pointer(av_frame_set_color_range) := _GetProcAddress('av_frame_set_color_range');
    Pointer(av_get_colorspace_name) := _GetProcAddress('av_get_colorspace_name');
    Pointer(av_frame_alloc) := _GetProcAddress('av_frame_alloc');
    Pointer(av_frame_free) := _GetProcAddress('av_frame_free');
    Pointer(av_frame_ref) := _GetProcAddress('av_frame_ref');
    Pointer(av_frame_clone) := _GetProcAddress('av_frame_clone');
    Pointer(av_frame_unref) := _GetProcAddress('av_frame_unref');
    Pointer(av_frame_move_ref) := _GetProcAddress('av_frame_move_ref');
    Pointer(av_frame_get_buffer) := _GetProcAddress('av_frame_get_buffer');
    Pointer(av_frame_is_writable) := _GetProcAddress('av_frame_is_writable');
    Pointer(av_frame_make_writable) := _GetProcAddress('av_frame_make_writable');
    Pointer(av_frame_copy) := _GetProcAddress('av_frame_copy');
    Pointer(av_frame_copy_props) := _GetProcAddress('av_frame_copy_props');
    Pointer(av_frame_get_plane_buffer) := _GetProcAddress('av_frame_get_plane_buffer');
    Pointer(av_frame_new_side_data) := _GetProcAddress('av_frame_new_side_data');
    Pointer(av_frame_new_side_data_from_buf) := _GetProcAddress('av_frame_new_side_data_from_buf');
    Pointer(av_frame_get_side_data) := _GetProcAddress('av_frame_get_side_data');
    Pointer(av_frame_remove_side_data) := _GetProcAddress('av_frame_remove_side_data');
    Pointer(av_frame_apply_cropping) := _GetProcAddress('av_frame_apply_cropping');
    Pointer(av_frame_side_data_name) := _GetProcAddress('av_frame_side_data_name');
    Pointer(av_get_sample_fmt_name) := _GetProcAddress('av_get_sample_fmt_name');
    Pointer(av_get_sample_fmt) := _GetProcAddress('av_get_sample_fmt');
    Pointer(av_get_alt_sample_fmt) := _GetProcAddress('av_get_alt_sample_fmt');
    Pointer(av_get_packed_sample_fmt) := _GetProcAddress('av_get_packed_sample_fmt');
    Pointer(av_get_planar_sample_fmt) := _GetProcAddress('av_get_planar_sample_fmt');
    Pointer(av_get_sample_fmt_string) := _GetProcAddress('av_get_sample_fmt_string');
    Pointer(av_get_bytes_per_sample) := _GetProcAddress('av_get_bytes_per_sample');
    Pointer(av_sample_fmt_is_planar) := _GetProcAddress('av_sample_fmt_is_planar');
    Pointer(av_samples_get_buffer_size) := _GetProcAddress('av_samples_get_buffer_size');
    Pointer(av_samples_fill_arrays) := _GetProcAddress('av_samples_fill_arrays');
    Pointer(av_samples_alloc) := _GetProcAddress('av_samples_alloc');
    Pointer(av_samples_alloc_array_and_samples) := _GetProcAddress('av_samples_alloc_array_and_samples');
    Pointer(av_samples_copy) := _GetProcAddress('av_samples_copy');
    Pointer(av_samples_set_silence) := _GetProcAddress('av_samples_set_silence');
    Pointer(av_opt_show2) := _GetProcAddress('av_opt_show2');
    Pointer(av_opt_set_defaults) := _GetProcAddress('av_opt_set_defaults');
    Pointer(av_opt_set_defaults2) := _GetProcAddress('av_opt_set_defaults2');
    Pointer(av_set_options_string) := _GetProcAddress('av_set_options_string');
    Pointer(av_opt_set_from_string) := _GetProcAddress('av_opt_set_from_string');
    Pointer(av_opt_free) := _GetProcAddress('av_opt_free');
    Pointer(av_opt_flag_is_set) := _GetProcAddress('av_opt_flag_is_set');
    Pointer(av_opt_set_dict) := _GetProcAddress('av_opt_set_dict');
    Pointer(av_opt_set_dict2) := _GetProcAddress('av_opt_set_dict2');
    Pointer(av_opt_get_key_value) := _GetProcAddress('av_opt_get_key_value');
    Pointer(av_opt_eval_flags) := _GetProcAddress('av_opt_eval_flags');
    Pointer(av_opt_find) := _GetProcAddress('av_opt_find');
    Pointer(av_opt_find2) := _GetProcAddress('av_opt_find2');
    Pointer(av_opt_next) := _GetProcAddress('av_opt_next');
    Pointer(av_opt_child_next) := _GetProcAddress('av_opt_child_next');
    Pointer(av_opt_child_class_next) := _GetProcAddress('av_opt_child_class_next');
    Pointer(av_opt_set) := _GetProcAddress('av_opt_set');
    Pointer(av_opt_set_int) := _GetProcAddress('av_opt_set_int');
    Pointer(av_opt_set_double) := _GetProcAddress('av_opt_set_double');
    Pointer(av_opt_set_q) := _GetProcAddress('av_opt_set_q');
    Pointer(av_opt_set_bin) := _GetProcAddress('av_opt_set_bin');
    Pointer(av_opt_set_image_size) := _GetProcAddress('av_opt_set_image_size');
    Pointer(av_opt_set_pixel_fmt) := _GetProcAddress('av_opt_set_pixel_fmt');
    Pointer(av_opt_set_sample_fmt) := _GetProcAddress('av_opt_set_sample_fmt');
    Pointer(av_opt_set_video_rate) := _GetProcAddress('av_opt_set_video_rate');
    Pointer(av_opt_set_channel_layout) := _GetProcAddress('av_opt_set_channel_layout');
    Pointer(av_opt_set_dict_val) := _GetProcAddress('av_opt_set_dict_val');
    Pointer(av_opt_get) := _GetProcAddress('av_opt_get');
    Pointer(av_opt_get_int) := _GetProcAddress('av_opt_get_int');
    Pointer(av_opt_get_double) := _GetProcAddress('av_opt_get_double');
    Pointer(av_opt_get_q) := _GetProcAddress('av_opt_get_q');
    Pointer(av_opt_get_image_size) := _GetProcAddress('av_opt_get_image_size');
    Pointer(av_opt_get_pixel_fmt) := _GetProcAddress('av_opt_get_pixel_fmt');
    Pointer(av_opt_get_sample_fmt) := _GetProcAddress('av_opt_get_sample_fmt');
    Pointer(av_opt_get_video_rate) := _GetProcAddress('av_opt_get_video_rate');
    Pointer(av_opt_get_channel_layout) := _GetProcAddress('av_opt_get_channel_layout');
    Pointer(av_opt_get_dict_val) := _GetProcAddress('av_opt_get_dict_val');
    Pointer(av_opt_ptr) := _GetProcAddress('av_opt_ptr');
    Pointer(av_opt_freep_ranges) := _GetProcAddress('av_opt_freep_ranges');
    Pointer(av_opt_query_ranges) := _GetProcAddress('av_opt_query_ranges');
    Pointer(av_opt_copy) := _GetProcAddress('av_opt_copy');
    Pointer(av_opt_query_ranges_default) := _GetProcAddress('av_opt_query_ranges_default');
    Pointer(av_opt_is_set_to_default) := _GetProcAddress('av_opt_is_set_to_default');
    Pointer(av_opt_is_set_to_default_by_name) := _GetProcAddress('av_opt_is_set_to_default_by_name');
    Pointer(av_opt_serialize) := _GetProcAddress('av_opt_serialize');
    Pointer(av_log) := _GetProcAddress('av_log');
    Pointer(av_vlog) := _GetProcAddress('av_vlog');
    Pointer(av_log_get_level) := _GetProcAddress('av_log_get_level');
    Pointer(av_log_set_level) := _GetProcAddress('av_log_set_level');
    Pointer(av_log_set_callback) := _GetProcAddress('av_log_set_callback');
    Pointer(av_log_default_callback) := _GetProcAddress('av_log_default_callback');
    Pointer(av_default_item_name) := _GetProcAddress('av_default_item_name');
    Pointer(av_default_get_category) := _GetProcAddress('av_default_get_category');
    Pointer(av_log_format_line) := _GetProcAddress('av_log_format_line');
    Pointer(av_log_format_line2) := _GetProcAddress('av_log_format_line2');
    Pointer(av_log_set_flags) := _GetProcAddress('av_log_set_flags');
    Pointer(av_log_get_flags) := _GetProcAddress('av_log_get_flags');
    Pointer(avutil_version) := _GetProcAddress('avutil_version');
    Pointer(av_version_info) := _GetProcAddress('av_version_info');
    Pointer(avutil_configuration) := _GetProcAddress('avutil_configuration');
    Pointer(avutil_license) := _GetProcAddress('avutil_license');
    Pointer(av_get_media_type_string) := _GetProcAddress('av_get_media_type_string');
    Pointer(av_get_picture_type_char) := _GetProcAddress('av_get_picture_type_char');
    Pointer(av_int_list_length_for_size) := _GetProcAddress('av_int_list_length_for_size');
    Pointer(av_fopen_utf8) := _GetProcAddress('av_fopen_utf8');
    Pointer(av_get_time_base_q) := _GetProcAddress('av_get_time_base_q');
    Pointer(av_fourcc_make_string) := _GetProcAddress('av_fourcc_make_string');
    Pointer(av_file_map) := _GetProcAddress('av_file_map');
    Pointer(av_file_unmap) := _GetProcAddress('av_file_unmap');
    Pointer(av_tempfile) := _GetProcAddress('av_tempfile');
    Pointer(av_strerror) := _GetProcAddress('av_strerror');
    Pointer(av_get_cpu_flags) := _GetProcAddress('av_get_cpu_flags');
    Pointer(av_force_cpu_flags) := _GetProcAddress('av_force_cpu_flags');
    Pointer(av_parse_cpu_caps) := _GetProcAddress('av_parse_cpu_caps');
    Pointer(av_cpu_count) := _GetProcAddress('av_cpu_count');
    Pointer(av_cpu_max_align) := _GetProcAddress('av_cpu_max_align');
    Pointer(av_audio_fifo_free) := _GetProcAddress('av_audio_fifo_free');
    Pointer(av_audio_fifo_alloc) := _GetProcAddress('av_audio_fifo_alloc');
    Pointer(av_audio_fifo_realloc) := _GetProcAddress('av_audio_fifo_realloc');
    Pointer(av_audio_fifo_write) := _GetProcAddress('av_audio_fifo_write');
    Pointer(av_audio_fifo_peek) := _GetProcAddress('av_audio_fifo_peek');
    Pointer(av_audio_fifo_peek_at) := _GetProcAddress('av_audio_fifo_peek_at');
    Pointer(av_audio_fifo_read) := _GetProcAddress('av_audio_fifo_read');
    Pointer(av_audio_fifo_drain) := _GetProcAddress('av_audio_fifo_drain');
    Pointer(av_audio_fifo_reset) := _GetProcAddress('av_audio_fifo_reset');
    Pointer(av_audio_fifo_size) := _GetProcAddress('av_audio_fifo_size');
    Pointer(av_audio_fifo_space) := _GetProcAddress('av_audio_fifo_space');
    Pointer(av_strstart) := _GetProcAddress('av_strstart');
    Pointer(av_stristart) := _GetProcAddress('av_stristart');
    Pointer(av_stristr) := _GetProcAddress('av_stristr');
    Pointer(av_strnstr) := _GetProcAddress('av_strnstr');
    Pointer(av_strlcpy) := _GetProcAddress('av_strlcpy');
    Pointer(av_strlcat) := _GetProcAddress('av_strlcat');
    Pointer(av_d2str) := _GetProcAddress('av_d2str');
    Pointer(av_get_token) := _GetProcAddress('av_get_token');
    Pointer(av_strtok) := _GetProcAddress('av_strtok');
    Pointer(av_strcasecmp) := _GetProcAddress('av_strcasecmp');
    Pointer(av_strncasecmp) := _GetProcAddress('av_strncasecmp');
    Pointer(av_strireplace) := _GetProcAddress('av_strireplace');
    Pointer(av_basename) := _GetProcAddress('av_basename');
    Pointer(av_dirname) := _GetProcAddress('av_dirname');
    Pointer(av_match_name) := _GetProcAddress('av_match_name');
    Pointer(av_append_path_component) := _GetProcAddress('av_append_path_component');
    Pointer(av_escape) := _GetProcAddress('av_escape');
    Pointer(av_utf8_decode) := _GetProcAddress('av_utf8_decode');
    Pointer(av_match_list) := _GetProcAddress('av_match_list');
    Pointer(av_bprint_init) := _GetProcAddress('av_bprint_init');
    Pointer(av_bprint_init_for_buffer) := _GetProcAddress('av_bprint_init_for_buffer');
    Pointer(av_vbprintf) := _GetProcAddress('av_vbprintf');
    Pointer(av_bprint_chars) := _GetProcAddress('av_bprint_chars');
    Pointer(av_bprint_append_data) := _GetProcAddress('av_bprint_append_data');
    Pointer(av_bprint_strftime) := _GetProcAddress('av_bprint_strftime');
    Pointer(av_bprint_get_buffer) := _GetProcAddress('av_bprint_get_buffer');
    Pointer(av_bprint_clear) := _GetProcAddress('av_bprint_clear');
    Pointer(av_bprint_finalize) := _GetProcAddress('av_bprint_finalize');
    Pointer(av_bprint_escape) := _GetProcAddress('av_bprint_escape');
    Pointer(av_display_rotation_get) := _GetProcAddress('av_display_rotation_get');
    Pointer(av_display_rotation_set) := _GetProcAddress('av_display_rotation_set');
    Pointer(av_display_matrix_flip) := _GetProcAddress('av_display_matrix_flip');
    Pointer(av_expr_parse_and_eval) := _GetProcAddress('av_expr_parse_and_eval');
    Pointer(av_expr_parse) := _GetProcAddress('av_expr_parse');
    Pointer(av_expr_eval) := _GetProcAddress('av_expr_eval');
    Pointer(av_expr_free) := _GetProcAddress('av_expr_free');
    Pointer(av_strtod) := _GetProcAddress('av_strtod');
    Pointer(av_fifo_alloc) := _GetProcAddress('av_fifo_alloc');
    Pointer(av_fifo_alloc_array) := _GetProcAddress('av_fifo_alloc_array');
    Pointer(av_fifo_free) := _GetProcAddress('av_fifo_free');
    Pointer(av_fifo_freep) := _GetProcAddress('av_fifo_freep');
    Pointer(av_fifo_reset) := _GetProcAddress('av_fifo_reset');
    Pointer(av_fifo_size) := _GetProcAddress('av_fifo_size');
    Pointer(av_fifo_space) := _GetProcAddress('av_fifo_space');
    Pointer(av_fifo_generic_peek_at) := _GetProcAddress('av_fifo_generic_peek_at');
    Pointer(av_fifo_generic_peek) := _GetProcAddress('av_fifo_generic_peek');
    Pointer(av_fifo_generic_read) := _GetProcAddress('av_fifo_generic_read');
    Pointer(av_fifo_generic_write) := _GetProcAddress('av_fifo_generic_write');
    Pointer(av_fifo_realloc2) := _GetProcAddress('av_fifo_realloc2');
    Pointer(av_fifo_grow) := _GetProcAddress('av_fifo_grow');
    Pointer(av_fifo_drain) := _GetProcAddress('av_fifo_drain');
    Pointer(av_hwdevice_find_type_by_name) := _GetProcAddress('av_hwdevice_find_type_by_name');
    Pointer(av_hwdevice_get_type_name) := _GetProcAddress('av_hwdevice_get_type_name');
    Pointer(av_hwdevice_iterate_types) := _GetProcAddress('av_hwdevice_iterate_types');
    Pointer(av_hwdevice_ctx_alloc) := _GetProcAddress('av_hwdevice_ctx_alloc');
    Pointer(av_hwdevice_ctx_init) := _GetProcAddress('av_hwdevice_ctx_init');
    Pointer(av_hwdevice_ctx_create) := _GetProcAddress('av_hwdevice_ctx_create');
    Pointer(av_hwdevice_ctx_create_derived) := _GetProcAddress('av_hwdevice_ctx_create_derived');
    Pointer(av_hwframe_ctx_alloc) := _GetProcAddress('av_hwframe_ctx_alloc');
    Pointer(av_hwframe_ctx_init) := _GetProcAddress('av_hwframe_ctx_init');
    Pointer(av_hwframe_get_buffer) := _GetProcAddress('av_hwframe_get_buffer');
    Pointer(av_hwframe_transfer_data) := _GetProcAddress('av_hwframe_transfer_data');
    Pointer(av_hwframe_transfer_get_formats) := _GetProcAddress('av_hwframe_transfer_get_formats');
    Pointer(av_hwdevice_hwconfig_alloc) := _GetProcAddress('av_hwdevice_hwconfig_alloc');
    Pointer(av_hwdevice_get_hwframe_constraints) := _GetProcAddress('av_hwdevice_get_hwframe_constraints');
    Pointer(av_hwframe_constraints_free) := _GetProcAddress('av_hwframe_constraints_free');
    Pointer(av_hwframe_map) := _GetProcAddress('av_hwframe_map');
    Pointer(av_hwframe_ctx_create_derived) := _GetProcAddress('av_hwframe_ctx_create_derived');
    Pointer(av_get_bits_per_pixel) := _GetProcAddress('av_get_bits_per_pixel');
    Pointer(av_get_padded_bits_per_pixel) := _GetProcAddress('av_get_padded_bits_per_pixel');
    Pointer(av_pix_fmt_desc_get) := _GetProcAddress('av_pix_fmt_desc_get');
    Pointer(av_pix_fmt_desc_next) := _GetProcAddress('av_pix_fmt_desc_next');
    Pointer(av_pix_fmt_desc_get_id) := _GetProcAddress('av_pix_fmt_desc_get_id');
    Pointer(av_pix_fmt_get_chroma_sub_sample) := _GetProcAddress('av_pix_fmt_get_chroma_sub_sample');
    Pointer(av_pix_fmt_count_planes) := _GetProcAddress('av_pix_fmt_count_planes');
    Pointer(av_color_range_name) := _GetProcAddress('av_color_range_name');
    Pointer(av_color_range_from_name) := _GetProcAddress('av_color_range_from_name');
    Pointer(av_color_primaries_name) := _GetProcAddress('av_color_primaries_name');
    Pointer(av_color_primaries_from_name) := _GetProcAddress('av_color_primaries_from_name');
    Pointer(av_color_transfer_name) := _GetProcAddress('av_color_transfer_name');
    Pointer(av_color_transfer_from_name) := _GetProcAddress('av_color_transfer_from_name');
    Pointer(av_color_space_name) := _GetProcAddress('av_color_space_name');
    Pointer(av_color_space_from_name) := _GetProcAddress('av_color_space_from_name');
    Pointer(av_chroma_location_name) := _GetProcAddress('av_chroma_location_name');
    Pointer(av_chroma_location_from_name) := _GetProcAddress('av_chroma_location_from_name');
    Pointer(av_get_pix_fmt) := _GetProcAddress('av_get_pix_fmt');
    Pointer(av_get_pix_fmt_name) := _GetProcAddress('av_get_pix_fmt_name');
    Pointer(av_get_pix_fmt_string) := _GetProcAddress('av_get_pix_fmt_string');
    Pointer(av_read_image_line2) := _GetProcAddress('av_read_image_line2');
    Pointer(av_read_image_line) := _GetProcAddress('av_read_image_line');
    Pointer(av_write_image_line2) := _GetProcAddress('av_write_image_line2');
    Pointer(av_write_image_line) := _GetProcAddress('av_write_image_line');
    Pointer(av_pix_fmt_swap_endianness) := _GetProcAddress('av_pix_fmt_swap_endianness');
    Pointer(av_get_pix_fmt_loss) := _GetProcAddress('av_get_pix_fmt_loss');
    Pointer(av_find_best_pix_fmt_of_2) := _GetProcAddress('av_find_best_pix_fmt_of_2');
    Pointer(av_image_fill_max_pixsteps) := _GetProcAddress('av_image_fill_max_pixsteps');
    Pointer(av_image_get_linesize) := _GetProcAddress('av_image_get_linesize');
    Pointer(av_image_fill_linesizes) := _GetProcAddress('av_image_fill_linesizes');
    Pointer(av_image_fill_pointers) := _GetProcAddress('av_image_fill_pointers');
    Pointer(av_image_alloc) := _GetProcAddress('av_image_alloc');
    Pointer(av_image_copy_plane) := _GetProcAddress('av_image_copy_plane');
    Pointer(av_image_copy) := _GetProcAddress('av_image_copy');
    Pointer(av_image_copy_uc_from) := _GetProcAddress('av_image_copy_uc_from');
    Pointer(av_image_fill_arrays) := _GetProcAddress('av_image_fill_arrays');
    Pointer(av_image_get_buffer_size) := _GetProcAddress('av_image_get_buffer_size');
    Pointer(av_image_copy_to_buffer) := _GetProcAddress('av_image_copy_to_buffer');
    Pointer(av_image_check_size) := _GetProcAddress('av_image_check_size');
    Pointer(av_image_check_size2) := _GetProcAddress('av_image_check_size2');
    Pointer(av_image_check_sar) := _GetProcAddress('av_image_check_sar');
    Pointer(av_image_fill_black) := _GetProcAddress('av_image_fill_black');
    Pointer(av_gettime) := _GetProcAddress('av_gettime');
    Pointer(av_gettime_relative) := _GetProcAddress('av_gettime_relative');
    Pointer(av_gettime_relative_is_monotonic) := _GetProcAddress('av_gettime_relative_is_monotonic');
    Pointer(av_usleep) := _GetProcAddress('av_usleep');
    Pointer(av_malloc) := _GetProcAddress('av_malloc');
    Pointer(av_mallocz) := _GetProcAddress('av_mallocz');
    Pointer(av_malloc_array) := _GetProcAddress('av_malloc_array');
    Pointer(av_mallocz_array) := _GetProcAddress('av_mallocz_array');
    Pointer(av_calloc) := _GetProcAddress('av_calloc');
    Pointer(av_realloc) := _GetProcAddress('av_realloc');
    Pointer(av_reallocp) := _GetProcAddress('av_reallocp');
    Pointer(av_realloc_f) := _GetProcAddress('av_realloc_f');
    Pointer(av_realloc_array) := _GetProcAddress('av_realloc_array');
    Pointer(av_reallocp_array) := _GetProcAddress('av_reallocp_array');
    Pointer(av_fast_realloc) := _GetProcAddress('av_fast_realloc');
    Pointer(av_fast_malloc) := _GetProcAddress('av_fast_malloc');
    Pointer(av_fast_mallocz) := _GetProcAddress('av_fast_mallocz');
    Pointer(av_free) := _GetProcAddress('av_free');
    Pointer(av_freep) := _GetProcAddress('av_freep');
    Pointer(av_strdup) := _GetProcAddress('av_strdup');
    Pointer(av_strndup) := _GetProcAddress('av_strndup');
    Pointer(av_memdup) := _GetProcAddress('av_memdup');
    Pointer(av_memcpy_backptr) := _GetProcAddress('av_memcpy_backptr');
    Pointer(av_dynarray_add) := _GetProcAddress('av_dynarray_add');
    Pointer(av_dynarray_add_nofree) := _GetProcAddress('av_dynarray_add_nofree');
    Pointer(av_dynarray2_add) := _GetProcAddress('av_dynarray2_add');
    Pointer(av_max_alloc) := _GetProcAddress('av_max_alloc');
    Pointer(av_timecode_adjust_ntsc_framenum2) := _GetProcAddress('av_timecode_adjust_ntsc_framenum2');
    Pointer(av_timecode_get_smpte_from_framenum) := _GetProcAddress('av_timecode_get_smpte_from_framenum');
    Pointer(av_timecode_make_string) := _GetProcAddress('av_timecode_make_string');
    Pointer(av_timecode_make_smpte_tc_string) := _GetProcAddress('av_timecode_make_smpte_tc_string');
    Pointer(av_timecode_make_mpeg_tc_string) := _GetProcAddress('av_timecode_make_mpeg_tc_string');
    Pointer(av_timecode_init) := _GetProcAddress('av_timecode_init');
    Pointer(av_timecode_init_from_string) := _GetProcAddress('av_timecode_init_from_string');
    Pointer(av_timecode_check_frame_rate) := _GetProcAddress('av_timecode_check_frame_rate');
    Pointer(av_gcd) := _GetProcAddress('av_gcd');
    Pointer(av_rescale) := _GetProcAddress('av_rescale');
    Pointer(av_rescale_rnd) := _GetProcAddress('av_rescale_rnd');
    Pointer(av_rescale_q) := _GetProcAddress('av_rescale_q');
    Pointer(av_rescale_q_rnd) := _GetProcAddress('av_rescale_q_rnd');
    Pointer(av_compare_ts) := _GetProcAddress('av_compare_ts');
    Pointer(av_compare_mod) := _GetProcAddress('av_compare_mod');
    Pointer(av_rescale_delta) := _GetProcAddress('av_rescale_delta');
    Pointer(av_add_stable) := _GetProcAddress('av_add_stable');
    Pointer(av_parse_ratio) := _GetProcAddress('av_parse_ratio');
    Pointer(av_parse_video_size) := _GetProcAddress('av_parse_video_size');
    Pointer(av_parse_video_rate) := _GetProcAddress('av_parse_video_rate');
    Pointer(av_parse_color) := _GetProcAddress('av_parse_color');
    Pointer(av_get_known_color_name) := _GetProcAddress('av_get_known_color_name');
    Pointer(av_parse_time) := _GetProcAddress('av_parse_time');
    Pointer(av_find_info_tag) := _GetProcAddress('av_find_info_tag');
    Pointer(av_small_strptime) := _GetProcAddress('av_small_strptime');
    Pointer(av_timegm) := _GetProcAddress('av_timegm');
    Pointer(av_md5_alloc) := _GetProcAddress('av_md5_alloc');
    Pointer(av_md5_init) := _GetProcAddress('av_md5_init');
    Pointer(av_md5_update) := _GetProcAddress('av_md5_update');
    Pointer(av_md5_final) := _GetProcAddress('av_md5_final');
    Pointer(av_md5_sum) := _GetProcAddress('av_md5_sum');
    Pointer(av_assert0_fpu) := _GetProcAddress('av_assert0_fpu');
    Pointer(av_mastering_display_metadata_alloc) := _GetProcAddress('av_mastering_display_metadata_alloc');
    Pointer(av_mastering_display_metadata_create_side_data) := _GetProcAddress('av_mastering_display_metadata_create_side_data');
    Pointer(av_content_light_metadata_alloc) := _GetProcAddress('av_content_light_metadata_alloc');
    Pointer(av_content_light_metadata_create_side_data) := _GetProcAddress('av_content_light_metadata_create_side_data');
    Pointer(av_pixelutils_get_sad_fn) := _GetProcAddress('av_pixelutils_get_sad_fn');
    Pointer(av_get_random_seed) := _GetProcAddress('av_get_random_seed');
    Pointer(av_aes_alloc) := _GetProcAddress('av_aes_alloc');
    Pointer(av_aes_init) := _GetProcAddress('av_aes_init');
    Pointer(av_aes_crypt) := _GetProcAddress('av_aes_crypt');
    Pointer(av_twofish_alloc) := _GetProcAddress('av_twofish_alloc');
    Pointer(av_twofish_init) := _GetProcAddress('av_twofish_init');
    Pointer(av_twofish_crypt) := _GetProcAddress('av_twofish_crypt');
    Pointer(av_tea_alloc) := _GetProcAddress('av_tea_alloc');
    Pointer(av_tea_init) := _GetProcAddress('av_tea_init');
    Pointer(av_tea_crypt) := _GetProcAddress('av_tea_crypt');
    Pointer(av_tx_init) := _GetProcAddress('av_tx_init');
    Pointer(av_tx_uninit) := _GetProcAddress('av_tx_uninit');
    Pointer(av_lfg_init) := _GetProcAddress('av_lfg_init');
    Pointer(av_lfg_init_from_data) := _GetProcAddress('av_lfg_init_from_data');
    Pointer(av_bmg_get) := _GetProcAddress('av_bmg_get');   
    Pointer(av_aes_ctr_init) := _GetProcAddress('av_aes_ctr_init');
    Pointer(av_aes_ctr_free) := _GetProcAddress('av_aes_ctr_free');
    Pointer(av_aes_ctr_crypt) := _GetProcAddress('av_aes_ctr_crypt');
    Pointer(av_aes_ctr_get_iv) := _GetProcAddress('av_aes_ctr_get_iv');
    Pointer(av_aes_ctr_set_random_iv) := _GetProcAddress('av_aes_ctr_set_random_iv');
    Pointer(av_aes_ctr_set_iv) := _GetProcAddress('av_aes_ctr_set_iv');
    Pointer(av_aes_ctr_set_full_iv) := _GetProcAddress('av_aes_ctr_set_full_iv');
    Pointer(av_aes_ctr_increment_iv) := _GetProcAddress('av_aes_ctr_increment_iv');  
    Pointer(av_sscanf) := _GetProcAddress('av_sscanf');
    Pointer(av_base64_decode) := _GetProcAddress('av_base64_decode');
    Pointer(av_base64_encode) := _GetProcAddress('av_base64_encode');
    Pointer(av_blowfish_alloc) := _GetProcAddress('av_blowfish_alloc');
    Pointer(av_blowfish_init) := _GetProcAddress('av_blowfish_init');
    Pointer(av_blowfish_crypt_ecb) := _GetProcAddress('av_blowfish_crypt_ecb');
    Pointer(av_blowfish_crypt) := _GetProcAddress('av_blowfish_crypt');
    Pointer(av_buffer_replace) := _GetProcAddress('av_buffer_replace'); 
    Pointer(av_buffer_pool_buffer_get_opaque) := _GetProcAddress('av_buffer_pool_buffer_get_opaque');
    Pointer(av_camellia_init) := _GetProcAddress('av_camellia_init');
    Pointer(av_camellia_crypt) := _GetProcAddress('av_camellia_crypt');
    Pointer(av_cast5_init) := _GetProcAddress('av_cast5_init');
    Pointer(av_cast5_crypt) := _GetProcAddress('av_cast5_crypt');
    Pointer(av_cast5_crypt2) := _GetProcAddress('av_cast5_crypt2');
    Pointer(av_channel_name) := _GetProcAddress('av_channel_name');
    Pointer(av_channel_name_bprint) := _GetProcAddress('av_channel_name_bprint');
    Pointer(av_channel_description) := _GetProcAddress('av_channel_description');
    Pointer(av_channel_description_bprint) := _GetProcAddress('av_channel_description_bprint');
    Pointer(av_channel_layout_from_mask) := _GetProcAddress('av_channel_layout_from_mask');
    Pointer(av_channel_layout_from_string) := _GetProcAddress('av_channel_layout_from_string');
    Pointer(av_channel_layout_default) := _GetProcAddress('av_channel_layout_default');
    Pointer(av_channel_layout_standard) := _GetProcAddress('av_channel_layout_standard');
    Pointer(av_channel_layout_uninit) := _GetProcAddress('av_channel_layout_uninit');
    Pointer(av_channel_layout_copy) := _GetProcAddress('av_channel_layout_copy');
    Pointer(av_channel_layout_describe) := _GetProcAddress('av_channel_layout_describe');
    Pointer(av_channel_layout_describe_bprint) := _GetProcAddress('av_channel_layout_describe_bprint');
    Pointer(av_channel_layout_index_from_channel) := _GetProcAddress('av_channel_layout_index_from_channel');
    Pointer(av_channel_layout_index_from_string) := _GetProcAddress('av_channel_layout_index_from_string');
    Pointer(av_channel_layout_subset) := _GetProcAddress('av_channel_layout_subset');
    Pointer(av_channel_layout_check) := _GetProcAddress('av_channel_layout_check');
    Pointer(av_channel_layout_compare) := _GetProcAddress('av_channel_layout_compare');  
    Pointer(av_cpu_force_count) := _GetProcAddress('av_cpu_force_count');
    Pointer(av_cpu_max_align) := _GetProcAddress('av_cpu_max_align');  
    Pointer(av_dovi_alloc) := _GetProcAddress('av_dovi_alloc');
    Pointer(av_downmix_info_update_side_data) := _GetProcAddress('av_downmix_info_update_side_data');
    Pointer(av_encryption_info_alloc) := _GetProcAddress('av_encryption_info_alloc');
    Pointer(av_encryption_info_clone) := _GetProcAddress('av_encryption_info_clone');
    Pointer(av_encryption_info_free) := _GetProcAddress('av_encryption_info_free');
    Pointer(av_encryption_info_get_side_data) := _GetProcAddress('av_encryption_info_get_side_data');
    Pointer(av_encryption_info_add_side_data) := _GetProcAddress('av_encryption_info_add_side_data');
    Pointer(av_encryption_init_info_alloc) := _GetProcAddress('av_encryption_init_info_alloc');
    Pointer(av_encryption_init_info_free) := _GetProcAddress('av_encryption_init_info_free');
    Pointer(av_encryption_init_info_get_side_data) := _GetProcAddress('av_encryption_init_info_get_side_data');
    Pointer(av_encryption_init_info_add_side_data) := _GetProcAddress('av_encryption_init_info_add_side_data');    
    Pointer(av_expr_count_vars) := _GetProcAddress('av_expr_count_vars');
    Pointer(av_expr_count_func) := _GetProcAddress('av_expr_count_func'); 
    Pointer(av_film_grain_params_alloc) := _GetProcAddress('av_film_grain_params_alloc');
    Pointer(av_film_grain_params_create_side_data) := _GetProcAddress('av_film_grain_params_create_side_data'); 
    Pointer(av_hash_alloc) := _GetProcAddress('av_hash_alloc');
    Pointer(av_hash_names) := _GetProcAddress('av_hash_names');
    Pointer(av_hash_get_name) := _GetProcAddress('av_hash_get_name');
    Pointer(av_hash_get_size) := _GetProcAddress('av_hash_get_size');
    Pointer(av_hash_init) := _GetProcAddress('av_hash_init');
    Pointer(av_hash_update) := _GetProcAddress('av_hash_update');
    Pointer(av_hash_final) := _GetProcAddress('av_hash_final');
    Pointer(av_hash_final_bin) := _GetProcAddress('av_hash_final_bin');
    Pointer(av_hash_final_hex) := _GetProcAddress('av_hash_final_hex');
    Pointer(av_hash_final_b64) := _GetProcAddress('av_hash_final_b64');
    Pointer(av_hash_freep) := _GetProcAddress('av_hash_freep');
    Pointer(av_dynamic_hdr_plus_alloc) := _GetProcAddress('av_dynamic_hdr_plus_alloc');
    Pointer(av_dynamic_hdr_plus_create_side_data) := _GetProcAddress('av_dynamic_hdr_plus_create_side_data');
    Pointer(av_dynamic_hdr_vivid_alloc) := _GetProcAddress('av_dynamic_hdr_vivid_alloc');
    Pointer(av_dynamic_hdr_vivid_create_side_data) := _GetProcAddress('av_dynamic_hdr_vivid_create_side_data');
    Pointer(av_hmac_alloc) := _GetProcAddress('av_hmac_alloc');
    Pointer(av_hmac_free) := _GetProcAddress('av_hmac_free');
    Pointer(av_hmac_init) := _GetProcAddress('av_hmac_init');
    Pointer(av_hmac_update) := _GetProcAddress('av_hmac_update');
    Pointer(av_hmac_final) := _GetProcAddress('av_hmac_final');
    Pointer(av_hmac_calc) := _GetProcAddress('av_hmac_calc');     
    Pointer(av_log_once) := _GetProcAddress('av_log_once');   
    Pointer(av_image_fill_plane_sizes) := _GetProcAddress('av_image_fill_plane_sizes');  
    Pointer(av_image_copy_plane_uc_from) := _GetProcAddress('av_image_copy_plane_uc_from');  
    Pointer(av_opt_eval_int) := _GetProcAddress('av_opt_eval_int');
    Pointer(av_opt_eval_int64) := _GetProcAddress('av_opt_eval_int64');
    Pointer(av_opt_eval_float) := _GetProcAddress('av_opt_eval_float');
    Pointer(av_opt_eval_double) := _GetProcAddress('av_opt_eval_double');
    Pointer(av_opt_eval_q) := _GetProcAddress('av_opt_eval_q');
    Pointer(av_opt_child_class_iterate) := _GetProcAddress('av_opt_child_class_iterate');  
    Pointer(av_size_mult) := _GetProcAddress('av_size_mult');   
    Pointer(av_hwdevice_ctx_create_derived_opts) := _GetProcAddress('av_hwdevice_ctx_create_derived_opts'); 
    Pointer(av_crc_init) := _GetProcAddress('av_crc_init');
    Pointer(av_crc_get_table) := _GetProcAddress('av_crc_get_table');
    Pointer(av_csp_luma_coeffs_from_avcsp) := _GetProcAddress('av_csp_luma_coeffs_from_avcsp');
    Pointer(av_csp_primaries_desc_from_id) := _GetProcAddress('av_csp_primaries_desc_from_id');
    Pointer(av_detection_bbox_create_side_data) := _GetProcAddress('av_detection_bbox_create_side_data');

    //.........................................

    Result := True;

end;


//=======================================================================
//=======================================================================
//=======================================================================

Function RSHIFT(a, b: int): int;Inline;
Begin
   If a > 0 Then
      Result := ((a) + ((1 Shl (b)) Shr 1)) Shr (b)
   Else
      Result := ((a) + ((1 Shl (b)) Shr 1) - 1) Shr (b);
End;

Function ROUNDED_DIV(a, b: int): int;Inline;
Begin
   If a > 0 Then
      Result := a + (b Shr 1)
   Else
      Result := a - (b Shr 1) Div b;
End;

Function FFUDIV(a, b: int): int;Inline;
Begin
   If a > 0 Then
      Result := a
   Else
      Result := a - b + 1;
   Result := Result Div b;

End;

Function FFUMOD(a, b: int): int;Inline;
Begin
   Result := a - b * FFUDIV(a, b);
End;

Function FFABS(a: int): int;Inline;
Begin
   If a >= 0 Then
      Result := a
   Else
      Result := -a;
End;

Function FFSIGN(a: int): int;Inline;
Begin
   If a > 0 Then
      Result := 1
   Else
      Result := -1;
End;

Function FFNABS(a: int): int;Inline;
Begin
   If a <= 0 Then
      Result := a
   Else
      Result := -a;
End;

Function FFDIFFSIGN(x, y: int): Boolean;Inline;
Begin
   Result := FFSIGN(x) <> FFSIGN(y);
End;

Function FFMAX(a, b: int): int;Inline;
Begin
   If a > b Then
      Result := a
   Else
      Result := b;
End;

Function av_clip_c(a: int; amin: int; amax: int): int;Inline;
Begin

   If (a < amin) Then
      Result := amin
   Else If (a > amax) Then
      Result := amax
   Else
      Result := a;
End;

Function av_clip64_c(a: int64_t; amin: int64_t; amax: int64_t): int64_t;Inline;
Begin

   If (a < amin) Then
      Result := amin
   Else If (a > amax) Then
      Result := amax
   Else
      Result := a;
End;

Function av_clip_uint8_c(a: int): uint8_t;Inline;
Begin

   If (a And (Not $FF)) <> 0 Then
      Result := (Not a) Shr 31
   Else
      Result := a;
End;

Function av_clip_int8_c(a: int): int8_t;Inline;
Begin
   If ((a + $80) And (Not $FF)) <> 0 Then
      Result := (a Shr 31) Xor $7F
   Else
      Result := a;
End;

Function av_clip_uint16_c(a: int): uint16_t;Inline;
Begin
   If (a And (Not $FFFF)) <> 0 Then
      Result := (Not a) Shr 31
   Else
      Result := a;
End;

Function av_clip_int16_c(a: int): int16_t;Inline;
Begin
   If ((a + $8000) And (Not $FFFF)) <> 0 Then
      Result := (a Shr 31) Xor $7FFF
   Else
      Result := a;
End;

Function av_clipl_int32_c(a: int64_t): int32_t;Inline;
Begin
   If ((a + $80000000) And (Not $FFFFFFFF)) <> 0 Then
      Result := ((a Shr 63) Xor $7FFFFFFF)
   Else
      Result := a;
End;

Function av_clip_intp2_c(a: int; p: int): int;Inline;
Begin
   If ((a + (1 Shl p)) And (Not ((2 Shl p) - 1))) <> 0 Then
      Result := (a Shr 31) Xor ((1 Shl p) - 1)
   Else
      Result := a;
End;

Function av_clip_uintp2_c(a: int; p: int): uint;Inline;
Begin
   If (a And (Not ((1 Shl p) - 1))) <> 0 Then
      Result := (Not a) Shr 31 And ((1 Shl p) - 1)
   Else
      Result := a;
End;

Function av_mod_uintp2_c(a: uint; p: uint): uint;Inline;
Begin
   Result := a And ((uint(1) Shl p) - 1);
End;

Function av_sat_add32_c(a: int; b: int): int;Inline;
Begin
   Result := av_clipl_int32_c(a + b);
End;

Function av_sat_dadd32_c(a: int; b: int): int;Inline;
Begin
   Result := av_sat_add32_c(a, av_sat_add32_c(b, b));
End;

Function av_sat_sub32_c(a: int; b: int): int;Inline;
Begin
   Result := av_clipl_int32_c(a - b);
End;

Function av_sat_dsub32_c(a: int; b: int): int; Inline;
Begin
   Result := av_sat_sub32_c(a, av_sat_add32_c(b, b));
End;

Function av_clipf_c(a: float; amin: float; amax: float): float;Inline;
Begin
   If a < amin Then
      Result := amin
   Else If a > amax Then
      Result := amax
   Else
      Result := a;
End;

Function av_clipd_c(a: double; amin: double; amax: double): double;Inline;
Begin
   If a < amin Then
      Result := amin
   Else If a > amax Then
      Result := amax
   Else
      Result := a;
End;

Function av_ceil_log2_c(x: int): int;Inline;
Begin
   Result := av_log2((x - 1) Shl 1);
End;

Function av_popcount_c(x: uint32_t): int; Inline;
Begin
   x := x - ((x Shr 1) And $55555555);
   x := (x And $33333333) + ((x Shr 2) And $33333333);
   x := (x + (x Shr 4)) And $0F0F0F0F;
   x := x + (x Shr 8);
   Result := (x + (x Shr 16)) And $3F;
End;

Function av_popcount64_c(x: uint64_t): int;Inline;
Begin
   Result := av_popcount_c(x) + av_popcount_c(x Shr 32);
End;

Function av_parity_c(v: uint32_t): int; Inline;
Begin
   Result := av_popcount_c(v) And 1;
End;

Function av_make_q(_num: int; _den: int): AVRational; Inline;
Begin
   Result.num := _num;
   Result.den := _den;
End;

Function av_cmp_q(a, b: AVRational): int; Inline;
Var
   tmp: int64_t;
Begin
   tmp := a.num * b.den - b.num * a.den;
   If (tmp <> 0) Then
      Result := ((tmp Xor a.den Xor b.den) Shr 63) Or 1
   Else If (b.den And a.den) <> 0 Then
      Result := 0
   Else If (a.num And b.num) <> 0 Then
      Result := (a.num Shr 31) - (b.num Shr 31)
   Else
      Result := -MaxInt;
End;

Function av_q2d(a: AVRational): double; Inline;
Begin
   Result := a.num / a.den;
End;

Function av_inv_q(q: AVRational): AVRational; Inline;
Begin
   Result.den := q.den;
   Result.num := q.num;
End;

Function av_x_if_null(Const p: Pointer; Const x: Pointer): Pointer;Inline;
Begin

   If Assigned(p) Then
      Result := p
   Else
      Result := x;
End;

Function av_opt_set_int_list(obj: Pointer; name: PAnsiChar; list: Pointer; item_size: int; term: int64_t; flags: int): Integer;Inline;
Begin
   If av_int_list_length(list, item_size, term) > MaxInt / item_size Then
      Result := AVERROR_EINVAL
   Else
      Result := av_opt_set_bin(obj, name, puint8_t(list), av_int_list_length(list, item_size, term) * item_size, flags);
End;

Function av_int_list_length(list: Pointer; item_size: int; term: int64_t): int;Inline;
Begin
   Result := av_int_list_length_for_size(item_size, list, term);
End;

Function av_make_error_string(errbuf: PAnsiChar; errbuf_size: size_t; errnum: int): PAnsiChar;
Begin
   av_strerror(errnum, @errbuf, errbuf_size);
   Result := @errbuf;
End;

Var
   error_str: Array[0..AV_ERROR_MAX_STRING_SIZE - 1] Of AnsiChar;

Function av_err2str(errnum: int): PAnsiChar;
Begin
   FillChar(error_str, SizeOf(error_str), 0);
   av_make_error_string(@error_str, AV_ERROR_MAX_STRING_SIZE, errnum);
   Result := @error_str;
End;

Function av_strnlen(Const s: PAnsiChar; len: size_t): size_t;Inline;
Begin
   Result := 0;
   While s[Result] <> #0 Do
      inc(Result);
End;

Function av_isdigit(c: int): Boolean;Inline;
Begin
   Result := (AnsiChar(c) >= '0') And (AnsiChar(c) <= '9');
End;

Function av_isgraph(c: int): Boolean; Inline;
Begin
   Result := (c > 32) And (c < 127);
End;

Function av_isspace(c1: int): Boolean;Inline;
Var
   c: AnsiChar;
Begin
   c := AnsiChar(c1);
   Result :=
      (c = ' ') Or
      (c = #$0C) Or
      (c = #$0A) Or
      (c = #$0D) Or
      (c = #$09) Or
      (c = #$0B);
End;

Function av_toupper(c1: int): int; Inline;
Var
   c: AnsiChar;
Begin
   c := AnsiChar(c1);
   Result := Ord(c);
   If (c >= 'a') And (c <= 'z') Then
      Result := Result Xor $20;
End;

Function av_tolower(c1: int): int; Inline;
Var
   c: AnsiChar;
Begin
   c := AnsiChar(c1);
   Result := Ord(c);
   If (c >= 'A') And (c <= 'Z') Then
      Result := Result Xor $20;
End;

Function av_isxdigit(c1: int): Boolean;Inline;
Var
   c: AnsiChar;
Begin
   c1 := av_tolower(c1);
   c := AnsiChar(AnsiChar(c1));
   Result := av_isdigit(c1) Or ((c >= 'a') And (c <= 'f'));
End;

Function av_bprint_is_complete(Const buf: pAVBPrint): Boolean;Inline;
Begin
   Result := buf^.len < buf^.size;
End;

Function av_fifo_peek2(Const f: pAVFifoBuffer; offs: int): puint8_t;Inline;
Var
   ptr: puint8_t;
Begin
   ptr := f^.rptr + offs;
   If (ptr >= f^._end) Then
      ptr := f^.buffer + (ptr - f^._end)
   Else If (ptr < f^.buffer) Then
      ptr := f^._end - (f^.buffer - ptr);
   Result := ptr;
End;

Function av_ts_make_string(buf: PAnsiChar; ts: int64_t): PAnsiChar;
Var
   p: AnsiString;
   m: size_t;
Begin

   If (ts = AV_NOPTS_VALUE) Then
      p := 'NOPTS'
   Else
      str(ts, p);
   m := length(p);
   If m > AV_TS_MAX_STRING_SIZE Then
      m := AV_TS_MAX_STRING_SIZE;
   move(p[1], buf^, m);
   Result := buf;
End;

Var
   av_ts_buf: Array[0..AV_TS_MAX_STRING_SIZE] Of AnsiChar;

Function av_ts2str(ts: int64_t): PAnsiChar;
Begin
   FillChar(av_ts_buf, SizeOf(av_ts_buf), 0);
   Result := av_ts_make_string(@av_ts_buf[0], ts);
End;

Function av_ts_make_time_string(buf: PAnsiChar; ts: int64_t; tb: pAVRational): PAnsiChar;
Var
   p: AnsiString;
   m: size_t;
Begin

   If (ts = AV_NOPTS_VALUE) Then
      p := 'NOPTS'
   Else
      str((av_q2d(tb^) * ts): 1: 6, p);
   m := length(p);
   If m > AV_TS_MAX_STRING_SIZE Then
      m := AV_TS_MAX_STRING_SIZE;
   move(p[1], buf^, m);
   Result := buf;
End;

Function av_ts2timestr(ts: int64_t; tb: pAVRational): PAnsiChar;
Begin
   FillChar(av_ts_buf, SizeOf(av_ts_buf), 0);
   Result := av_ts_make_time_string(@av_ts_buf[0], ts, tb);
End;

Function av_parse_ratio_quiet(q: pAVRational; Const str: PAnsiChar; max: int): int;Inline;
Begin
   Result := av_parse_ratio(q, str, max, AV_LOG_MAX_OFFSET, Nil);
End;

Function av_int2float(i: uint32_t): float;Inline;
Begin
   Result := av_intfloat32(i).f;
End;

Function av_float2int(f: float): uint32_t; Inline;
Begin
   Result := av_intfloat32(f).i;
End;

Function av_int2double(i: uint64_t): double; Inline;
Begin
   Result := av_intfloat64(i).f;
End;

Function av_double2int(f: double): uint64_t;Inline;
Begin
   Result := av_intfloat64(f).i;
End;

Function av_lfg_get(c: pAVLFG): uint; Inline;
Begin

   Result := c^.state[(c^.index - 24) And 63] + c^.state[(c^.index - 55) And 63];
   c^.state[c^.index And 63] := Result;

   c^.index := c^.index + 1;

End;

Function av_mlfg_get(c: pAVLFG): uint;Inline;
Var
   a, b: uint;
Begin

   a := c^.state[(c^.index - 55) And 63];

   b := c^.state[(c^.index - 24) And 63];

   Result := 2 * a * b + a + b;
   c^.state[c^.index And 63] := Result;

End;


End.
